# destripe -- interactively remove striping from a coadded image
#
# 960405: created by Pat Hall from phip$checkfringe.cl
# 960805: fixed error of default objmask value not being !OBJMASK as claimed
#
# **edit image list & display option?
# **want a mkskycor that is not line-by-line... **images/mfilters package? test

procedure destripe(fiximage,outname,objmask)

string fiximage {prompt="Name of image to be corrected"}
string outname  {prompt="Name of image to create"}
string objmask  {prompt="Object mask to use (default=!OBJMASK)"}

int    ngrow    {1,min=0,max=10,prompt="Number of rings to grow around bad pixels",mode="q"}
int  boxmin  {5,prompt="MKSKYCOR: Minimum smoothing box size at edges",mode="q"}
int  boxmax  {5,prompt="MKSKYCOR: Maximum smoothing box size",mode="q"}
bool ok=no   {prompt="Continue to next step (yes) or redo (no)?",mode="q"}
bool rot=no  {prompt="Redo using rotated image?",mode="q"}
int  z       {0,prompt="Number of image to save (0 to delete all)?",mode="q"}
bool echog   {yes,prompt="Beep when task is done?"}

begin

string logimex, logsurf, countfile, rrootname, iimagelist, ooutname, ffunc
string wweighting, fiximg, ffringe, titlestring, fixin, oobjmask, objminv
string fixroot, outroot, extn
real max, nncols, nnlines, sscale, rorig
int x, y, zz, xxord, yyord, xx, nngrow, bboxmin, bboxmax, r, s
bool go, ffit, xxterms, ook, first, rrot

extn=".fits"

fixin=fiximage
	fiximg="_z."//fixin
	fileroot(fixin,validim+)
	fixroot="_z."//fileroot.root
	if(fileroot.extension!="")extn="."//fileroot.extension
ooutname=outname
	fileroot(ooutname,validim+)
	outroot=fileroot.root
oobjmask=objmask
	fileroot(oobjmask,validim+)
	if(fileroot.root==""){
		imgets(fixin,"OBJMASK")
		fileroot(imgets.value,validim+)
		oobjmask=fileroot.root
	}
	objminv=fileroot.root//"inv_z.pl"
	if(access(objminv))objminv=mktemp("objminv")
print("Enter initial values of adjustable parameters:")
nngrow=ngrow
bboxmin=boxmin
bboxmax=boxmax
	zz=0

# check for necessary stuff; if not defined abort task.
go=yes
if(!deftask("addring")) {go=no}
if(!deftask("fileroot")) {go=no}
if(!deftask("minv")) {go=no}
if(!defpac("nmisc")) {go=no}
if(!defpac("ccdred")) {go=no}
if(go) {

print("Beginning...")

imdel(fiximg,ver-,>&"dev$null")
imdel("_.tmp"//fiximg,ver-,>&"dev$null")
imdel("_.tmp"//objminv,ver-,>&"dev$null")

# ** make optional	 #	if (!access(fiximg) 
# iterstat to get mode
iterstat(fixin,nsigrej=5.,maxiter=10.,print-,verbose-)
# imar to subtract mode
imar(fixin,"-",iterstat.valmode,fiximg,divzero=0.,ver-,noact-)

# OBJECT MASKING STEP
ook=no
first=yes
while(!ook){
	print("Masking objects on coadded image...")
# if not first time through, get params
	if(!first){
		nngrow=ngrow
		imdel("_.tmp"//fiximg,ver-)
		imdel(objminv,ver-)
	}

# addring and/or minv
	if(nngrow!=0){
		addring(oobjmask,"_.tmp"//objminv,ngrow=nngrow,badval=0)
		minv("_.tmp"//objminv,objminv)
		imdel("_.tmp"//objminv,ver-,>&"dev$null")
        } else {
		minv(oobjmask,objminv)
	}

# nmisc$fixpix
	imcopy(fiximg,"_.tmp"//fiximg,ver-)
	fixpix("_.tmp"//fiximg,objminv,ninterp="1")

# acceptable?
	if(first)display(fixin, 1)
	display("_.tmp"//fiximg, 2)
	beep
	ook=ok
	first=no
}
imdel(objminv,ver-)
imdel(fiximg,ver-)
imcopy("_.tmp"//fiximg,fiximg,ver-)


# SKY SMOOTHING & SUBTRACTION STEP
ccdproc.fixpix=no
ccdproc.overscan=no
ccdproc.trim=no
ccdproc.zerocor=no
ccdproc.darkcor=no
ccdproc.flatcor=no
ccdproc.illumcor=no
ccdproc.fringecor=no
ccdproc.readcor=no
ccdproc.scancor=no
x=1
y=2
r=0
ook=no
rrot=no
# loop as long as desired
while(!ook) {
	print("Making stripe image and subtracting from original image...")
# get new params if redoing things
	if(x!=1){
		bboxmin=boxmin
		bboxmax=boxmax
# If rotating, copy original image to temporary image, rotate by 90 degrees
# r number of times, transposing first if rotating 360 degrees or more,
# and not doing anything if more than 720 degrees of rotation are requested.
	    if(rot){
		rrot=yes
		s=1
		r=r+1
		rorig=r
		imdel(fiximg,ver-)
		imcopy("_.tmp"//fiximg,fiximg,ver-)
		if(r>=8){
print("WARNING! All possible permutations of rotation and transposition")
print("have been done, so pick one as the best and stick with it.")
                } else {
			if(r>=4){
				print("Transposing image...")
				imtranspose(fiximg,fiximg)
				r=r-4
                	}
			while(s<=r){
				print("Rotating image by "//r*90//" degrees...")
				imtranspose(fiximg//"[*,-*]",fiximg)
				s+=1
			}
		}
	    }
	}


# make the stripe correction image
        if(access(fixroot//"_"//str(x)//".imh"))imdel(fixroot//"_"//str(x),ver-)
        if(access(fixroot//"_"//str(x)//".hhh"))imdel(fixroot//"_"//str(x),ver-)
	mkskycor(fiximg,fixroot//"_"//str(x),ccdtype="",xboxmin=bboxmin,yboxmin=bboxmin,xboxmax=bboxmax,yboxmax=bboxmax,clip+,lowsig=2.5,highsig=2.5,>&"dev$null")

# if rotating
	if(rrot){
		s=1
		if(rorig<8){
			print("Setting images back to original orientation...")
                        if(rorig>=4){
                          imtranspose(fixroot//"_"//str(x),fixroot//"_"//str(x))
				imtranspose(fiximg,fiximg)
                                rorig=rorig-4
                        }
                        while(s<=rorig){
                imtranspose(fixroot//"_"//str(x)//"[-*,*]",fixroot//"_"//str(x))
				imtranspose(fiximg//"[-*,*]",fiximg)
                                s+=1
                        }
                }
	}

# imar orig - mkskycor
        if(access(outroot//"_"//str(x)//".imh"))imdel(outroot//"_"//str(x),ver-)
        if(access(outroot//"_"//str(x)//".hhh"))imdel(outroot//"_"//str(x),ver-)
	imar(fixin,"-",fixroot//"_"//str(x),outroot//"_"//str(x),divzero=0.,ver-,noact-)

# inspect the subtracted image 
	print ("===> Displaying striped and de-striped images #"//str(x)//"...")
	display(fixroot//"_"//str(x), 3)
	display(outroot//"_"//str(x), 4)

# abort loop?
	beep
	ook=ok
	rrot=no
	x+=1
}


# CLEAN UP
zz=z
if(zz!=0){
  imcopy(fixroot//"_"//str(zz),outroot//"stripe",ver-) 
  imcopy(outroot//"_"//str(zz),ooutname,ver-) 
print("Image "//ooutname//" created using stripe model "//outroot//"stripe")
  hedit(ooutname,"STRIPEIM",outroot//"stripe",add+,ver-,show-,update+)
  hedit(outroot//"stripe","title","Stripe image used to create "//ooutname,add+,ver-,update+,show-) 
}
else{
  print("No images saved.")
}
y=0
zz=0
while(y<x){
	imdel(fixroot//"_"//str(y),ver-, >& "dev$null")
	imdel(ooutname//"_"//str(y),ver-, >& "dev$null")
	y+=1
}
imdel("_.tmp"//fiximg,ver-,>&"dev$null")
imdel(fiximg,ver-)

}
else{
print("Tasks/packages `nmisc', `ccdred', `minv', `addring', or `fileroot' aren't loaded")
print("and this task needs them to run.  Please load them (nmisc last) and start again.")
}

if(echog)beep

end
