# domeflat.cl -- flatfield IR images with a previously created flatfield image
#
# Calls script addcomment, fileroot.
#
# The products of this routine are:
# --flattened images (optionally dark subtracted and bad pixel zeroed): 
#	*//Flatstring
# --a file (imagelist//Flatstring) which contains the names of those images
# --the logfile
#
# CHANGES:
# 940815: made sure task copied images so it could work when darksub=no 
# 960228: added DARKSUB and FLATIMG header cards, other minor changes
# 960301: fixed BPM header card error, added logfile & header cards 
# 	  and streamlined the task to more closely match irflat.cl
# 990127: removed scan of output of "time" task due to problem w/linux
# 990306: don't subtract dark if already done
# 990317: added tpl option
# 990320: fixed bug in darksub option
# 990506: added ZEROOUT keyword creation

procedure domeflat(imagelist,Flatstring,darksub,dark,flatfield,zeroout,badmask,logfile)

string imagelist   {prompt="List of images to process"}
string Flatstring  {prompt="String to be appended to flattened images"}
bool   darksub     {prompt="Subtract Dark frame?"}
string dark        {prompt="Name of the Dark frame to use"}
string flatfield   {prompt="Name of the flatfield frame to use"}
bool   zeroout=yes {prompt="Zero out bad pixels on flattened images?"}
string badmask     {prompt="Name of the bad pixel mask (good pixels=1)"}
string logfile     {prompt="Name of the log file"}

bool   tpl=no      {prompt="Throughput-correct images after flatfielding?"}
string tplname     {prompt="FULL Name of throughput correction image"}
bool   echog       {yes,prompt="Beep when task is done?"}

struct *list

begin

string rrootname, ddark, llogfile, bbadmask, eextension, ffextension
int sstartno, eendno, i, j, numpix
string flatlist, flat, flatbase, meanfile, currentflat, fflatfield, ttplname
string iimagelist, FlatImageList, fflatstr, base, check, flatjoinlist
string datefile, thedate, ickinfo, name, ext, predsimage, postdsimage
real mean, meanbad
bool ddarksub, zzeroout, go, ttpl
struct datelist

iimagelist = imagelist
list = iimagelist
fflatstr = Flatstring
ddarksub = darksub
ddark = dark
fflatfield = flatfield
	if (fflatfield=="") {
	    go=no
	    print ("WARNING! Flatfield image must be defined... task aborted.")
	}
zzeroout = zeroout
bbadmask = badmask
llogfile = logfile
ttpl = tpl
	if (ttpl) ttplname=tplname

# The "go" parameter won't let the task run if you haven't loaded needed tasks.
go=yes
if(!deftask("addcomment")) {go=no}
if(go) {

#set up FlatImageList
FlatImageList = iimagelist//fflatstr  # @file for flattened images
flatjoinlist = "listjoin."//iimagelist//fflatstr  # @file for making the flat
if(access(FlatImageList))
  delete(FlatImageList,ver-)
while (fscan (list, name) !=EOF) {
  fileroot(name,validim+)
  base=fileroot.root
  ext=fileroot.extension
  if(ext!="")ext="."//ext
  print(base//fflatstr//ext, >> FlatImageList)
}

#Set up log file, appending if it already exists
print(" ")
print("Let the games begin...")
if(llogfile == "")
  llogfile="log."//iimagelist
print("", >> llogfile)
print("===> PHIIRS.DOMEFLAT:", >> llogfile)
print("===> Input images from "//iimagelist//" flattened with "//fflatfield, >> llogfile)
print("===> with darksub="//ddarksub//" using dark image="//ddark, >> llogfile)
print("===> and zeroout="//zzeroout//" using bad pixel mask="//bbadmask, >> llogfile)
print("", >> llogfile)

# Add the BPM to the image header
hedit("@"//iimagelist,"BPM",bbadmask,add+,ver-,show+,update+)
hedit("@"//iimagelist,"BPMASK",bbadmask,add+,ver-,show+,update+)

# Subtract off Dark Current
if (ddarksub) {
	print("===> Subtract off Dark Current")
	print("===> Subtract off Dark Current", >> llogfile)
	join (iimagelist, FlatImageList, out=flatjoinlist)
	list = flatjoinlist
	while (fscan (list, predsimage, postdsimage) !=EOF) {
		imgets(predsimage,"darksub",>&"dev$null")
		if(imgets.value=="0"){
			imarith (predsimage,"-",ddark,postdsimage,ver-,noact-)
			hedit(postdsimage,"DARKSUB",ddark,add+,ver-,sho+,updat+)
		  print(predsimage//" - "//ddark//" "//postdsimage, >> llogfile)
		} else {
		  	imcopy (predsimage, postdsimage, ver-)
		  	print (predsimage//" already darksubtracted; just copied to "//postdsimage, >> llogfile)
		  	print (predsimage//" already darksubtracted; just copied to "//postdsimage)
		}
	}
	del(flatjoinlist,ver-)
} else {
	print("===> Preparing...")
	imcopy("@"//iimagelist, "@"//FlatImageList, verbose-)
}

# Divide All Images by the Flat Field
print("===> Correct All Images for Flat Field")
imarith("@"//FlatImageList, "/", fflatfield, "@"//FlatImageList, verbose-)
hedit("@"//FlatImageList,"FLATIMG",fflatfield,add+,ver-,show+,update+)

# Zero out Bad Pixels if desired
if (zzeroout) {
	print("===> Zero out bad pixels")
	imarith("@"//FlatImageList, "*", bbadmask, "@"//FlatImageList, verbose-)
	hedit("@"//FlatImageList,"ZEROOUT",bbadmask,del-,add+,ver-,sho-,upd+)
}

# Do throughput correction if desired.
if (ttpl) {
	print("===> Divide by Throughput Correction")
	print("===> Divide by Throughput Correction", >> llogfile)
	list = FlatImageList
	while (fscan (list, name) !=EOF) {
		imgets(name,"tplimg",>&"dev$null")
		if(imgets.value=="0"){
			imar(name, "/", ttplname, name, ver-)
			hedit(name,fields="TPL",value="TPL correction made using image"//ttplname,add+,ver-,up+,sho-)
		   hedit(name,fields="TPLIMG",value=ttplname,add+,ver-,up+,sho-)
		  	print(name//" / "//ttplname//" "//name, >> llogfile)
		} else {
		  	print(name//" already throughput-corrected using "//imgets.value, >> llogfile)
	       print(name//" already throughput-corrected using "//imgets.value)
		}
	}
}

# Add comment cards to header of flattened image
#time | scan (datelist)
del ("_gooDaTe", ver-, >& "dev$null")
!date > _gooDaTe
type ("_gooDaTe") | scan (datelist)
del ("_gooDaTe", ver-, >& "dev$null")
addcomment ("@"//FlatImageList,"phiirs.domeflat: "//datelist,ver-)
addcomment ("@"//FlatImageList,"Flattened w/"//fflatfield//", darksub="//ddarksub//", zeroout="//zzeroout,ver-)

# Update log file
print("", >> llogfile)
print("===> Flatfielded images listed in "//FlatImageList, >> llogfile)
print("===> PHIIRS.DOMEFLAT finished at "//datelist, >> llogfile)

print("...it is accomplished.  Flatfielded images listed in "//FlatImageList//". ")
print("Next task to run is `irsky'.")
}

else {
        print("WARNING: the `addcomment' task is not loaded!") 
        print("Please load it and restart this task.")
        }

if(echog)beep

end

