# irflat.cl -- flatfield IR images with a running flat
#
# Calls scripts addcomment, fileroot, iterstat.
#
# The products of this routine are:
# --flatfielded images: *//flatstring
# --a file (imagelist//flatstring) which contains the names of those images
# --the logfile
# --optionally: flatfield images called *//"flatfield"//flatstring.imh
#
# CHANGES:
# 950117: created from irskynew.cl
# 950121: added "flatsave" parameter and use of iterstat to determine scaling
# 950122: FINAL VERSION REPLACES OLD IRFLAT.CL 
#           -- MAKE ONLY COSMETIC CHANGES HEREAFTER!
# 950208: begin adding object-masking ability
# 950209: finalized object-masking ability & added header & logfile comments
# 950223: fixed problem with objmask/bpmask confusion
# 960226: explicit request for logfile name
# 971026: changed to allow gain & rdnoise taken from image headers
# 990113: now prints range of images used in flattening to the screen
# 990114: removed scan of output of "time" task due to problem w/linux
# 990317: added tpl option
# 990420: added premask and forcescale options
# 990513: fixed premask / forcescale bug & streamlined masking procedure
# 990521: fixed bug of using "img" where "name" was required,
#	  and of not always deleting "maskmask"
# 000528: really fixed bug of not always deleting "maskmask" this time, I think
#
# **does irflat assume goodval=1 & require minv/imcalc?

procedure irflat(imagelist,flatstring,number,objmask,logfile)

string imagelist  {prompt="List of images to process (do not use an `@')"}
string flatstring {"ff",prompt="String to be appended to flattened images"}
int    number  {8,min=2,prompt="TOTAL # of images for each flat (EVEN #s only)"}
bool   objmask {no,prompt="Use object masking when constructing flats?"}
string logfile=""      {prompt="Name of log file"}

bool   premask=yes     {prompt="Mask when finding scaling for creating flats?"}
bool   forcescale=no   {prompt="Force recalculation of image scalings?"}
bool   stdpars=yes     {prompt="Use std. imcombine params to create flats?"}
bool   flatsave=no     {prompt="Save individual flatfield images?"}
bool   same=yes        {prompt="Do object masks all have same suffix?",mode="q"}
string suffix          {prompt="Suffix of object mask",mode="q"}
bool   tpl=no          {prompt="Throughput-correct images after flatfielding?"}
string tplname         {prompt="FULL Name of throughput correction image"}
bool   v210=no         {prompt="Older version of IRAF w/imcombine bug?"}
bool   echog  	       {yes,prompt="Beep when task is done?"}
pset   imflatpars      {prompt="Imcombine parameters"}
struct *list
struct *list2

begin

string rrootname, ddark, llogfile, bbadmask, eextension, img, ssuffix, oosuffix
string flatlist, flat, flatbase, meanfile, currentflat, base, omask
string iimagelist, FlatImageList, fflatstr, check, ttplname, templist, flatdiv
string datefile, thedate, ickinfo, name, tempstring, otherstring, tempfile
string s_sigma, s_plfile, s_combine, s_reject, s_offsets, s_masktyp, bpmask
string s_scale, s_weight, s_statsec, s_expname, s_zero, s_snoise, ext
string s_rdnoise, s_gain, maskmask
real s_maskval, s_blank, s_lthresh, s_hthresh, s_nlow, s_nhigh, s_nkeep
real s_lsigma, s_hsigma, s_pclip, s_grow, s_sigscale
real mean, meanbad, nnumber, flatnorm, valmed
int sstartno, eendno, i, j, numpix, x, y, z, nnnumber, nf
bool zzeroout, ttpl, sstdpars, go, fflatsave, oobjmask, ssame, mmasksave
bool force_scale, pre_mask
struct datelist

iimagelist = imagelist
fflatstr = flatstring
nnnumber = number
nnumber = nnnumber / 2
oobjmask = objmask
llogfile = logfile

pre_mask = premask
force_scale = forcescale
sstdpars = stdpars
fflatsave = flatsave
ttpl = tpl
	if (ttpl) ttplname=tplname

# Imcombine parameters
s_plfile = i_plfile
s_sigma = i_sigma
s_combine = i_combine
s_reject = i_reject
s_masktyp = i_masktyp
s_maskval = i_maskval
s_blank = i_blank
s_scale = i_scale
s_zero = i_zero
s_weight = i_weight
s_statsec = i_statsec
s_expname = i_expname
s_lthresh = i_lthresh
s_hthresh = i_hthresh
s_nlow = i_nlow
s_nhigh = i_nhigh
s_nkeep = i_nkeep
s_lsigma = i_lsigma
s_hsigma = i_hsigma
s_rdnoise = i_rdnoise
s_gain = i_gain
s_snoise = i_snoise
s_sigscale = i_sigscale
s_pclip = i_pclip
s_grow = i_grow

# The "go" parameter won't let the task run if you haven't loaded the needed
# tasks or if you're using IRAF v2.10 and haven't loaded the ccdred package.
go=yes
if(!deftask("addcomment")) {go=no}
if(!deftask("fileroot")) {go=no}
if(!deftask("iterstat")) {go=no}
if(!defpac("ccdred") && v210) {go=no}
if(go) {

# Miscellaneous
FlatImageList = iimagelist//fflatstr  # @file for flat-divided images
if(access(FlatImageList))delete(FlatImageList,ver-)
count(iimagelist) | scan (nf)
if(nnnumber>=nf){
	nnnumber=nf - 1
	nnumber=nnnumber / 2
  	print("Number of frames to use for flat is too big!  Resetting to "//nnnumber//".")
}

#Set up log file, appending if it already exists
print(" "); print("Let the games begin...")
if(llogfile == "") llogfile="log."//iimagelist
print("", >> llogfile); print("===> PHIIRS.IRFLAT:", >> llogfile)
print("===> Input images from "//iimagelist//" flattened with "//nnnumber//" images total", >> llogfile)
print("===> with stdpars="//sstdpars//" and object masking="//oobjmask, >> llogfile)
print("", >> llogfile)

# Calculate scalings for all frames.  If we are using object masking, first
# get the name of the object mask from the image header keyword OBJMASK.
# If no OBJMASK keyword, find the object mask name and create OBJMASK keyword.
# Insert OBJMASK into BPM keyword temporarily, for later use in imcombine.
print("===> Calculating scalings and checking for object masks if desired...")
list=iimagelist
x=0
while (fscan(list,name) !=EOF) {
# If (and only if) BPMASK keyword not already in header, put it there
	imgets(name,"BPMASK",>&"dev$null")
        if(imgets.value=="0"){
               	imgets(name,"BPM",>&"dev$null")
               	bpmask=imgets.value
               	hedit(name,"BPMASK",bpmask,add+,del-,ver-,show-,upd+)
	} else {
               	bpmask=imgets.value
        }
# If doing object masking, get mask name and insert in image header if needed.
	if(oobjmask){
		fileroot(name,validim+)
		name=fileroot.root
# Check if image has object mask already listed in image header
		imgets(name,"OBJMASK",>&"dev$null")
		omask=imgets.value
# If not, check to see if same suffix is to be used for all images' objmasks.
		if(omask=="0" && x==0){
		   print("No object mask listed in header of image "//name//".")
			ssame=same
			if(ssame)ssuffix=suffix
        if(ssame)print("===> Inserting object mask names into image headers...")
			x+=1
		}	
# Insert the object mask name in the header if necessary
		if(omask=="0"){
	    		if(ssame){
			  omask=name//ssuffix//".pl"
			  hedit(name,"OBJMASK",omask,add+,del-,ver-,show-,update+)
			} else {
		   print("No object mask listed in header of image "//name//".")
			  omask=name//suffix//".pl"
			  hedit(name,"OBJMASK",omask,add+,del-,ver-,show-,update+)
	        	}
		}
# Create mask to use in premasking
                maskmask=mktemp("maskmask")
                imarith(omask,"*",bpmask,maskmask,divzero=0.,pixtype="real",calctype="real",ver-,noact-)
# Put name of mask to use in premasking in header
		hedit (name,"BPM",maskmask,add+,del-,ver-,show-,update+)
		hedit (name,"MASKMASK",maskmask,add+,del-,ver-,show-,update+)
	} else {
                maskmask=mktemp("maskmask")
                imcopy(bpmask,maskmask,ver-)
		hedit (name,"MASKMASK",maskmask,add+,del-,ver-,show-,update+)
        }
# Calculate median of image.
        if (!forcescale) {
                imgets (name,"FLATVAL",>&"dev$null")
                if (imgets.value == "0") force_scale = yes
        }
        if (force_scale) {
                if(pre_mask) {
                	iterstat.nsigrej=5.
                	iterstat.maxiter=10
       maskstat(name,maskmask,1.,lower=INDEF,upper=INDEF,iterstat+,ver-,printit-)
		} else {
	iterstat(name,nsigrej=5.,maxiter=10,lower=INDEF,upper=INDEF,print-,ver-)
		}
		valmed = 1./iterstat.median
		hedit (name,"FLATVAL",valmed,add+,del-,ver-,show-,update+)
	}
}

# Cycle through list of images, making flat for each
list = iimagelist
while (fscan (list, name) !=EOF) {
# Create names for flat image and flat-divided image
	fileroot(name, validim+)
  	base=fileroot.root
  	ext=fileroot.extension
  	if(ext!="")ext="."//ext
  	flat=base//"flatfield"//fflatstr//ext
  	flatdiv=base//fflatstr//ext
# Extract images needed to make flat for each individual image
# This next bit is better done as a call to "xlist", but it works so hey.
	templist = mktemp("templist")
	grep ("-n", name, iimagelist, >> templist)
	list2 = templist
	while (fscan (list2, tempstring) !=EOF) {
		otherstring=":"
        	x=stridx(otherstring, tempstring) - 1
        	otherstring=substr(tempstring,1,x)
        }
	del(templist, ver-)
	print(otherstring, >> templist)
	type(templist) | average >& "dev$null"
	del(templist, ver-)
	y=average.sum
	count(iimagelist, >> templist)
	list2=templist
	while (fscan (list2, x, z, tempstring, otherstring) !=EOF) {
		#if(y>=x-3)y=x-3
		if(y>=x-nnumber)y=x-nnumber
	}
	del(templist, ver-)
	if(y<=nnumber+1)y=nnumber+1
	x=y-nnumber
	z=y+nnumber-1
	grep ("-v", name, iimagelist, >> templist)
	tempfile = mktemp("tempfile")
	fields (templist, 1, lines=x//"-"//z, >> tempfile)
	del(templist,ver-)

# Use this list of images to make the flat
	print("Flatfielding image "//name//" --> "//flatdiv//" w/other images from "//x//"-"//z+1)
	if(sstdpars){
			if(v210){
combine("@"//tempfile, flat, sigma=s_sigma, delete-, clobber-, plfile=s_plfile, combine="median", reject="ccdclip", outtype="real", project-, offsets="none", masktype="none", maskvalue=s_maskval, blank=0., scale="!FLATVAL", zero="none", weight="none", statsec="", lthresh=-100000, hthresh=100000, nlow=1, nhigh=1, nkeep=1, mclip+, lsigma=3., hsigma=3., rdnoise=s_rdnoise, gain=s_gain, snoise="0.", sigscale=0.1, pclip=-0.5, grow=0.)
			} else {
				if(oobjmask){
imcombine("@"//tempfile, flat, sigma=s_sigma, plfile=s_plfile, combine="median", reject="ccdclip", project-, offsets="none", masktype="badvalue", maskvalue=0., blank=0., scale="!FLATVAL", zero="none", weight="none", statsec="", expname=s_expname, lthresh=-100000, hthresh=100000, nlow=1, nhigh=1, nkeep=1, mclip+, lsigma=3, hsigma=3, rdnoise=s_rdnoise, gain=s_gain, snoise="0.", sigscale=0.1, pclip=-0.5, grow=0., logfile='STDOUT', >> llogfile)
				} else {
imcombine("@"//tempfile, flat, sigma=s_sigma, plfile=s_plfile, combine="median", reject="ccdclip", project-, offsets="none", masktype="none", maskvalue=s_maskval, blank=0., scale="!FLATVAL", zero="none", weight="none", statsec="", expname=s_expname, lthresh=-100000, hthresh=100000, nlow=1, nhigh=1, nkeep=1, mclip+, lsigma=3, hsigma=3, rdnoise=s_rdnoise, gain=s_gain, snoise="0.", sigscale=0.1, pclip=-0.5, grow=0., logfile='STDOUT', >> llogfile)
				}
			}
		} else {
		if(v210){
combine("@"//tempfile, flat, sigma=s_sigma, delete-, clobber-, plfile=s_plfile, combine=s_combine, reject=s_reject, outtype="real", project-, offsets="none", masktype="none", maskvalue=s_maskval, blank=s_blank, scale=s_scale, zero=s_zero, weight=s_weight, statsec=s_statsec, lthresh=s_lthresh, hthresh=s_hthresh, nlow=s_nlow, nhigh=s_nhigh, nkeep=s_nkeep, mclip+, lsigma=s_lsigma, hsigma=s_hsigma, rdnoise=s_rdnoise, gain=s_gain, snoise=s_snoise, sigscale=s_sigscale, pclip=s_pclip, grow=s_grow)
		} else {	
			if(oobjmask){
imcombine("@"//tempfile, flat, sigma=s_sigma, plfile=s_plfile, combine=s_combine, reject=s_reject, project-, offsets="none", masktype="badvalue", maskvalue=0., blank=s_blank, scale=s_scale, zero=s_zero, weight=s_weight, statsec=s_statsec, expname=s_expname, lthresh=s_lthresh, hthresh=s_hthresh, nlow=s_nlow, nhigh=s_nhigh, nkeep=s_nkeep, mclip+, lsigma=s_lsigma, hsigma=s_hsigma, rdnoise=s_rdnoise, gain=s_gain, snoise=s_snoise, sigscale=s_sigscale, pclip=s_pclip, grow=s_grow, logfile='STDOUT', >> llogfile)
			} else {	
imcombine("@"//tempfile, flat, sigma=s_sigma, plfile=s_plfile, combine=s_combine, reject=s_reject, project-, offsets="none", masktype=s_masktyp, maskvalue=s_maskval, blank=s_blank, scale=s_scale, zero=s_zero, weight=s_weight, statsec=s_statsec, expname=s_expname, lthresh=s_lthresh, hthresh=s_hthresh, nlow=s_nlow, nhigh=s_nhigh, nkeep=s_nkeep, mclip+, lsigma=s_lsigma, hsigma=s_hsigma, rdnoise=s_rdnoise, gain=s_gain, snoise=s_snoise, sigscale=s_sigscale, pclip=s_pclip, grow=s_grow, logfile='STDOUT', >> llogfile)
			}
		}
	}

	del(tempfile,ver-)
# Normalize the flat by its iterative mean
	if(pre_mask){
		iterstat.nsigrej=5.
                iterstat.maxiter=10
        maskstat(flat,bpmask,1.,lower=INDEF,upper=INDEF,iterstat+,ver-,printit-)
	} else {
	iterstat(flat,nsigrej=5.,maxiter=10,lower=0.001,upper=INDEF,print-,ver-)
	}
	flatnorm = iterstat.mean
	imar(flat, "/", flatnorm, flat, verbose-)

# Divide the input image by the Flat 
	imarith(name, "/", flat, flatdiv, verbose-)

# Add comment cards to header of flattened image
	del ("_1oogDaTe", ver-, >& "dev$null")
	!date > _1oogDaTe
	type ("_1oogDaTe") | scan (datelist)
	del ("_1oogDaTe", ver-, >& "dev$null")
	addcomment (flatdiv,"phiirs.irflat: "//datelist,ver-)
	addcomment (flatdiv,"Flattened w/"//nnnumber//" images, stdpars="//sstdpars//", objmask="//oobjmask,ver-)

# Added flattened image name to output image list and delete flat if desired.
	print(flatdiv, >> FlatImageList)
	if(fflatsave) {
# Insert title into flat image header
		hedit (flat, "title", "Flatfield for image "//name//" normalized by "//flatnorm, add+,del-,ver-,show-,update+)
		print(flat, >> iimagelist//"flatfield"//fflatstr)
	} else {
		imdel(flat, ver-)
	}
}

# Replace BPM keyword with bad pixel mask if object masking was done,
# and delete maskmask in any case
list=iimagelist
while (fscan (list, name) !=EOF) {
	fileroot(name, validim+)
	imgets(name,"MASKMASK",>&"dev$null")
	maskmask=imgets.value
	imdel(maskmask,ver-,>&"dev$null")
	hedit(name,"MASKMASK","",del+,add-,ver-,show-,update+)
	if(oobjmask){ 
  		flatdiv=fileroot.root//fflatstr//ext
		imgets(name,"BPMASK",>&"dev$null")
		bpmask=imgets.value
		hedit(name,"BPM",bpmask,add+,del-,ver-,show-,update+)
		hedit(flatdiv,"BPM",bpmask,add+,del-,ver-,show-,update+)
	}
}

# Do throughput correction if desired.
if (ttpl) {
	print("===> Divide by Throughput Correction")
	print("===> Divide by Throughput Correction", >> llogfile)
	list = FlatImageList
	while (fscan (list, name) !=EOF) {
		imgets(name,"tplimg",>&"dev$null")
		if(imgets.value=="0"){
			imar(name, "/", ttplname, name, ver-)
			hedit(name,fields="TPL",value="TPL correction made using image"//ttplname,add+,del-,ver-,up+,sho-)
		   hedit(name,fields="TPLIMG",value=ttplname,add+,del-,ver-,up+,sho-)
		  	print(name//" / "//ttplname//" "//name, >> llogfile)
		} else {
		  	print(name//" already throughput-corrected using "//imgets.value, >> llogfile)
	       print(name//" already throughput-corrected using "//imgets.value)
		}
	}
}

print("", >> llogfile)
print("===> Flatfielded images listed in "//FlatImageList, >> llogfile)
if(fflatsave)print("===> Flatfield images listed in "//iimagelist//"flatfield"//fflatstr, >> llogfile)
del ("_2oogDaTe", ver-, >& "dev$null")
!date > _2oogDaTe
type ("_2oogDaTe") | scan (datelist)
del ("_2oogDaTe", ver-, >& "dev$null")
print("===> PHIIRS.IRFLAT finished at "//datelist, >> llogfile)

print("It is accomplished.  Flatfielded images listed in "//FlatImageList//".")
if(fflatsave)print("Flatfield images listed in "//iimagelist//"flatfield"//fflatstr//".")
print("Next task to run is `irsky'.")
} else {
	print("WARNING: package noao.imred.ccdred and/or other necessary")
	print("tasks are not loaded!  Please load them and restart this task.")
}

if(echog)beep

end

