# irsky.cl -- sky-subtract images using running median sky
#
# Calls scripts addcomment, fileroot, iterstat, xlist.
# Also calls minv and maskstat if object masking is used.
#
# The products of this routine are:
# --skysubbed images: *//skystring
# --a file (imagelist//skystring) which contains the names of those images
# --the logfile
# --optionally: sky images called *//"skyimage"//skystring.imh
#
# CHANGES:
# 900924: dimsum.xslm originally created by Peter Eisenhardt
# 921207: Added Boolean parameter premask-- determines if scaling of images
# by their medians excludes masked regions.  For cases where variation due to 
# objects exceeds the residual flatfield variation in the input images, this 
# parameter should be yes.  For dim objects and noticeable residual flatfield 
# problems it should be no. Even with premask-, imcombine will mask out pixels.
# 950121: created from dimsum.xslm, replaces old irsky.cl
# 950123: adjusted default imcombine parameters away from dimsum default
# 950209: begin adjusting object-masking capability
# 950522: adjusted tpl subroutine slightly
# 960226: explicit request for logfile name
# 960313: incorporate holes masks into individual-image bad pixel masks
# 960404: fixed minor bugs of bpm for input images being altered
#         and wrong output names for bp+holes masks
# 960413: changed masking to mask w/objmask+bpmask or bpmask, depending,
#         and premask to work even if objmask=no
# 970426: added addition of TPLIMG header card if tpl=yes
# 971026: changed to allow gain & rdnoise taken from image headers
# 990114: removed scan of UNIX 'time' task due to problem w/LINUX
# 990217: added shortcut option for scaling & subtracting sky image
# 990307: added addsky option
# 990320: added subpattern option
# 990406: changed ITERMEAN to ITERMED in scaling of sky image before subtraction
# 990419: iter/maskstat addheader+ explicitly if shortcut=yes
# 990513: speeded up holesimg creation through logical use of imcalc
# 990523: fixed bug of not always deleting "maskmask"

procedure irsky(imagelist,skystring,number,objmask,logfile)

string	imagelist  {prompt="List of images to sky subtract"}
string 	skystring  {"ss",prompt="Suffix for sky subtracted output images"}
int	number     {8,min=2,prompt="EVEN Number of images to use in sky frame"}
bool	objmask	   {no,prompt="Use object masking when constructing skies?"}
string  logfile="" {prompt="Name of log file"}

bool	stdpars=yes  {prompt="Use standard imcombine params to create sky?"}
bool 	sky_del	   {yes,prompt="Delete sky images?"}
bool    same=yes   {prompt="Use same suffix for all object masks?",mode="q"}
string  suffix     {prompt="Suffix of object mask",mode="q"}
bool subpattern=no {prompt="Subtract pattern noise (before tpl)?"}
bool    tpl=no     {prompt="Do throughput correction on image after skysub?"}
string  tplname    {prompt="FULL Name of throughput correction image"}
bool	addsky   {no, prompt="Add constant median sky level after subtraction?"}
bool	forcescale {no, prompt="Force recalculation of image medians?"}
bool 	premask	   {no, prompt="Mask when scaling before creating sky?"}
bool 	csky_del   {yes, prompt="Delete csky images?"}
bool 	myway	   {no, prompt="I want *all* imcombine parameters my way!"}
bool 	shortcut   {yes, prompt="Use quick way to scale sky for subtraction?"}
pset    imskypars  {prompt="Imcombine parameters"}
bool    echog      {yes, prompt="Beep when task is done?"}
struct	*list
struct	*imglist

begin

	string	imlist		# equals imagelist
	int	nnumber		# equals number
	int	nnnumber	# equals number
	bool	oobjmask	# equals objmask
	bool	sstdpars	# equals stdpars
	string 	img		# fscanned name of individual input image
	string 	ext		# extension of individual input image
	string	subimg		# sky subtracted output image
	string	skyimg		# sky image for image "img"
	string	holesimg 	# mask identifying "holes" in output image
	int	nsmin		# equals nskymin most of the time
	int	nf		# number frames in imagelist
	int	ip		# number of current image to sky subtract
	int	start, finish	# first & last image #'s in list to use for sky
	real	valmed	 	# median value
	int	maxcsky		# minimum pixels used
	int 	nskyframes	# number of sky frames used
	int 	nholes		# number of pixels found to be in "holes"
	struct	theadline	# temporary struct for header information
	string	templist	# temporary image list file
	string	sskystring	# suffix to append to sky-subtracted images
	string	ssuffix	 	# suffix of object mask images
	string	maskmask, maskfile, filemask 	# various temporary masks
	bool	scalecalc	# calculate image scalings
	bool	ttpl	 	# do throughput correction?
	bool	go		# necessary tasks loaded?
	string	ttplname	# throughput correction image name
	string  omask		# object mask
	string  bpmask		# bad pixel mask
	string  bp_mask		# bad pixel mask

string s_sigma, s_plfile, s_combine, s_reject, s_offsets, s_masktyp
string s_scale, s_weight, s_statsec, s_expname, s_zero, llogfile, rrootname
string s_rdnoise, s_gain, subpat_list
real s_maskval, s_blank, s_lthresh, s_hthresh, s_nlow, s_nhigh, s_nkeep
real s_lsigma, s_hsigma, s_pclip, s_grow, s_sigscale
real x, valsky
bool ssame, short_cut, add_sky, sub_pattern

# Get query parameters and initialize
#	rrootname = rootname
	imlist = imagelist
	sskystring = skystring
	nnnumber = number
		nnumber = nnnumber / 2
	oobjmask = objmask
	llogfile = logfile

	sstdpars = stdpars
	sub_pattern = subpattern
	ttpl = tpl
		if (ttpl) ttplname=tplname
	add_sky = addsky
	short_cut = shortcut

# Imcombine parameters
s_sigma = i_sigma
s_plfile = i_plfile
s_combine = i_combine
s_reject = i_reject
s_masktyp = i_masktyp
s_maskval = i_maskval
s_blank = i_blank
s_scale = i_scale
s_zero = i_zero
s_weight = i_weight
s_statsec = i_statsec
s_expname = i_expname
s_lthresh = i_lthresh
s_hthresh = i_hthresh
s_nlow = i_nlow
s_nhigh = i_nhigh
s_nkeep = i_nkeep
s_lsigma = i_lsigma
s_hsigma = i_hsigma
s_rdnoise = i_rdnoise
s_gain = i_gain
s_sigscale = i_sigscale
s_pclip = i_pclip
s_grow = i_grow

# The "go" parameter won't let the task run if the needed tasks aren't loaded.
go=yes
if(!deftask("addcomment")) {go=no}
if(!deftask("fileroot")) {go=no}
if(!deftask("iterstat")) {go=no}
if(!deftask("xlist")) {go=no}
if(!defpac("imgtools")) {go=no}
#if(short_cut && !defpac("imgtools")) {go=no}
if(oobjmask && !deftask("minv")) {go=no}
if(oobjmask && !deftask("maskstat")) {go=no}
if(go) {
# Miscellaneous
del(imlist//sskystring, ver-, >& "dev$null")
count(imlist) | scan (nf)
if (nnnumber >= nf) {
  nnnumber = nf - 1
  nnumber = nnnumber / 2
  print("Number of images to use for sky is too big!  Resetting to "//nnnumber//".")
}

# Set up log file, appending if it already exists
if(llogfile == "")llogfile="log."//imlist
print("", >> llogfile)
print("===> PHIIRS.IRSKY:", >> llogfile)
print("===> Input images from "//imlist//" skysubbed with "//nnnumber//" images total", >> llogfile)
print("===> with stdpars="//sstdpars//", object masking="//oobjmask//", addsky="//add_sky, >> llogfile)
print("", >> llogfile)

# Calculate scalings for all frames.  If we are using object masking, first 
# get the name of the object mask from the image header keyword OBJMASK.
# If no OBJMASK keyword, insert object mask name into both it and BPM 
# keywords for later use in imcombine.
print("===> Calculating scalings and checking for object masks if desired...")
imglist = imlist
x=0
while (fscan (imglist, img) !=EOF) {
        imgets(img,"BPM",>&"dev$null")
        bpmask=imgets.value
	if(oobjmask){
        	fileroot(img,validim+)
        	img=fileroot.root
# Check if image has object mask already listed in image header
        	imgets(img,"OBJMASK",>&"dev$null")
        	omask=imgets.value
# If not, check to see if same suffix is to be used for all images' objmasks.
        	if(omask=="0" && x==0){
                  print("No object mask listed in header of image `"//img//"'.")
                	ssame=same
                	if(ssame)ssuffix=suffix
	if(ssame)print("===> Inserting object mask names into image headers...")
                	x+=1
                }
# Insert the object mask name in the header
        	if(omask=="0"){
                    if(ssame){
                	omask=img//ssuffix//".pl"
                	hedit(img,"OBJMASK",omask,add+,ver-,show-,update+)
                    } else {
                    print("No object mask listed in header of image "//img//".")
                        omask=img//suffix//".pl"
                        hedit(img,"OBJMASK",omask,add+,ver-,show-,update+)
                    }
                }
        	hedit (img,"BPM",omask,add+,ver-,show-,update+)
# If (and only if) BPMASK keyword not already in header, put it there
        	imgets(img,"BPMASK",>&"dev$null")
          if(imgets.value=="0")hedit(img,"BPMASK",bpmask,add+,ver-,show-,updat+)
# Create file to use in premasking
		maskmask=mktemp("maskmask")
        	imarith(omask,"*",bpmask,maskmask,divzero=0.,pixtype="real",calctype="real",ver-,noact-)
        } else {
		maskmask=mktemp("maskmask")
        	imcopy(bpmask,maskmask,ver-)
        }
	flpr; flpr; flpr

# Calculate the median of the image.  If premask=yes, use only unmasked pixels.
# Record the median value into the image header with the card SKYMED.
# If SKYMED card already exists in the header, do not recalculate unless
# the parameter forcescale=yes.
	scalecalc = forcescale
	if (!forcescale) {
		imgets (img,"SKYMED",>&"dev$null")
		if (imgets.value == "0") scalecalc = yes
	}
	if (scalecalc) {
	    if(premask) {
	        iterstat.nsigrej=5.
	        iterstat.maxiter=10
       maskstat(img,maskmask,1.,lower=INDEF,upper=INDEF,iterstat+,ver-,printit-)
#      		imdel (maskmask,ver-)
	    } else {
         iterstat(img,nsigrej=5.,maxiter=10,lower=INDEF,upper=INDEF,print-,ver-)
#      		imdel (maskmask,ver-)
	    }
	   	valmed = 1./iterstat.median
	   	hedit (img,"SKYMED",valmed,add+,ver-,show-,update+)
	}
      	imdel (maskmask,ver-)
}

flpr; flpr; flpr

# Begin main loop
imglist = imlist
ip=0
while (fscan(imglist,img) != EOF) {
	ip+=1
	if (ip-nnumber < 1) {
		start  = 1
		finish = 1+nnnumber
	   } else if (ip+nnumber > nf) {
		start  = nf-nnnumber
		finish = nf
	   } else {
		start  = ip - nnumber
		finish = ip + nnumber
	}

# Construct lists of input images and masks for this sky image
	nskyframes = finish - start
	templist = mktemp("tmp$slm")
	xlist (imlist,templist,start,finish,ip,prefix="")

# Construct sky. If we are using object masking, the parameter imcombine.plfile 
# is used to create a .pl file called csky_//prefix//img//.pl which, for each 
# pixel in the sky frame, counts the number of images excluded from the final 
# average by either pixel masking or rejection.  This will later be used to 
# create the "holes" image.  If there are any "holes" (i.e. regions where no 
# images contributed to the final sky), they are set to the value of the 
# parameter "imcombine.blank" (0 for stdpars) in the final image.  

	flpr; flpr; flpr

# set up sky-subtracted image name and output list
	fileroot (img,validim+)
	ext = fileroot.extension
	if(ext!="")ext="."//ext
	subimg = fileroot.root//sskystring
	skyimg = fileroot.root//"skyimage"//sskystring//ext
	print(subimg//ext, >> imlist//sskystring)
	print ("Sky subtracting ",fileroot.root," --> ",subimg,": frame ",ip,"w/sky frames "//start//"-"//finish)

	if (myway) {
    print("WARNING! Unless certain parameters are set correctly in imskypars,")
    print("doing things `your way' could cause problems later on with PHIIRS.")
imcombine("@"//templist, skyimg, sigma=s_sigma, plfile=s_plfile, combine=s_combine,reject=s_reject, project-, offsets="none", masktype=s_masktyp, maskvalue=s_maskval, blank=s_blank, scale=s_scale, zero=s_zero, weight=s_weight, statsec=s_statsec, expname=s_expname, lthresh=s_lthresh, hthresh=s_hthresh, nlow=s_nlow, nhigh=s_nhigh, nkeep=s_nkeep, mclip+, lsigma=s_lsigma, hsigma=s_hsigma, rdnoise=s_rdnoise, gain=s_gain, sigscale=s_sigscale, pclip=s_pclip, grow=s_grow, logfile='STDOUT', >> llogfile)
	} else {
		if (oobjmask) {
			if(sstdpars){
imcombine("@"//templist,skyimg,sigma="",plfile="csky_"//subimg//".pl",combine="average",reject="ccdclip",project-,offsets="none",masktype="badvalue",maskvalue=0.,blank=0.,scale="!SKYMED",zero="none",weight="none",statsec=s_statsec,expname=s_expname,lthresh=INDEF,hthresh=INDEF,nlow=1,nhigh=1,nkeep=1,mclip+,lsigma=3,hsigma=3,rdnoise=s_rdnoise,gain=s_gain,sigscale=s_sigscale,pclip=s_pclip,grow=s_grow,logfile='STDOUT', >> llogfile)
			} else {
imcombine("@"//templist,skyimg,sigma=s_sigma,plfile="csky_"//subimg//".pl",combine=s_combine,reject=s_reject,project-,offsets="none",masktype="badvalue",maskvalue=0.,blank=s_blank,scale=s_scale,zero=s_zero,weight=s_weight,statsec=s_statsec,expname=s_expname,lthresh=s_lthresh,hthresh=s_hthresh,nlow=s_nlow,nhigh=s_nhigh,nkeep=s_nkeep,mclip+,lsigma=s_lsigma,hsigma=s_hsigma,rdnoise=s_rdnoise,gain=s_gain,sigscale=s_sigscale,pclip=s_pclip,grow=s_grow,logfile='STDOUT', >> llogfile)
			}
		} else {
			if(sstdpars){
imcombine("@"//templist,skyimg,sigma="",plfile="",combine="average",reject="ccdclip",project-,offsets="none",masktype="none",maskvalue=s_maskval,blank=0.,scale="!SKYMED",zero="none",weight="none",statsec=s_statsec,expname=s_expname,lthresh=INDEF,hthresh=INDEF,nlow=1,nhigh=1,nkeep=1,mclip+,lsigma=3,hsigma=3,rdnoise=s_rdnoise,gain=s_gain,sigscale=s_sigscale,pclip=s_pclip,grow=s_grow,logfile='STDOUT', >> llogfile)
			} else {
imcombine("@"//templist,skyimg,sigma=s_sigma,plfile=s_plfile,combine=s_combine,reject=s_reject,project-,offsets="none",masktype=s_masktyp,maskvalue=s_maskval,blank=s_blank,scale=s_scale,zero=s_zero,weight=s_weight,statsec=s_statsec,expname=s_expname,lthresh=s_lthresh,hthresh=s_hthresh,nlow=s_nlow,nhigh=s_nhigh,nkeep=s_nkeep,mclip+,lsigma=s_lsigma,hsigma=s_hsigma,rdnoise=s_rdnoise,gain=s_gain,sigscale=s_sigscale,pclip=s_pclip,grow=s_grow,logfile='STDOUT', >> llogfile)
			}
		}
	}
	flpr; flpr; flpr

# Divide image by sky frame to produce temporary ratio image _sclmsk//img.
	imdelete ("_sclmsk"//img,ver-,>&"dev$null")
	imarith(img,"/",skyimg,"_sclmsk"//img,divzero=0.,
		pixtype="real",calctype="real",ver-,noact-)
	flpr; flpr; flpr

# Original slower way of scaling and subtracting sky image
	if (!short_cut) {
# Calculate median of unmasked (sky) pixels in _sclmsk and save in variable 
# 'valmed'.  Delete _sclmsk image afterward.
		if (oobjmask) {
			imgets (img,"OBJMASK")
			maskfile = imgets.value
			imgets (img,"BPMASK")
			filemask = imgets.value
			maskmask=mktemp("maskmask")
        		imarith(filemask,"*",maskfile,maskmask,divzero=0.,pixtype="real",calctype="real",ver-,noact-)
			iterstat.nsigrej=5
			iterstat.maxiter=10
			maskstat("_sclmsk"//img,maskmask,1.,lower=0.,upper=INDEF,iterstat+,ver-,printit-,>>llogfile)
			imdel(maskmask,ver-)
		} else {
			iterstat.nsigrej=5
			iterstat.maxiter=10
#iterstat("_sclmsk"//img,nsigrej=5.,maxit=10,lower=INDEF,upper=INDEF,pri-,ver-)
			maskstat("_sclmsk"//img,"!BPMASK",1.,lower=INDEF,upper=INDEF,iterstat+,ver-,print-)
		}
		valmed = iterstat.median
		imdelete ("_sclmsk"//img,ver-)
		flpr; flpr; flpr
# Rescale sky to object frame median 
		imar(skyimg,"*",valmed,skyimg,pix="real",calc="real",ve-)
# Subtract pattern noise, and add sky back in if desired
		if (sub_pattern) {
		    imar(img,"-",skyimg,subimg,pixt="real",calct="real",ver-)
		    subpat_list = mktemp("subpat")
		    print (subimg, >> subpat_list)
		    if (add_sky) {
			imdel ("addsky_pattern.fits",ver-,>& "dev$null")
			if (!access("blank_pattern.fits")) imar (subimg, "-", subimg, "blank_pattern.fits", pix="real", calc="real", ver-)
			imar ("blank_pattern.fits", "-", valmed, "addsky_pattern.fits", pix="real", calc="real", ver-)
			subpattern (subpat_list, "", startline=1, finishline=512, blankname="addsky_pattern.fits", ver-, useimgs+, saveimgs-)
			imdel ("addsky_pattern.fits",ver-,>& "dev$null")
		    } else {
			subpattern (subpat_list, "", startline=1, finishline=512, blankname="blank_pattern.fits", ver-, useimgs+, saveimgs-)
		    }
		    del (subpat_list, ver-)
		} else {
# Don't subtract pattern noise, but subtract sky & poss. add sky level back in.
		    if (add_sky) {
                       imar(skyimg,"-",valmed,skyimg,pix="real",calc="real",ve-)
                    }
                    flpr; flpr; flpr
		    imar(img,"-",skyimg,subimg,pixt="real",calct="real",ve-)
		    flpr; flpr; flpr
		}
# New, speedier shortcut method to scale & subtract sky image
	} else {
		imdelete ("_sclmsk"//img,ver-)
		if (premask) {
                	maskstat(skyimg,"!BPMASK",1.,nsigrej=5.,maxiter=10,lower=INDEF,upper=INDEF,iterstat+,addh+,ver-,print+, >> llogfile)
                	valsky = iterstat.median
        	} else {
                	iterstat(skyimg,nsigrej=5.,maxiter=10,lower=INDEF,upper=INDEF,print+,addh+,ver-, >> llogfile)
                	valsky = iterstat.median
        	}
# Get sky image scaling
		imgets (img,"ITERMED",>&"dev$null")
		valmed=real(imgets.value)/valsky
# Scale & subtract sky image & pattern noise, & add back in sky level if desired
		if (sub_pattern) {
	        imcalc(img//","//skyimg,subimg,"im1-im2*"//valmed,pi="real",ve-)
		    subpat_list = mktemp("subpat")
		    print (subimg, >> subpat_list)
		    if (add_sky) {
                        imdel ("addsky_pattern.fits",ver-,>& "dev$null")
                        if (!access("blank_pattern.fits")) imar (subimg, "-", subimg, "blank_pattern.fits", pix="real", calc="real", ver-)
		        imar ("blank_pattern.fits", "-", valmed, "addsky_pattern.fits", pix="real", calc="real", ver-)
		        subpattern (subpat_list, "", startline=1, finishline=512, blankname="addsky_pattern.fits", ver-, useimgs+, saveimgs-)
		        imdel ("addsky_pattern.fits",ver-,>& "dev$null")
		    } else {
		        subpattern (subpat_list, "", startline=1, finishline=512, blankname="blank_pattern.fits", ver-, useimgs+, saveimgs-)
		    }
		    del (subpat_list, ver-)
		} else {
# Don't subtract pattern noise, but subtract sky & poss. add sky level back in.
		    if (add_sky) {
   imcalc(img//","//skyimg,subimg,"im1-im2*"//valmed//"+"//valmed,pi="real",ve-)
		    } else {
		imcalc(img//","//skyimg,subimg,"im1-im2*"//valmed,pi="real",ve-)
		    }
		}
	}

# If we are using object masking, find "holes" in the sky subtracted frame 
# wherever the sky frame had no unrejected pixels entering into the coaverage.
	if (oobjmask) {
		holesimg = subimg//".holes.pl"
		if (access(holesimg)) delete (holesimg,ve-) 
                imstat("csky_"//subimg//".pl",fo-,fields="max") | scan (maxcsky)
		if (maxcsky == nskyframes) {
              	     imstat ("csky_"//subimg,for-,fields="npix",lower=nskyframes,upper=nskyframes) | scan(nholes)
		     print("There are ",nholes," holes in output image ",subimg)
        print("There are ",nholes," holes in output image ",subimg, >> llogfile)
# 0 to nskyframes-1 are OK --> 1
# nskyframes to INDEF are holes --> 0
		     imcalc("csky_"//subimg//".pl",holesimg,"if im1 .ge. "//nskyframes//" then 0 else 1",ver-)
#		     imcopy("csky_"//subimg//".pl",holesimg,ve-)
#		     imreplace (holesimg,0,lower=0,upper=nskyframes-1)
#		     imreplace (holesimg,1,lower=nskyframes,upper=INDEF)
#		     minv (holesimg,holesimg)
		     imarith (subimg,"*",holesimg,subimg,ver-)
# Multiply holes mask into bad pixel mask for that image
		     imgets(subimg,"BPMASK",>&"dev$null")
                     bpmask=imgets.value
		     fileroot (img,validim+)
          	     if (access(fileroot.root//skystring//bpmask)) imdelete (fileroot.root//skystring//bpmask,ver-)
	      imarith(bpmask,"*",holesimg,fileroot.root//skystring//bpmask,ver-)
# Used to save holes mask & header keyword; now just delete since it's in bpmask
	             hedit(fileroot.root//skystring//bpmask,fields="title",value="Bad Pixel and Sky Holes Mask for "//subimg,add+,ver-,up+,sho-)
		     imdel(holesimg,ver-)	
    		     hedit(subimg,fields="BPMASK",value=fileroot.root//skystring//bpmask,ad+,ve-,up+,sho-)
		}
		if (csky_del) delete ("csky_"//subimg//".pl",ve-)
# End if (oobjmask)
	}

# Do throughput correction if desired.
	if (ttpl) {
		imar(subimg,"/",ttplname,subimg,ver-)
	 	hedit(subimg,fields="TPL",value="TPL correction made using image "//ttplname,add+,ver-,up+,sho-)
	 	hedit(subimg,fields="TPLIMG",value=ttplname,add+,ver-,up+,sho-)
	}

# Add comment cards to header of sky subtracted image.
	del ("_3oogDaTe", ver-, >& "dev$null")
	!date > _3oogDaTe
	type ("_3oogDaTe") | scan (theadline)
	del ("_3oogDaTe", ver-, >& "dev$null")
        addcomment (subimg,"phiirs.irsky: "//theadline,ver-)
	addcomment (subimg,"Skysubbed w/"//nnnumber//" images, stdpars="//sstdpars//", objmask="//oobjmask,ver-)
	
# Clean up.
	if(sky_del){
	  imdelete(skyimg,ver-)
	} else {
	  hedit(skyimg,"title","Sky for "//img//" from frames "//start//"-"//finish//" in "//imlist//" normalized by "//valmed,add+,ver-,show-,update+)
	  print(skyimg, >> imlist//"skyimage"//sskystring)
	}
	delete(templist,ver-)

# End main loop 

}

# Replace BPM keyword with bad pixel mask
if(oobjmask){
	imglist=imlist
	while (fscan (imglist, img) !=EOF) {
	        fileroot(img, validim+)
		ext=fileroot.extension
		if(ext!="")ext="."//ext
		subimg=fileroot.root//sskystring//ext
        	imgets(subimg,"BPMASK",>&"dev$null")
        	bpmask=imgets.value
        	hedit(subimg,"BPM",bpmask,add+,ver-,show-,update+)
        	imgets(img,"MASKMASK",>&"dev$null")
        	imdel(maskmask,ver-,>&"dev$null")
        	hedit(img,"MASKMASK","",del+,add-,ver-,show-,update+)
	}
}

print("===> Sky-subtracted images listed in "//imlist//sskystring, >> llogfile)
if(!sky_del)print("===> Sky-subtracted images listed in "//imlist//"skyimage"//sskystring, >> llogfile)
#time | scan (theadline)
del ("_4oogDaTe", ver-, >& "dev$null")
!date > _4oogDaTe
type ("_4oogDaTe") | scan (theadline)
del ("_4oogDaTe", ver-, >& "dev$null")
print("===> PHIIRS.IRSKY finished at "//theadline, >> llogfile)

print("...all done.  Sky-subtracted images listed in "//imlist//sskystring//".")
if(!sky_del)print("Sky images listed in "//imlist//"skyimage"//sskystring//".")
print("Next task to run is `irshift' (or `irshift2' if necessary).")

# End if(go)

} else {
        print("WARNING: some/all of the necessary tasks are not loaded!")
        print("Please load them and restart this task.")
}

	imglist = ""

if(echog)beep

end


