# photscale -- calculate photometric scalings of a data set.
#
# Originally created by Mark Dickinson.  Modified by Pat Hall.
#
# **requires stsdas (for igi)
#
# 960323: created from photdiff.cl
# 960325: added normalization option
# 960326: truncate output to 159 characters so IRAF can handle it
# 960327: fixed bugs relating to 159-char IRAF limit by using awk not IRAF
# 960411: added option to just replot old results w/o changing or redoing them
# 960716: added option to use aperture corrections if desired

procedure photscale (photfile,prefix,suffix)

string	photfile	{prompt="Photometry data file from photcheck"}
string	prefix		{prompt="Prefix for output file(s)"}
string	suffix		{prompt="Suffix for output file(s)"}

bool apcor   {no, prompt="Use aperture corrections?"}
string apfile {no, prompt="Aperture correction file (coadded image first)"}
bool display {yes, prompt="Automatically display lightcurve (needs STSDAS)?"}
bool results {no, prompt="Output computed scales and weights?", mode="q"}
bool  newref {no, prompt="Select new reference image?", mode="q"}
int refimagenum {min=1,prompt="Number of new reference image", mode="q"}
bool    replace {no, prompt="Replace a point with average of others?", mode="q"}
int     newpoint {min=1,prompt="Image number of point to replace", mode="q"}
string avgpoints {prompt="Points to avg. for replacement (enter as `1,2')", mode="q"}
string   reenter {prompt="Re-enter points (enter as `1,2')", mode="q"}
bool      normal {no, prompt="Normalize scales to a certain image?", mode="q"}
int normimagenum {min=1,prompt="Number of normalization image", mode="q"}
bool   echog        {yes,prompt="Beep when task is done?"}
struct   *outdat

begin

	string	phfile,outp	# tmp. working versions of input/output files
	string	phiirsdir	# directory containing PHIIRS scripts
	string	igicommands, filename, tmpout, tmpfile, ssuffix, s1, s2, s3
	string  aavgpoints, tmpstart, tmpmid, tmpend, pphotfile, s4, s5, tmpawk
	string  aapfile
	real	ut, scale, serr, delta, derr, value1, error1, value2, error2
	real	xmin, xmax, ymin, ymax, valuemag, errormag, error0, value0
	real	xplotmin, xplotmax, yplotmin, yplotmax
	real	normvalue, normserr, normdelta, normderr
	int	x, y, z, rrefimagenum, nnewpoint, nnormimagenum, point1, point2
	bool	rresults, nnewref, rreplace, iterate, nnormal, normed, refereed
	bool    justplot, aapcor

# Get query parameters.
pphotfile = photfile
	phfile = mktemp(pphotfile)
	copy(pphotfile,phfile)
tmpout = prefix
	outp = mktemp(tmpout)
ssuffix = suffix

aapcor = apcor
	if(aapcor)aapfile=apfile

# Check for presence of output files
justplot=no
if(access(tmpout//".photscales"//ssuffix)){
	beep
       print("Output file "//tmpout//".photscales"//ssuffix//" already exists.")
	print ("No new files will be created; only plotting will be done.")
	justplot=yes
}

## Check for presence of filecalc task (in CTIO package)
#if(!deftask("filecalc")){
#print ("Task `filecalc' is not loaded.  Please load and restart task.")
#	return
#}

# Check for presence of the photscale.awk & photscale2.awk scripts in PHIIRS
# source code directory.  If either are not present, return error message.
phiirsdir = osfn("phiirs$")
if ((!access(phiirsdir//"photscale.awk")) || (!access(phiirsdir//"photscale2.awk"))) {
	print ("ERROR: Cannot access "//phiirsdir//"$photscale.awk or photscale2.awk!")
	return
}

# MAIN LOOP
# Set up loop to allow repeated iterations using different reference images
print ("Beginning...")
iterate=yes
nnewref=no
rreplace=no
nnormal=no
normed=no
refereed=no
while(iterate){

# NEW REFERENCE IMAGE
if(nnewref){
# If new reference image, what image number?
	refereed=yes
	rrefimagenum=refimagenum
# Get that reference image's data and insert it as the first line w/awk
	tmpawk=mktemp("tmpawk")
	tmpfile=mktemp("tmpfile")
	print ("NR == "//rrefimagenum+1//", NR == "//rrefimagenum+1//" {print $0}", >> tmpawk)
	awk ("-f",tmpawk,phfile,>> tmpfile)
	del(tmpawk,ver-)
	tmpawk=mktemp("tmpawk")
	print ("NR != 1, NR != 1 {print $0}", >> tmpawk)
	awk ("-f",tmpawk,phfile,>> tmpfile)
	del(tmpawk,ver-)
# Use awk script to calculate photometric scalings between frames.
	if(!aapcor){
		awk ("-f",phiirsdir//"photscale.awk",tmpfile,>"_tmp."//outp)
# If aperture corrections...
	} else {
		join(aapfile, tmpfile, maxch=9999, ver-, >> tmpfile//"2")
	      awk ("-f",phiirsdir//"photscale2.awk",tmpfile//"2",>"_tmp."//outp)
		del(tmpfile//"2", ver-)
	}
# Truncate output to 159 characters so IRAF can handle it.  Lines are formatted
# such that no mag,err pair should be cut off in the middle via this truncation
	tmpawk=mktemp("tmpawk")
	print ("{print (substr($0,1,159))}", >> tmpawk)
	awk ("-f",tmpawk,"_tmp."//outp,> outp)
	del("_tmp."//outp,ver-)
	del(tmpawk,ver-)
	del(phfile,ver-)
	rename(tmpfile,phfile)
} else {
# If first time through, calculate photometric scalings between frames.
	if(!rreplace && !nnormal){
		del(outp,ver-,>&"dev$null")
		if(!aapcor){
		     awk ("-f",phiirsdir//"photscale.awk",phfile,>"_tmp."//outp)
# If aperture corrections...
		} else {
			join(aapfile, phfile, maxch=9999, ver-, >> phfile//"2")
	       awk ("-f",phiirsdir//"photscale2.awk",phfile//"2",>"_tmp."//outp)
			del(phfile//"2", ver-)
		}
# Truncate output to 159 characters so IRAF can handle it.  Lines are formatted
# such that no mag,err pair should be cut off in the middle via this truncation
      		tmpawk=mktemp("tmpawk")
        	print ("{print (substr($0,1,159))}", >> tmpawk)
        	awk ("-f",tmpawk,"_tmp."//outp,> outp)
        	del("_tmp."//outp,ver-)
        	del(tmpawk,ver-)
	}
}

# If display==yes, check to see if stsdas.graphics.stplot is loaded, and if 
# not, load it, then call stsdas.graphics.stplot.igi to display light curve.
	if (display) {
		if (!defpac("stsdas")) stsdas
		if (!defpac("graphics")) graphics
		if (!defpac("stplot")) stplot
		xplotmin = 0.
		xplotmax = 1.
		ymin = 1.E20
		ymax = -1.E20
		outdat = outp
		del("_plot."//outp, ver-, >& "dev$null")
# First, scan data values to determine plot limits.  Skip first line.
	if (fscan(outdat,filename) == EOF) error (0,"No lines in file ",outdat)
		while (fscan(outdat,filename,ut,delta,derr,scale,serr) != EOF) {
			xplotmax = xplotmax + 1.
			if(delta!=INDEF && derr!=INDEF){
				ymin = min(ymin,delta-derr)
				ymax = max(ymax,delta+derr)
			} else {
				if(delta!=INDEF) {
					derr=99.999
				} else {
					delta=0.
					derr=99.999
				}
			}
		     print(filename//" ",ut,delta,derr,scale,serr, >> "_plot."//outp)
		}
		yplotmin = ymin - 0.2 * (ymax-ymin)
		yplotmax = ymax + 0.2 * (ymax-ymin)
		
# Now open temporary file and create igi command macro.
		igicommands = mktemp("plotdiff")
	print ("limits ",xplotmin,xplotmax,yplotmin,yplotmax, >igicommands)
		print ("box", >>igicommands)
		print ("xlabel image number", >>igicommands)
		print ("ylabel \gDm: reference - image", >>igicommands)
		print ("title Relative magnitude differences: ", tmpout//".photscales"//ssuffix, >>igicommands)
		print ("ltype 1", >>igicommands)
		print ("relocate ",xplotmin," 0.0 ", >>igicommands)
		print ("draw ",xplotmax," 0.0 ", >>igicommands)
		print ("ltype 0", >>igicommands)
		print ("data _plot."//outp, >>igicommands)
#		print ("data ",outp, >>igicommands)
#		print ("lines 2 256 ", >>igicommands)
		print ("lines 1 256 ", >>igicommands)
		print ("xcolumn 2", >>igicommands)
		print ("xevaluate r", >>igicommands)
		print ("ycolumn 3", >>igicommands)
		print ("ecol 4", >>igicommands)
		print ("ptype 4 0", >>igicommands)
		print ("points", >>igicommands)
		print ("error 2", >>igicommands)
		print ("error 4", >>igicommands)

# Execute igi with command macro, exit, then bring up graphics cursor to
# allow user to snapshot if desired.
		igi (initcmd="input "//igicommands//" ; end", >"dev$null")
		=gcur
		delete (igicommands,ver-)
		delete ("_plot."//outp,ver-,>&"dev$null")
	}

# default: do not iterate, choose new reference image, or replace point(s)
iterate=no
nnewref=no
nnormal=no
rreplace=no

# New reference image?
if(!justplot)nnewref=newref
if(nnewref){
	iterate=yes
} else {
# 
# POINT REPLACEMENT
# Replace a point?
	if(!justplot)rreplace=replace
if(rreplace){
	iterate=yes
# Which point?
	nnewpoint=newpoint
# Replace with average of what two points? (enter as 1,2)
	aavgpoints=avgpoints
#if no , "re-enter (enter as 1,2):"
	while(stridx(",",aavgpoints)==0){
		aavgpoints=reenter
	}
	point1=int(substr(aavgpoints,1,stridx(",",aavgpoints)-1))
      point2=int(substr(aavgpoints,stridx(",",aavgpoints)+1,strlen(aavgpoints)))

# Extract the value of point1 w/awk 
        tmpawk=mktemp("tmpawk")
        tmpfile=mktemp("tmpfile")
        print("NR == "//point1+1//", NR == "//point1+1//" {print $0}",>> tmpawk)
        awk ("-f",tmpawk,outp,>> tmpfile)
        del(tmpawk,ver-)
	outdat=tmpfile
	while(fscan(outdat,filename,ut,delta,derr,scale,serr) != EOF) {
		value1=scale
		error1=serr
	}
        del(tmpfile,ver-,>&"dev$null")

# Extract the value of point2 w/awk
        tmpawk=mktemp("tmpawk")
        tmpfile=mktemp("tmpfile")
        print("NR == "//point2+1//", NR == "//point2+1//" {print $0}",>> tmpawk)
        awk ("-f",tmpawk,outp,>> tmpfile)
        del(tmpawk,ver-)
	outdat=tmpfile
	while(fscan(outdat,filename,ut,delta,derr,scale,serr) != EOF) {
		value2=scale
		error2=serr
	}
        del(tmpfile,ver-,>&"dev$null")

# Calculate the new point
	value0=0.5*(value1+value2)
	error0=sqrt(error1*error1+error2*error2)
	valuemag=-2.5*log10(value0)
	errormag=(2.5*error0)/(log(10.0)*value0)

# Put in file w/awk, leaving out individ. star dm values to denote replacement
        tmpawk=mktemp("tmpawk")
	tmpstart=mktemp("tmpstart")
	tmpmid=mktemp("tmpmid")
	tmpend=mktemp("tmpend")
        tmpfile=mktemp("tmpfile")
print("NR == 1, NR == "//nnewpoint//" {print $0}",>> tmpawk)
        awk ("-f",tmpawk,outp,>> tmpstart)
        del(tmpawk,ver-)
        tmpawk=mktemp("tmpawk")
print("NR >= "//nnewpoint+2//" {print $0}",>> tmpawk)
        awk ("-f",tmpawk,outp,>> tmpend)
        del(tmpawk,ver-)
        tmpawk=mktemp("tmpawk")
print("NR == "//nnewpoint+1//" {print $0}",>> tmpawk)
        awk ("-f",tmpawk,outp,>> tmpfile)
        del(tmpawk,ver-)
	outdat=tmpfile
	while(fscan(outdat,filename,ut,delta,derr,scale,serr) != EOF) {
	      print(filename//" "//ut//" "//valuemag//" "//errormag//" "//value0//" "//error0, >> tmpmid)
	}
	del(outp, ver-)
	concat(tmpstart//","//tmpmid//","//tmpend, outp)
	del(tmpfile,ver-,>&"dev$null")
	del(tmpstart,ver-,>&"dev$null")
	del(tmpmid,ver-,>&"dev$null")
	del(tmpend,ver-,>&"dev$null")

# End of loop for point replacement
} else {

# NORMALIZATION
# Normalize scales?
if(!justplot)nnormal=normal
if(nnormal){
	normed=yes
	iterate=yes
# Normalize with what image?
	nnormimagenum=normimagenum
# Extract the scaling value of nnormimagenum
        tmpawk=mktemp("tmpawk")
        tmpfile=mktemp("tmpfile")
        print("NR == "//nnormimagenum+1//", NR == "//nnormimagenum+1//" {print $0}",>> tmpawk)
        awk ("-f",tmpawk,outp,>> tmpfile)
        del(tmpawk,ver-)
        outdat=tmpfile
        while(fscan(outdat,filename,ut,delta,derr,scale,serr) != EOF) {
                normvalue=scale
                normdelta=delta
        }
        del(tmpfile,ver-,>&"dev$null")

# Normalize all scales and errors by scaling value of nnormimagenum
    if(access(tmpout//".photold"//ssuffix))del(tmpout//".photold"//ssuffix,ver-)
	rename (outp,tmpout//".photold"//ssuffix)
	outdat=tmpout//".photold"//ssuffix
	while(fscan(outdat,s2,s3,delta,derr,scale,serr) != EOF) {
# If delta, scale, and error are all defined, print normalized values of all
		if(delta!=INDEF && scale!=INDEF && serr!=INDEF){
#		s1="one"
#		print(s1,delta,derr,scale,serr)
			print(s2//" "//s3//" "//delta-normdelta//" "//derr//" "//scale/normvalue//" "//serr/normvalue, >> outp)
		} else {
#		s1="two"
#		print(s1,delta,derr,scale,serr)
# Set up strings containing values of delta and derr
			if(delta==INDEF){s4="INDEF"}
			else{s4=str(delta-normdelta)}
			if(derr==INDEF){s5="INDEF"}
			else{s5=str(derr)}
# If only scale error is undefined, print INDEF value for it
			if(scale!=INDEF){
				print(s2//" "//s3//" "//s4//" "//s5//" "//scale/normvalue//" INDEF", >> outp)
# If both scale and scale error are undefined, print INDEF value for both
			} else {
				print(s2//" "//s3//" "//s4//" "//s5//" INDEF INDEF", >> outp)
			}
		}
	}
# End of loop for normalization
}
# End of loop for no point replacement
}
# End of loop for no new reference image
}
# End of loop for a given reference image
outdat=""
}

# Output scales and weights?
if(!justplot){
    rename(outp,tmpout//".photscales"//ssuffix)
    if(refereed){rename(phfile,tmpout//".photfile"//ssuffix)}
    else{del(phfile,ver-)}
    rresults=results
    if(rresults){
	fields(tmpout//".photscales"//ssuffix, 5, lines="2-10000", >> tmpout//".scales"//ssuffix)
#filecalc(tmpout//".photscales"//ssuffix, "1/($6*$6)", lines="2-10000", >> tmpout//".weights"//ssuffix)
	del ("tmpout.*s.ssuffix", ver-, >& "dev$null")
	copy (tmpout//".photscales"//ssuffix, "tmpout.photscales.ssuffix", ver-)
	!/scrs1/pathall/irtfphotscale.p tmpout.photscales.ssuffix > tmpout.weights.ssuffix
	copy ("tmpout.weights.ssuffix", tmpout//".weights"//ssuffix, ver-)
	del ("tmpout.*s.ssuffix", ver-, >& "dev$null")
    }
    print("File "//tmpout//".photscales"//ssuffix//" was created")
    if(rresults){
	print("along with "//tmpout//".scales"//ssuffix//" and "//tmpout//".weights"//ssuffix)
        print("(either or both of which can be used to re-ircoadd the images)")
    }
    if(refereed){
      print("along with "//tmpout//".photfile"//ssuffix//" (which is a copy of")
	print(pphotfile//" except with a new reference image)")
    }
    if(normed){
        print("along with "//tmpout//".photold"//ssuffix//" (which contains pre-normalization")
	print("scales and individual star magnitude differences)")
    }
} else {
    del(outp,ver-,>&"dev$null")	
    del(phfile,ver-,>&"dev$null")	
}

if(echog)beep

end

