# mkobjmask.cl -- script to interactively create object mask
#
# originally created 11 July 1992 by John S. Ward
# 930228: Now excludes zero exposure regions from computation of pixel sigma.
# 930726: Modified by MD to call sigmanorm and use makeobjmask.cl.
# 950208: Renamed mkobjmask. Various minor adjustments.
# 950209: inv parameter: prefer to have masks with objects=0 not 1.
# 960301: display object-masked image in interactive mode
# 960405: removed "inv" param and added interactive iteration mode
# 990224: added areamask bits, changed iterstat to maskstat
# 990225: added subtract option and moved things around a bit
# 990306: fixed bug of display of normalized image even if interact=no
# 990307: prints parameter values during initial makeobjmask step
#
# Calls scripts makeobjmask, sigmanorm, iterstat, sctv

procedure mkobjmask(image,expname)

string	image	 {prompt="Image for making mask (best include extension)"}
string	expname		 {prompt="Exposure map (default=image//em)"}
string  suffix="objmask" {prompt="Suffix of output object mask"}

string areamask=""   {prompt="Exposed area mask (default: create image//mask)"}
bool   subtract=no    {prompt="Subtract average from image before making mask?"}
string substring="sub" {prompt="String to append to subtracted image, if made"}
string prefix="norm" {prompt="Prefix for temporary normalized image"}
int    nsmooth {3,min=0,prompt="Boxcar smoothing filter size (ODD values only)"}
int	subsample {1,min=1,prompt="Subsampling factor before median filtering"}
int	filtsize {15,min=1,prompt="Filter size for local median sky evaluation"}
int	ngrow	{1,min=0,prompt="Width of rings to grow around masked objects"}
bool	interact	{yes,prompt="Interactively examine normalized image?"}
real	threshold	{prompt="Cutoff point for replacement",mode="q"}
bool	redo    	{no,prompt="Redo with different parameters?",mode="q"}
int     z    {0,prompt="Number of image to save (zero to delete all)?",mode="q"}

int     i_nsmooth   {3,min=0,prompt="Boxcar smoothing filter size (ODD values only)",mode="q"}
int     i_subsample {1,min=1,prompt="Subsampling factor before median filtering",mode="q"}
int     i_filtsize  {15,min=1,prompt="Filter size for local median sky evaluation",mode="q"}
int	i_ngrow	    {1,min=0,prompt="Width of rings to grow around masked objects",mode="q"}
bool   echog        {yes,prompt="Beep when task is done?"}

begin

string	img		# equals image
string	ext		# equals image extension
string	eexpname	# equals expname
string	ssuffix		# equals suffix
string	objmask		# name of output mask
string	area_mask	# equals areamask
string	sub_string	# equals substring
string	pprefix		# equals prefix
real	th		# equals threshold
real	recthresh	# recommended thresholding value
real	minval		# minimum value in mask image
real	maxexp		# peak exposure time
real	flagval		# flag value
bool	rredo		# iterate mask determination?
bool	sub_tract	# equals subtract
int x    # number to append to tmp. images; increased by 1 each time
int y    # dummy number; looped through to delete leftover images at end
int zz   # dummy variable for `z'
bool ook # dummy variable set equal to `ok' to keep loop going or not
int nnsmooth, ssubsample, ffiltsize, nngrow	# dummy variables

# Get query parameters
fileroot (image,validim+)
	img = fileroot.root
	ext = fileroot.extension
	if(ext!="")ext="."//ext
eexpname = expname
	if(eexpname=="")eexpname=img//"em"
ssuffix = suffix
	objmask = img//ssuffix

area_mask = areamask
	if(area_mask=="")area_mask=img//"mask.pl"
sub_tract = subtract
sub_string = substring
	if (sub_tract && access(img//sub_string)) {
		print (img//sub_string//" already exists!  Using it...")
		img=img//sub_string
		sub_tract=no
	}
pprefix = prefix
nnsmooth=nsmooth
ssubsample=subsample
ffiltsize=filtsize
nngrow=ngrow

# Create area mask if it doesn't already exist
if (!access(area_mask)) {
	print("Creating "//area_mask//"...")
	imcopy(eexpname,area_mask,ver-)
	imreplace(area_mask,0.,lower=INDEF,upper=0.01)
	imreplace(area_mask,1.,lower=0.01,upper=INDEF)
}

# Subtract masked iterative mean if desired
if (sub_tract) {
	print("Creating "//img//sub_string//"...")
	imdel (img//sub_string, ver-, >& "dev$null")
	maskstat(img,area_mask,1.,nsigrej=5.,maxiter=5.,lower=INDEF,upper=INDEF,ver-,iterstat+,print-,addh+)
	imar(img,"-",iterstat.mean,img//sub_string,calctype="real",ver-,noact-)
	img=img//sub_string
}

# Display initial image
if (interact) {
	print ("Displaying image "//img//"...")
	display(img,1)
}

# Normalize image to uniform pixel-to-pixel rms using sq.root of exposure map.
print ("Normalizing image "//img//" to uniform RMS...")
if(!access(pprefix//img//ext))sigmanorm(img,normimage=eexpname,prefix=pprefix)

# Find thresholding level
print ("Finding thresholding level using maskstat...")
maskstat(pprefix//img,area_mask,1.,nsigrej=5.,maxiter=10.,lower=INDEF,upper=INDEF,ver-,iterstat+,print-,addh+)
recthresh = 4.5 / nsmooth * iterstat.sigma
print ("Recommended thresholding level for mask is ",recthresh)

# Allow user to interactively examine (unsmoothed) normalized image to set 
# threshold for masking.
#if (interact) imexamine(pprefix//img,display="display(image='$1',frame=2,zs+)")
if (interact) display(pprefix//img,frame=2,zs+)

# If interactive, query for final threshold to adopt.
# If not interactive, use the default recommended threshold.
if (interact) {
	beep
	threshold = recthresh
	th = threshold
} else {
	th = recthresh
}

# Apply thresholding to image using makeobjmask.
print ("Creating mask with nsmooth="//nsmooth//", subsample="//subsample//", filtsize="//filtsize//", ngrow="//ngrow)
makeobjmask (pprefix//img,suffix=ssuffix,headlist="",subsample=subsample,filtsize=filtsize,nsmooth=nsmooth,threshtype="constant",constthresh=th,ngrow=ngrow,statsec="",checklimits+,inv+,ver-)
imrename (pprefix//img//ssuffix//".pl",objmask//".pl",ver-)

# Display original image and mask if interact=yes.  
if (interact) {
	sctv(objmask//".pl",3)
	imarith(img,"*",objmask//".pl","del_"//img)
	display("del_"//img,4)
	imdel("del_"//img,ver-)

# Allow iteration if desired.
	x=2
	beep
	rredo=redo
	while(rredo){
  		if(x==2){
		  imren(objmask//".pl",pprefix//img//ssuffix//"_1.pl",ver-)
			if (!access(objmask//"orig.pl")) imcopy (pprefix//img//ssuffix//"_1.pl",objmask//"orig.pl",ver-, >& "dev$null")
		}
	print("Recommended thresholding level for mask is ",recthresh)
		th = threshold
		nnsmooth=i_nsmooth
		ssubsample=i_subsample
		ffiltsize=i_filtsize
		nngrow=i_ngrow
		makeobjmask (pprefix//img,suffix=ssuffix//"_"//str(x),headlist="",subsample=ssubsample,filtsize=ffiltsize,nsmooth=nnsmooth,threshtype="constant",constthresh=th,ngrow=nngrow,statsec="",checklimits+,inv+,ver-)
		sctv(pprefix//img//ssuffix//"_"//str(x)//".pl",3)
	     imar(img,"*",pprefix//img//ssuffix//"_"//str(x)//".pl","del_"//img)
		display("del_"//img,4)
		imdel("del_"//img,ver-)
# Another iteration?
		beep
		rredo=redo
		x+=1
	}
}

# CLEAN UP
zz=1
if(interact)zz=z
if(zz!=0){
	imren(pprefix//img//ssuffix//"_"//str(zz)//".pl",objmask//".pl",ver-, >& "dev$null")
	if(access(objmask//"orig.pl")){
	  print("Object masks "//objmask//".pl & "//objmask//"orig.pl created.")
	} else {
		print("Object mask "//objmask//".pl created.")
	}
}
else{
     print("No images saved.")
}
y=0
zz=0
if(interact){
    while(y<x){
        imdel(pprefix//img//ssuffix//"_"//str(y)//".pl",ver-, >& "dev$null")
        y+=1
    }
}
print("Next task to run is `masklet'.")
imdelete (pprefix//img,ver-)

if(echog)beep

end

