# onis0451 -- shift & coadd ONIS data on MS0451+02

# 990518 -- created from onis3ew.cl
# 990519 -- sigmanorm of long-axis coadds commented out (not used)


# ** could do xcor in a chain: after coaddlongaxes, 
# 	displ AB #1 CD #2 & measure 1-star offset, then 
# 	EHI #3 FGI #4 MLK #1 ON #2 TSP #3 URQ #4 VWX #1
# ** would allow all shifts to be done at once

# --ideally would include multimasklet.cl instead of masklet.cl,
#	to identify & exclude spurious objects from bright star residuals.

# ** interactively ident star resids?  when?  (& make bpms)
# ** ideally want to ID star resids as you go thru the images in order
#    could do this in chainshift 
#    but how to apply marked x,y to final objmask image?
# ** if you have objmask and objmasknospur, want to tell it which single-letter
#	images take which object mask.  
# ** better write masklet to handle this.  But will it make a diff?

#get x,y for a bad pixel w/in area of a spurious object in the mask
#imreplace x,y to good
#get coords (listpix) of all bad pixels in x-1:x+1,y-1:y+1
#	if none, end
#	if some, imreplace to good and repeat with xmin-1:xmax+1,ymin-1:ymax+1
#		 where the xmin:xmax,ymin:ymax spans the range of bad pixels

# ** in pre-review: display xm=0.5 ym=0.5 with estimated offsets?

# layout of MS0451 dither pattern & image pairings (long axis vertical):
#			 BA  EHI   FGJ MLK URQ TSP (CD ON VWX)
#A1    L12 K11 J10       ABC LEMDN KFG JIH OPQ RST
#B2 E5/M13 F6  I9
#C3 D4/N14 G7  H8
#     O15
#     P16
#     Q17
#     R18
#     S19
#     T20

procedure onis0451(inlist,imageroot,niter)

string	inlist		{prompt="List of input images"}
string	imageroot	{prompt="Root name for output files"}
int    	niter=1     	{prompt="Number to append to output files"}

string	pairing="all"	{enum="ABC|LEMDN|KFG|JIH|OPQ|RST|all",prompt="Pair of images to process (or all)"}
bool inspect=yes	{prompt="Inspect image lists before proceeding?"}
bool shiftlongaxes=yes	{prompt="Find shifts for image pairings w/overlapping long axes?"}
bool coaddlongaxes=yes	{prompt="Coadd image pairings w/overlapping long axes?"}
bool nshiftall=yes	{prompt="Find shifts between all image pairings?"}
bool prereview		{prompt="Display all image for pre-nshift check?"}
bool p_tv	     {prompt="Use ptv instead of display for pre-nshift check?"}
bool use_images=yes  {prompt="Measure shifts from images (not file) in nshift?"}
bool display_im=yes	{prompt="Display images for imexam in nshift?"}
bool edit_files=yes     {prompt="Edit coords files after creation in nshift?"}
bool exclude	   {prompt="Exclude any image pairings from first-pass nshift?"}
string exstring	{prompt="Enter code(s) for excluded pairing(s) separated by commas (ABC|LEMDN|KFG|JIH|OPQ|RST)"}
bool redo=no		{prompt="Redo shift-finding if already done?"}
bool coaddallimages=yes	{prompt="Coadd all images?"}
bool secondpassxcor=yes	{prompt="Cross-correlate all images with coadded image?"}
bool coaddsecondpass=yes {prompt="Coadd images after second-pass cross-correlation?"}
bool domkobjmask=yes	{prompt="Make coadded-image object mask (INTERACTIVE)?"}
bool domasklet=yes	{prompt="Make individual-image object masks?"}
string imextn="fits"	{prompt="Image extension"}
int xsize=512		{prompt="X-axis individual image size (0 for autosense)"}
int ysize=1024		{prompt="Y-axis individual image size (0 for autosense)"}
bool echog=yes		{prompt="Beep when task is done?"}

struct *list
 
begin
 
string in_list, image_root, p_air, ref_image, tmpheaders, sigprefix, rroundname
string theory, method_shortaxes, tempfile2, msect, image_name, tempx, tempy
string rvw_image, ex_string, current_string, future_string, in_string, im_extn
string zero_out
int xmin, ymin, xint, yint, old_xint, old_yint, xmax_bcad, xmin_fead, ix, iy
int xmin_rstopq, xmax_rstopq, xrstopq, yrstopq, rstn1
int xmin_vwxopq, xmax_vwxopq, xvwxopq, yvwxopq, vwxn1
int xmin_jihkfg, xmax_jihkfg, xjihkfg, yjihkfg, jihn1
int xmin_ehikfg, xmax_ehikfg, xehikfg, yehikfg, ehin1
int n_iter, xbcad, ybcad, xfead, yfead, bcn1, fen1, x_shift, y_shift, xe, ye
int xref, yref, int_x, int_y, junkint, ilen, ipos
int oxdim, oydim     # dimensions of mask image
int ixdim, iydim     # dimensions of individual input frames
int xlemdnmost, ylemdnmost, xabcba, yabcba, xcdba, ycdba, xlemdnba, ylemdnba, xkfgba, ykfgba
int xjihba, yjihba, xonba, yonba, xrstba, yrstba, xopqba, yopqba, xvwxba, yvwxba
real abcba_xshiftexact, abcba_yshiftexact, lemdnmost_xshiftexact, lemdnmost_yshiftexact
real cdba_xshiftexact, cdba_yshiftexact, lemdnba_xshiftexact, lemdnba_yshiftexact
real kfgba_xshiftexact, kfgba_yshiftexact, jihba_xshiftexact, jihba_yshiftexact 
real onba_xshiftexact, onba_yshiftexact, rstba_xshiftexact, rstba_yshiftexact
real opqba_xshiftexact, opqba_yshiftexact, vwxba_xshiftexact, vwxba_yshiftexact 
real xbcad2, ybcad2, xfead2, yfead2, bcad_x, bcad_y, fead_x, fead_y
real xrstopq2, yrstopq2, rstopq_xlag, rstopq_ylag
real xvwxopq2, yvwxopq2, vwxopq_xlag, vwxopq_ylag
real xopq, yopq, xrst, yrst, xvwx, yvwx
real xjihkfg2, yjihkfg2, jihkfg_xlag, jihkfg_ylag
real xehikfg2, yehikfg2, ehikfg_xlag, ehikfg_ylag
real xkfg, ykfg, xjih, yjih, xehi, yehi
real xad, yad, xbc, ybc, xfe, yfe, bcad_xlag, bcad_ylag, fead_xlag, fead_ylag
real bcad_xfirstpass, bcad_yfirstpass, fead_xfirstpass, fead_yfirstpass
real rstopq_xfirstpass, rstopq_yfirstpass, vwxopq_xfirstpass, vwxopq_yfirstpass 
real jihkfg_xfirstpass, jihkfg_yfirstpass, ehikfg_xfirstpass, ehikfg_yfirstpass 
real xoff, yoff, step, new_x, new_y, old_x, old_y, step_x, step_y, xsig, ysig
bool in_spect, shift_longaxes, echo_g, coadd_longaxes, abort, go, do_masklet
bool coadd_allimages, xcor_secondpass, coadd_secondpass, do_mkobjmask, re_do
bool im_display, images_use, shift_shortaxes, coadd_shortaxes, xreg_secondpass
bool nshift_all, files_edit, ex_clude, pt_v, pre_review

in_list=inlist
image_root=imageroot
n_iter=niter
p_air=pairing

in_spect=inspect
shift_longaxes=shiftlongaxes
images_use=use_images
im_display=display_im
files_edit=edit_files
ex_string=""
re_do=redo
coadd_longaxes=coaddlongaxes
nshift_all=nshiftall
pre_review=prereview
pt_v=p_tv
coadd_allimages=coaddallimages
xcor_secondpass=secondpassxcor
coadd_secondpass=coaddsecondpass
do_mkobjmask=domkobjmask
do_masklet=domasklet
im_extn=imextn
echo_g=echog

rroundname=image_root//".xregshifts.shall"//n_iter
sigprefix=sigmanorm.prefix
abcba_xshiftexact = 0; abcba_yshiftexact = 0; cdba_xshiftexact = 0
cdba_xshiftexact = 0; lemdnba_xshiftexact = 0; lemdnba_yshiftexact = 0
kfgba_xshiftexact = 0; kfgba_yshiftexact = 0; jihba_xshiftexact = 0
jihba_yshiftexact = 0; onba_xshiftexact = 0; onba_yshiftexact = 0
rstba_xshiftexact = 0; rstba_yshiftexact = 0; opqba_xshiftexact = 0
opqba_yshiftexact = 0; vwxba_xshiftexact = 0; vwxba_yshiftexact = 0

go=yes

if(!defpac("imgtools")) {go=no}
if(go) {


# Separate input list into lists of images overlapping along long axes
print ("\n===> Checking image pairing lists...")
tmpheaders=mktemp("tmpheaders")
hsel ("@"//in_list, "$I,i_title", yes, >> tmpheaders)
if (!access(in_list//"_ABC")) {
 grep ('Position\ \ 1\ of\ 20', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_ABC")
 grep ('Position\ \ 2\ of\ 20', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_ABC")
 grep ('Position\ \ 3\ of\ 20', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_ABC")
 grep ('Position\ \ 1\ of\ 20', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_AB")
 grep ('Position\ \ 2\ of\ 20', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_AB")
 grep ('Position\ \ 3\ of\ 20', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_C")
 print (in_list//"_AB", >> "l_"//in_list//"_ABC")
 print (in_list//"_C", >> "l_"//in_list//"_ABC")
} else {
 print (in_list//"_ABC already exists; not created.")
}
if (!access(in_list//"_JIH")) {
 grep ('Position\ 10\ of\ 20', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_JIH")
 grep ('Position\ \ 9\ of\ 20', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_JIH")
 grep ('Position\ \ 8\ of\ 20', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_JIH")
 grep ('Position\ 10\ of\ 20', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_JI")
 grep ('Position\ \ 9\ of\ 20', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_JI")
 grep ('Position\ \ 8\ of\ 20', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_H")
 print (in_list//"_JI", >> "l_"//in_list//"_JIH")
 print (in_list//"_H", >> "l_"//in_list//"_JIH")
} else {
 print (in_list//"_JIH etc. already exist; not created.")
}
if (!access(in_list//"_KFG")) {
 grep ('Position\ 11\ of\ 20', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_KFG")
 grep ('Position\ \ 6\ of\ 20', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_KFG")
 grep ('Position\ \ 7\ of\ 20', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_KFG")
 grep ('Position\ 11\ of\ 20', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_KF")
 grep ('Position\ \ 6\ of\ 20', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_KF")
 grep ('Position\ \ 7\ of\ 20', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_G")
 print (in_list//"_KF", >> "l_"//in_list//"_KFG")
 print (in_list//"_G", >> "l_"//in_list//"_KFG")
} else {
 print (in_list//"_KFG etc. already exist; not created.")
}
if (!access(in_list//"_LEMDN")) {
 grep ('Position\ 12\ of\ 20', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_LEMDN")
 grep ('Position\ \ 5\ of\ 20', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_LEMDN")
 grep ('Position\ 13\ of\ 20', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_LEMDN")
 grep ('Position\ \ 4\ of\ 20', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_LEMDN")
 grep ('Position\ 14\ of\ 20', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_LEMDN")
 grep ('Position\ 12\ of\ 20', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_LEM")
 grep ('Position\ \ 5\ of\ 20', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_LEM")
 grep ('Position\ 13\ of\ 20', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_LEM")
 grep ('Position\ \ 4\ of\ 20', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_DN")
 grep ('Position\ 14\ of\ 20', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_DN")
 print (in_list//"_LEM", >> "l_"//in_list//"_LEMDN")
 print (in_list//"_DN", >> "l_"//in_list//"_LEMDN")
} else {
 print (in_list//"_LEMDN etc. already exist; not created.")
}
if (!access(in_list//"_RST")) {
 grep ('Position\ 18\ of\ 20', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_RST")
 grep ('Position\ 19\ of\ 20', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_RST")
 grep ('Position\ 20\ of\ 20', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_RST")
 grep ('Position\ 18\ of\ 20', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_RS")
 grep ('Position\ 19\ of\ 20', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_RS")
 grep ('Position\ 20\ of\ 20', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_T")
 print (in_list//"_RS", >> "l_"//in_list//"_RST")
 print (in_list//"_T", >> "l_"//in_list//"_RST")
} else {
 print (in_list//"_RST etc. already exist; not created.")
}
if (!access(in_list//"_OPQ")) {
 grep ('Position\ 15\ of\ 20', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_OPQ")
 grep ('Position\ 16\ of\ 20', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_OPQ")
 grep ('Position\ 17\ of\ 20', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_OPQ")
 grep ('Position\ 15\ of\ 20', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_OP")
 grep ('Position\ 16\ of\ 20', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_OP")
 grep ('Position\ 17\ of\ 20', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_Q")
 print (in_list//"_OP", >> "l_"//in_list//"_OPQ")
 print (in_list//"_Q", >> "l_"//in_list//"_OPQ")
} else {
 print (in_list//"_OPQ etc. already exist; not created.")
}
del(tmpheaders,ver-)
if (in_spect) vi (in_list//"_?? "//in_list//"_??? "//in_list//"_LEMDN")


# Do irshift on lists of images overlapping along long axes
if (shift_longaxes) {
print("\n===> Finding shifts for image pairings overlapping along long axes...")
	if (p_air=="ABC" || p_air=="all") {
		fields (in_list//"_ABC", 1, lines="6-6") | scan (ref_image)
		if (!access(image_root//".roundoff.shABC"//n_iter)) {
			print("\n----> ABC: overlap region is the ENTIRE reference image; using nshift <----")
			nshift ("l_"//in_list//"_ABC", in_list//"_ABC", ref_image, image_root, "shABC"//n_iter, useimages=images_use, displayim=im_display, editfiles=files_edit)
		}
	}
	if (p_air=="LEMDN" || p_air=="all") {
		fields (in_list//"_LEMDN", 1, lines="6-6") | scan (ref_image)
		if (!access(image_root//".roundoff.shLEMDN"//n_iter)) {
      			print("\n----> LEMDN:  overlap region is the ENTIRE reference image; using nshift <----")
			nshift ("l_"//in_list//"_LEMDN", in_list//"_LEMDN", ref_image, image_root, "shLEMDN"//n_iter, useimages=images_use, displayim=im_display, editfiles=files_edit)
		}
	}
	if (p_air=="KFG" || p_air=="all") {
		fields (in_list//"_KFG", 1, lines="6-6") | scan (ref_image)
		if (!access(image_root//".roundoff.shKFG"//n_iter)) {
      			print("\n----> KFG: overlap region is the ENTIRE reference image; using nshift <----")
			nshift ("l_"//in_list//"_KFG", in_list//"_KFG", ref_image, image_root, "shKFG"//n_iter, useimages=images_use, displayim=im_display, editfiles=files_edit)
		}
	}
	if (p_air=="JIH" || p_air=="all") {
		fields (in_list//"_JIH", 1, lines="6-6") | scan (ref_image)
		if (!access(image_root//".roundoff.shJIH"//n_iter)) {
      			print("\n----> JIH: overlap region is the ENTIRE reference image; using nshift <----")
			nshift ("l_"//in_list//"_JIH", in_list//"_JIH", ref_image, image_root, "shJIH"//n_iter, useimages=images_use, displayim=im_display, editfiles=files_edit)
		}
	}
	if (p_air=="OPQ" || p_air=="all") {
		fields (in_list//"_OPQ", 1, lines="6-6") | scan (ref_image)
		if (!access(image_root//".roundoff.shOPQ"//n_iter)) {
      			print("\n----> OPQ: overlap region is the ENTIRE reference image; using nshift <----")
			nshift ("l_"//in_list//"_OPQ", in_list//"_OPQ", ref_image, image_root, "shOPQ"//n_iter, useimages=images_use, displayim=im_display, editfiles=files_edit)
		}
	}
	if (p_air=="RST" || p_air=="all") {
		fields (in_list//"_RST", 1, lines="6-6") | scan (ref_image)
		if (!access(image_root//".roundoff.shRST"//n_iter)) {
      			print("\n----> RST: overlap region is the ENTIRE reference image; using nshift <----")
			nshift ("l_"//in_list//"_RST", in_list//"_RST", ref_image, image_root, "shRST"//n_iter, useimages=images_use, displayim=im_display, editfiles=files_edit)
		}
	}
}


# Coadd images overlapping along long axes
if (coadd_longaxes) {
	print ("\n===> Coadding image pairings overlapping along long axes...")
	if (p_air=="ABC" || p_air=="all") {
		imdel (image_root//"ABC"//n_iter, ver-, >& "dev$null")
		imdel (image_root//"ABC"//n_iter//"em", ver-, >& "dev$null")
		del (image_root//"ABC"//n_iter//".imshifts", ver-, >& "dev$null")
		imdel (sigprefix//image_root//"ABC"//n_iter, ver-, >& "dev$null")
		ircoadd (in_list//"_ABC", image_root//"ABC"//n_iter, "", image_root//".roundoff.shABC"//n_iter, "none", "mode", "none", 1, no, "log_"//image_root//"_allatonce", makeavg+, flip-)
#		sigmanorm (image_root//"ABC"//n_iter)
	}
	if (p_air=="LEMDN" || p_air=="all") {
		imdel (image_root//"LEMDN"//n_iter, ver-, >& "dev$null")
		imdel (image_root//"LEMDN"//n_iter//"em", ver-, >& "dev$null")
		del(image_root//"LEMDN"//n_iter//".imshifts", ver-, >& "dev$null")
		imdel(sigprefix//image_root//"LEMDN"//n_iter, ver-, >& "dev$null")
		ircoadd (in_list//"_LEMDN", image_root//"LEMDN"//n_iter, "", image_root//".roundoff.shLEMDN"//n_iter, "none", "mode", "none", 1, no, "log_"//image_root//"_allatonce", makeavg+, flip-)
#		sigmanorm (image_root//"LEMDN"//n_iter)
	}
	if (p_air=="KFG" || p_air=="all") {
		imdel (image_root//"KFG"//n_iter, ver-, >& "dev$null")
		imdel (image_root//"KFG"//n_iter//"em", ver-, >& "dev$null")
		del (image_root//"KFG"//n_iter//".imshifts", ver-, >& "dev$null")
		imdel (sigprefix//image_root//"KFG"//n_iter, ver-, >& "dev$null")
		ircoadd (in_list//"_KFG", image_root//"KFG"//n_iter, "", image_root//".roundoff.shKFG"//n_iter, "none", "mode", "none", 1, no, "log_"//image_root//"_allatonce", makeavg+, flip-)
#		sigmanorm (image_root//"KFG"//n_iter)
	}
	if (p_air=="JIH" || p_air=="all") {
		imdel (image_root//"JIH"//n_iter, ver-, >& "dev$null")
		imdel (image_root//"JIH"//n_iter//"em", ver-, >& "dev$null")
		del (image_root//"JIH"//n_iter//".imshifts", ver-, >& "dev$null")
		imdel (sigprefix//image_root//"JIH"//n_iter, ver-, >& "dev$null")
		ircoadd (in_list//"_JIH", image_root//"JIH"//n_iter, "", image_root//".roundoff.shJIH"//n_iter, "none", "mode", "none", 1, no, "log_"//image_root//"_allatonce", makeavg+, flip-)
#		sigmanorm (image_root//"JIH"//n_iter)
	}
	if (p_air=="OPQ" || p_air=="all") {
		imdel (image_root//"OPQ"//n_iter, ver-, >& "dev$null")
		imdel (image_root//"OPQ"//n_iter//"em", ver-, >& "dev$null")
		del (image_root//"OPQ"//n_iter//".imshifts", ver-, >& "dev$null")
		imdel (sigprefix//image_root//"OPQ"//n_iter, ver-, >& "dev$null")
		ircoadd (in_list//"_OPQ", image_root//"OPQ"//n_iter, "", image_root//".roundoff.shOPQ"//n_iter, "none", "mode", "none", 1, no, "log_"//image_root//"_allatonce", makeavg+, flip-)
#		sigmanorm (image_root//"OPQ"//n_iter)
	}
	if (p_air=="RST" || p_air=="all") {
		imdel (image_root//"RST"//n_iter, ver-, >& "dev$null")
		imdel (image_root//"RST"//n_iter//"em", ver-, >& "dev$null")
		del (image_root//"RST"//n_iter//".imshifts", ver-, >& "dev$null")
		imdel (sigprefix//image_root//"RST"//n_iter, ver-, >& "dev$null")
		ircoadd (in_list//"_RST", image_root//"RST"//n_iter, "", image_root//".roundoff.shRST"//n_iter, "none", "mode", "none", 1, no, "log_"//image_root//"_allatonce", makeavg+, flip-)
#		sigmanorm (image_root//"RST"//n_iter)
	}
}


# Find shifts between all image pairings
if (p_air=="all" && nshift_all) {
    if (!access(image_root//".shiftexact.shall"//n_iter)) {
# ** should test this out
	if (re_do) {
		del (image_root//".imshifts.shall*"//n_iter, ver-, >& "dev$null")
		del (image_root//".offsets.shall*"//n_iter, ver-, >& "dev$null")
		del (image_root//".roundoff.shall*"//n_iter, ver-, >& "dev$null")
		del (image_root//".shiftexact.shall*"//n_iter, ver-, >& "dev$null")
		del (image_root//".shiftfrac.shall*"//n_iter, ver-, >& "dev$null")
		del (image_root//".shiftint.shall*"//n_iter, ver-, >& "dev$null")
		del (image_root//".imshifts.shmost*"//n_iter, ver-, >& "dev$null")
		del (image_root//".offsets.shmost*"//n_iter, ver-, >& "dev$null")
		del (image_root//".roundoff.shmost*"//n_iter, ver-, >& "dev$null")
		del (image_root//".shiftexact.shmost*"//n_iter, ver-, >& "dev$null")
		del (image_root//".shiftfrac.shmost*"//n_iter, ver-, >& "dev$null")
		del (image_root//".shiftint.shmost*"//n_iter, ver-, >& "dev$null")
		print ("\n*allcoords*, *refcoords*, & *coarse* preserved; all else will be recreated.")
	}
	if (!access(in_list//"_coadd_"//n_iter)) {
		print (image_root//"ABC"//n_iter, >> in_list//"_coadd_"//n_iter)
		print (image_root//"LEMDN"//n_iter, >> in_list//"_coadd_"//n_iter)
		print (image_root//"KFG"//n_iter, >> in_list//"_coadd_"//n_iter)
		print (image_root//"JIH"//n_iter, >> in_list//"_coadd_"//n_iter)
		print (image_root//"OPQ"//n_iter, >> in_list//"_coadd_"//n_iter)
		print (image_root//"RST"//n_iter, >> in_list//"_coadd_"//n_iter)
	}
# initial pre-review 
	if (pre_review) {
	  print ("Central image will be displayed in frame 1, and others in 2.")
	  print ("Note any images that do not overlap the central image.")
	  list=in_list//"_coadd_"//n_iter
	  junkint = 1
	  while (fscan (list, rvw_image) !=EOF) {
		if (junkint==1) {
			if (pt_v) {
				ptv (rvw_image, frame=1)
			} else {
				display (rvw_image, frame=1)
			}
		} else {
			if (pt_v) {
				imexam (rvw_image, frame=2, keep-, display="ptv(image='$1',frame=2)")
			} else {
				display (rvw_image, junkint)
				imexam (rvw_image, frame=junkint, keep-, display="display(image='$1',frame=$2)")
			}
		}
		junkint = junkint + 1
		if (junkint==5) junkint = 2
	  }
	}
# exclude any images from nshift at first?
	print ("")
	ex_clude=exclude
# if excluding any images: 
	if (ex_clude) {
# **redo here too? not a global redo, since need to get paramaters below for use in coadding
		del ("_coaddTmp?_", ver-, >& "dev$null")
# get code(s) of excluded image(s)
		ex_string=exstring
# create file of codes of images excluded from most-images coadd
		files (ex_string, >> "_coaddTmp1_")
# create file of images included in most-images coadd
		copy (in_list//"_coadd_"//n_iter, "_coaddTmp2_", ver-)
		!fgrep -v -f _coaddTmp1_ _coaddTmp2_ > _coaddTmp3_
# **redo/!redo instead of delete & recreate?
		del (in_list//"_coaddmost_"//n_iter, ver-, >& "dev$null")
		rename ("_coaddTmp3_", in_list//"_coaddmost_"//n_iter, field="all", >& "dev$null")
# create file of images included in full coadd
# **redo/!redo instead of delete & recreate?
		del (in_list//"_coaddrest_"//n_iter, ver-, >& "dev$null")
		print (image_root//"LEMDN"//n_iter, > "_coaddTmp4_")
		!fgrep -f _coaddTmp1_ _coaddTmp2_ >> _coaddTmp4_
		print (image_root//"most"//n_iter//"_1stpass", >> "_coaddTmp4_")
		type ("_coaddTmp4_", >> in_list//"_coaddrest_"//n_iter)
# find shifts for most-images coadd
# **redo here too?
		if (!access(image_root//".roundoff.shmost"//n_iter)) nshift ("", in_list//"_coaddmost_"//n_iter, image_root//"LEMDN"//n_iter, image_root, "shmost"//n_iter, redo=re_do, useimages=images_use, displayim=im_display, editfiles=files_edit)
    		if (!access(image_root//".imshifts.shmost"//n_iter)) join (in_list//"_coaddmost_"//n_iter, image_root//".shiftexact.shmost"//n_iter, >> image_root//".imshifts.shmost"//n_iter)
    		grep ("ABC", image_root//".imshifts.shmost"//n_iter) | scan (theory, abcba_xshiftexact, abcba_yshiftexact)
  		grep ("LEMDN", image_root//".imshifts.shmost"//n_iter) | scan (theory, lemdnba_xshiftexact, lemdnba_yshiftexact)
  		grep ("KFG", image_root//".imshifts.shmost"//n_iter) | scan (theory, kfgba_xshiftexact, kfgba_yshiftexact)
  		grep ("JIH", image_root//".imshifts.shmost"//n_iter) | scan (theory, jihba_xshiftexact, jihba_yshiftexact)
  		grep ("OPQ", image_root//".imshifts.shmost"//n_iter) | scan (theory, opqba_xshiftexact, opqba_yshiftexact)
  		grep ("RST", image_root//".imshifts.shmost"//n_iter) | scan (theory, rstba_xshiftexact, rstba_yshiftexact)
	} else {
# if not excluding any images: 
		if (!access(image_root//".shiftexact.shall"//n_iter)) nshift ("", in_list//"_coadd_"//n_iter, image_root//"LEMDN"//n_iter, image_root, "shall"//n_iter, redo=re_do, useimages=images_use, displayim=im_display, editfiles=files_edit)
    		fields(image_root//".shiftexact.shall"//n_iter,"1-2",lines="1-1")|scan(abcba_xshiftexact,abcba_yshiftexact)
  		fields(image_root//".shiftexact.shall"//n_iter,"1-2",lines="2-2")|scan(lemdnba_xshiftexact,lemdnba_yshiftexact)
  		fields(image_root//".shiftexact.shall"//n_iter,"1-2",lines="3-3")|scan(kfgba_xshiftexact,kfgba_yshiftexact)
  		fields(image_root//".shiftexact.shall"//n_iter,"1-2",lines="4-4")|scan(jihba_xshiftexact,jihba_yshiftexact)
  		fields(image_root//".shiftexact.shall"//n_iter,"1-2",lines="5-5")|scan(opqba_xshiftexact,opqba_yshiftexact)
  		fields(image_root//".shiftexact.shall"//n_iter,"1-2",lines="6-6")|scan(rstba_xshiftexact,rstba_yshiftexact)
	}
    }
    if(abcba_xshiftexact>=0)xabcba=int(0.5+abcba_xshiftexact)
    if(abcba_yshiftexact>=0)yabcba=int(0.5+abcba_yshiftexact)
    if(abcba_xshiftexact<0)xabcba=int(-0.5+abcba_xshiftexact)
    if(abcba_yshiftexact<0)yabcba=int(-0.5+abcba_yshiftexact)
    if(lemdnba_xshiftexact>=0)xlemdnba=int(0.5+lemdnba_xshiftexact)
    if(lemdnba_yshiftexact>=0)ylemdnba=int(0.5+lemdnba_yshiftexact)
    if(lemdnba_xshiftexact<0)xlemdnba=int(-0.5+lemdnba_xshiftexact)
    if(lemdnba_yshiftexact<0)ylemdnba=int(-0.5+lemdnba_yshiftexact)
    if(kfgba_xshiftexact>=0)xkfgba=int(0.5+kfgba_xshiftexact)
    if(kfgba_yshiftexact>=0)ykfgba=int(0.5+kfgba_yshiftexact)
    if(kfgba_xshiftexact<0)xkfgba=int(-0.5+kfgba_xshiftexact)
    if(kfgba_yshiftexact<0)ykfgba=int(-0.5+kfgba_yshiftexact)
    if(jihba_xshiftexact>=0)xjihba=int(0.5+jihba_xshiftexact)
    if(jihba_yshiftexact>=0)yjihba=int(0.5+jihba_yshiftexact)
    if(jihba_xshiftexact<0)xjihba=int(-0.5+jihba_xshiftexact)
    if(jihba_yshiftexact<0)yjihba=int(-0.5+jihba_yshiftexact)
    if(rstba_xshiftexact>=0)xrstba=int(0.5+rstba_xshiftexact)
    if(rstba_yshiftexact>=0)yrstba=int(0.5+rstba_yshiftexact)
    if(rstba_xshiftexact<0)xrstba=int(-0.5+rstba_xshiftexact)
    if(rstba_yshiftexact<0)yrstba=int(-0.5+rstba_yshiftexact)
    if(opqba_xshiftexact>=0)xopqba=int(0.5+opqba_xshiftexact)
    if(opqba_yshiftexact>=0)yopqba=int(0.5+opqba_yshiftexact)
    if(opqba_xshiftexact<0)xopqba=int(-0.5+opqba_xshiftexact)
    if(opqba_yshiftexact<0)yopqba=int(-0.5+opqba_yshiftexact)
}


# coadd all images at once (w/intermediate most-image coadd if needed)
if (p_air=="all" && coadd_allimages) {

# first set minimum x,y offsets to be 0 & 0 (independently)
	if (!access(image_root//".roundoff.shABC"//n_iter//"min0")) {
		list=image_root//".roundoff.shABC"//n_iter
		xmin=9999; ymin=9999
		while (fscan(list,xint,yint) !=EOF) {
			if (xint < xmin) xmin = xint
			if (yint < ymin) ymin = yint
		}
		filecalc(image_root//".roundoff.shABC"//n_iter, "$1-"//xmin//";$2-"//ymin, format="%4.0f %4.0f", >> image_root//".roundoff.shABC"//n_iter//"min0")
	}
	if (!access(image_root//".roundoff.shLEMDN"//n_iter//"min0")) {
		list=image_root//".roundoff.shLEMDN"//n_iter
		xmin=9999; ymin=9999
		while (fscan(list,xint,yint) !=EOF) {
			if (xint < xmin) xmin = xint
			if (yint < ymin) ymin = yint
		}
		filecalc(image_root//".roundoff.shLEMDN"//n_iter, "$1-"//xmin//";$2-"//ymin, format="%4.0f %4.0f", >> image_root//".roundoff.shLEMDN"//n_iter//"min0")
	}
	if (!access(image_root//".roundoff.shKFG"//n_iter//"min0")) {
		list=image_root//".roundoff.shKFG"//n_iter
		xmin=9999; ymin=9999
		while (fscan(list,xint,yint) !=EOF) {
			if (xint < xmin) xmin = xint
			if (yint < ymin) ymin = yint
		}
		filecalc(image_root//".roundoff.shKFG"//n_iter, "$1-"//xmin//";$2-"//ymin, format="%4.0f %4.0f", >> image_root//".roundoff.shKFG"//n_iter//"min0")
	}
	if (!access(image_root//".roundoff.shJIH"//n_iter//"min0")) {
		list=image_root//".roundoff.shJIH"//n_iter
		xmin=9999; ymin=9999
		while (fscan(list,xint,yint) !=EOF) {
			if (xint < xmin) xmin = xint
			if (yint < ymin) ymin = yint
		}
		filecalc(image_root//".roundoff.shJIH"//n_iter, "$1-"//xmin//";$2-"//ymin, format="%4.0f %4.0f", >> image_root//".roundoff.shJIH"//n_iter//"min0")
	}
	if (!access(image_root//".roundoff.shOPQ"//n_iter//"min0")) {
		list=image_root//".roundoff.shOPQ"//n_iter
		xmin=9999; ymin=9999
		while (fscan(list,xint,yint) !=EOF) {
			if (xint < xmin) xmin = xint
			if (yint < ymin) ymin = yint
		}
		filecalc(image_root//".roundoff.shOPQ"//n_iter, "$1-"//xmin//";$2-"//ymin, format="%4.0f %4.0f", >> image_root//".roundoff.shOPQ"//n_iter//"min0")
	}
	if (!access(image_root//".roundoff.shRST"//n_iter//"min0")) {
		list=image_root//".roundoff.shRST"//n_iter
		xmin=9999; ymin=9999
		while (fscan(list,xint,yint) !=EOF) {
			if (xint < xmin) xmin = xint
			if (yint < ymin) ymin = yint
		}
		filecalc(image_root//".roundoff.shRST"//n_iter, "$1-"//xmin//";$2-"//ymin, format="%4.0f %4.0f", >> image_root//".roundoff.shRST"//n_iter//"min0")
	}

# if not excluding any images:
	if (!exclude) {
		if(ex_string=="")ex_string=exstring

# now offset by offsets to the central image (LEMDN)
		if(!access(in_list//"_all")) concat (in_list//"_ABC,"//in_list//"_LEMDN,"//in_list//"_KFG,"//in_list//"_JIH,"//in_list//"_OPQ,"//in_list//"_RST", in_list//"_all")
		if(!access(image_root//".roundoff.shall"//n_iter)) {
			filecalc(image_root//".roundoff.shABC"//n_iter//"min0","$1+"//xabcba//";$2+"//yabcba,format="%4.0f %4.0f", >> image_root//".roundoff.shall"//n_iter)
			filecalc(image_root//".roundoff.shLEMDN"//n_iter//"min0","$1+"//xlemdnba//";$2+"//ylemdnba,format="%4.0f %4.0f",>>image_root//".roundoff.shall"//n_iter)
			filecalc(image_root//".roundoff.shKFG"//n_iter//"min0","$1+"//xkfgba//";$2+"//ykfgba,format="%4.0f %4.0f", >> image_root//".roundoff.shall"//n_iter)
			filecalc(image_root//".roundoff.shJIH"//n_iter//"min0","$1+"//xjihba//";$2+"//yjihba,format="%4.0f %4.0f", >> image_root//".roundoff.shall"//n_iter)
			filecalc(image_root//".roundoff.shOPQ"//n_iter//"min0","$1+"//xopqba//";$2+"//yopqba,format="%4.0f %4.0f", >> image_root//".roundoff.shall"//n_iter)
			filecalc(image_root//".roundoff.shRST"//n_iter//"min0","$1+"//xrstba//";$2+"//yrstba,format="%4.0f %4.0f", >> image_root//".roundoff.shall"//n_iter)
		}
# do the coadd 
		print ("\n===> Coadding all images at once...")
		ircoadd (in_list//"_all", image_root//"all"//n_iter//"_1stpass", "", image_root//".roundoff.shall"//n_iter, "none", "mode", "none", 1, no, "log_"//in_list//"_all", stdpars-, makeavg+, flip-, imcoaddpars="phiirs$ircoaddonis.par")

# if excluding any images: 
	} else {

		if(!access(in_list//"_most")) {
# create list of all image lists 
			print (in_list//"_ABC\n"//in_list//"_LEMDN\n"//in_list//"_KFG\n"//in_list//"_JIH\n"//in_list//"_OPQ\n"//in_list//"_RST", > "_coaddTmp5_")
# create list of all image lists to be included in coadd
			!fgrep -v -f _coaddTmp1_ _coaddTmp5_ > _coaddTmp6_
			concat ("@_coaddTmp6_", in_list//"_most")
		}
# offset the shift lists of all the images to be included in the coadd
# by their offsets to the central image (LEMDN)
		if(!access(image_root//".1stpass.shmost"//n_iter)) {
# create list of codes of all image lists 
			print ("ABC\nLEMDN\nKFG\nJIH\nOPQ\nRST", > "_coaddTmp7_")
# create list of codes of all image lists to be included in coadd
			!fgrep -v -f _coaddTmp1_ _coaddTmp7_ > _coaddTmp8_
			list = "_coaddTmp8_"
			while (fscan (list, in_string) !=EOF) {
				if (in_string=="ABC") {
					filecalc (image_root//".roundoff.shABC"//n_iter//"min0", "$1+"//xabcba//";$2+"//yabcba, format="%4.0f %4.0f", >> image_root//".1stpass.shmost"//n_iter)
				} else if (in_string=="LEMDN") {
					filecalc (image_root//".roundoff.shLEMDN"//n_iter//"min0", "$1+"//xlemdnba//";$2+"//ylemdnba, format="%4.0f %4.0f", >> image_root//".1stpass.shmost"//n_iter)
				} else if (in_string=="KFG") {
					filecalc (image_root//".roundoff.shKFG"//n_iter//"min0", "$1+"//xkfgba//";$2+"//ykfgba, format="%4.0f %4.0f", >> image_root//".1stpass.shmost"//n_iter)
				} else if (in_string=="JIH") {
					filecalc (image_root//".roundoff.shJIH"//n_iter//"min0", "$1+"//xjihba//";$2+"//yjihba, format="%4.0f %4.0f", >> image_root//".1stpass.shmost"//n_iter)
				} else if (in_string=="OPQ") {
					filecalc (image_root//".roundoff.shOPQ"//n_iter//"min0", "$1+"//xopqba//";$2+"//yopqba, format="%4.0f %4.0f", >> image_root//".1stpass.shmost"//n_iter)
				} else if (in_string=="RST") {
					filecalc (image_root//".roundoff.shRST"//n_iter//"min0", "$1+"//xrstba//";$2+"//yrstba, format="%4.0f %4.0f", >> image_root//".1stpass.shmost"//n_iter)
				}
			}
			del ("_coaddTmp?_", ver-, >& "dev$null")
		}

# do the most-images coadd 
		if (!access(image_root//"most"//n_iter//"_1stpass."//im_extn)) {
			print ("\n===> Coadding most images at once...")
			ircoadd (in_list//"_most", image_root//"most"//n_iter//"_1stpass", "", image_root//".1stpass.shmost"//n_iter, "none", "mode", "none", 1, no, "log_"//in_list//"_all", stdpars-, makeavg+, flip-, imcoaddpars="phiirs$ircoaddonis.par")
		}

# do nshift on most-images coadd
		if (!access(image_root//".roundoff.shrest"//n_iter)) nshift ("", in_list//"_coaddrest_"//n_iter, image_root//"most"//n_iter//"_1stpass", image_root, "shrest"//n_iter, redo=re_do, useimages=images_use, displayim=im_display, editfiles=files_edit)
# get shifts for images excluded from most-images coadd
		if (!access(image_root//".imshifts.shrest"//n_iter)) join(in_list//"_coaddrest_"//n_iter,image_root//".shiftexact.shrest"//n_iter, >>image_root//".imshifts.shrest"//n_iter)
		grep("LEMDN",image_root//".imshifts.shrest"//n_iter)|scan(theory, lemdnmost_xshiftexact, lemdnmost_yshiftexact)
    		if(lemdnmost_xshiftexact>=0)xlemdnmost=int(0.5+lemdnmost_xshiftexact)
    		if(lemdnmost_yshiftexact>=0)ylemdnmost=int(0.5+lemdnmost_yshiftexact)
    		if(lemdnmost_xshiftexact<0)xlemdnmost=int(-0.5+lemdnmost_xshiftexact)
    		if(lemdnmost_yshiftexact<0)ylemdnmost=int(-0.5+lemdnmost_yshiftexact)
                if (stridx(ex_string,"ABC")!=0) {
			grep("ABC",image_root//".imshifts.shrest"//n_iter)|scan(theory, abcba_xshiftexact, abcba_yshiftexact)
    			if(abcba_xshiftexact>=0)xabcba=int(0.5+abcba_xshiftexact)-xlemdnmost
    			if(abcba_yshiftexact>=0)yabcba=int(0.5+abcba_yshiftexact)-ylemdnmost
    			if(abcba_xshiftexact<0)xabcba=int(-0.5+abcba_xshiftexact)-xlemdnmost
    			if(abcba_yshiftexact<0)yabcba=int(-0.5+abcba_yshiftexact)-ylemdnmost
                }
		if (stridx(ex_string,"LEMDN")!=0) {
			grep("LEMDN",image_root//".imshifts.shrest"//n_iter)|scan(theory,lemdnba_xshiftexact,lemdnba_yshiftexact)
    			if(lemdnba_xshiftexact>=0)xlemdnba=int(0.5+lemdnba_xshiftexact)-xlemdnmost
    			if(lemdnba_yshiftexact>=0)ylemdnba=int(0.5+lemdnba_yshiftexact)-ylemdnmost
    			if(lemdnba_xshiftexact<0)xlemdnba=int(-0.5+lemdnba_xshiftexact)-xlemdnmost
    			if(lemdnba_yshiftexact<0)ylemdnba=int(-0.5+lemdnba_yshiftexact)-ylemdnmost
                }
		if (stridx(ex_string,"KFG")!=0) {
			grep("KFG",image_root//".imshifts.shrest"//n_iter)|scan(theory,kfgba_xshiftexact,kfgba_yshiftexact)
    			if(kfgba_xshiftexact>=0)xkfgba=int(0.5+kfgba_xshiftexact)-xlemdnmost
    			if(kfgba_yshiftexact>=0)ykfgba=int(0.5+kfgba_yshiftexact)-ylemdnmost
    			if(kfgba_xshiftexact<0)xkfgba=int(-0.5+kfgba_xshiftexact)-xlemdnmost
    			if(kfgba_yshiftexact<0)ykfgba=int(-0.5+kfgba_yshiftexact)-ylemdnmost
                }
		if (stridx(ex_string,"JIH")!=0) {
			grep("JIH",image_root//".imshifts.shrest"//n_iter)|scan(theory,jihba_xshiftexact,jihba_yshiftexact)
    			if(jihba_xshiftexact>=0)xjihba=int(0.5+jihba_xshiftexact)-xlemdnmost
    			if(jihba_yshiftexact>=0)yjihba=int(0.5+jihba_yshiftexact)-ylemdnmost
    			if(jihba_xshiftexact<0)xjihba=int(-0.5+jihba_xshiftexact)-xlemdnmost
    			if(jihba_yshiftexact<0)yjihba=int(-0.5+jihba_yshiftexact)-ylemdnmost
                }
		if (stridx(ex_string,"OPQ")!=0) {
			grep("OPQ",image_root//".imshifts.shrest"//n_iter)|scan(theory,opqba_xshiftexact,opqba_yshiftexact)
    			if(opqba_xshiftexact>=0)xopqba=int(0.5+opqba_xshiftexact)-xlemdnmost
    			if(opqba_yshiftexact>=0)yopqba=int(0.5+opqba_yshiftexact)-ylemdnmost
    			if(opqba_xshiftexact<0)xopqba=int(-0.5+opqba_xshiftexact)-xlemdnmost
    			if(opqba_yshiftexact<0)yopqba=int(-0.5+opqba_yshiftexact)-ylemdnmost
                }
		if (stridx(ex_string,"RST")!=0) {
			grep("RST",image_root//".imshifts.shrest"//n_iter)|scan(theory,rstba_xshiftexact,rstba_yshiftexact)
    			if(rstba_xshiftexact>=0)xrstba=int(0.5+rstba_xshiftexact)-xlemdnmost
    			if(rstba_yshiftexact>=0)yrstba=int(0.5+rstba_yshiftexact)-ylemdnmost
    			if(rstba_xshiftexact<0)xrstba=int(-0.5+rstba_xshiftexact)-xlemdnmost
    			if(rstba_yshiftexact<0)yrstba=int(-0.5+rstba_yshiftexact)-ylemdnmost
                }

# now offset by offsets to the central image (LEMDN)
		if(!access(in_list//"_all")) concat (in_list//"_ABC,"//in_list//"_LEMDN,"//in_list//"_KFG,"//in_list//"_JIH,"//in_list//"_OPQ,"//in_list//"_RST", in_list//"_all")
		if(!access(image_root//".1stpass.shall"//n_iter)) {
			filecalc (image_root//".roundoff.shABC"//n_iter//"min0", "$1+"//xabcba//";$2+"//yabcba, format="%4.0f %4.0f", >> image_root//".1stpass.shall"//n_iter)
			filecalc (image_root//".roundoff.shLEMDN"//n_iter//"min0", "$1+"//xlemdnba//";$2+"//ylemdnba, format="%4.0f %4.0f", >> image_root//".1stpass.shall"//n_iter)
			filecalc (image_root//".roundoff.shKFG"//n_iter//"min0", "$1+"//xkfgba//";$2+"//ykfgba, format="%4.0f %4.0f", >> image_root//".1stpass.shall"//n_iter)
			filecalc (image_root//".roundoff.shJIH"//n_iter//"min0", "$1+"//xjihba//";$2+"//yjihba, format="%4.0f %4.0f", >> image_root//".1stpass.shall"//n_iter)
			filecalc (image_root//".roundoff.shOPQ"//n_iter//"min0", "$1+"//xopqba//";$2+"//yopqba, format="%4.0f %4.0f", >> image_root//".1stpass.shall"//n_iter)
			filecalc (image_root//".roundoff.shRST"//n_iter//"min0", "$1+"//xrstba//";$2+"//yrstba, format="%4.0f %4.0f", >> image_root//".1stpass.shall"//n_iter)
		}

# do the full coadd 
		print ("\n===> Coadding all images at once...")
		ircoadd (in_list//"_all", image_root//"all"//n_iter//"_1stpass", "", image_root//".1stpass.shall"//n_iter, "none", "mode", "none", 1, no, "log_"//in_list//"_all", stdpars-, makeavg+, flip-, imcoaddpars="phiirs$ircoaddonis.par")
	}
}


# cross-correlate individual images with coadded image
if (p_air=="all" && xcor_secondpass) {
	ixdim=xsize; iydim=ysize
	if(!access(sigprefix//image_root//"all"//n_iter//"_1stpass."//im_extn)){
		print ("\n===> Creating constant-RMS coadded image...")
		sigmanorm(image_root//"all"//n_iter//"_1stpass."//im_extn)
	}
	imgets(sigprefix//image_root//"all"//n_iter//"_1stpass","i_naxis1")
	oxdim=int(imgets.value)
	imgets(sigprefix//image_root//"all"//n_iter//"_1stpass","i_naxis2")
	oydim=int(imgets.value)
	list=image_root//"all"//n_iter//"_1stpass.imshifts"
	if (!access(image_root//"all"//n_iter//"_xregout")) {
	  print("\n===> Cross-correlating individual images w/coadded image...")
		while (fscan(list,image_name,x_shift,y_shift) !=EOF) {
# zero out bad pixels if not already done
			imgets(image_name,"ZEROOUT")
			zero_out=imgets.value
			if(zero_out=="0"){
				imgets(image_name,"BPMASK")
				if(imgets.value!="0"){
imar(image_name,"*",imgets.value,image_name,pixt="real",calct="real",ver-,noac-)
hedit(image_name,"ZEROOUT",imgets.value,add+,ver-,show-,update+)
				} else {
print("Bad pixels not zeroed for "//image_name//", and no BP mask in header!")
				}
			}
			if(xsize==0){
				imgets(image_name,"i_naxis1")
				ixdim=int(imgets.value)
			}
			if(ysize==0){
				imgets(image_name,"i_naxis2")
				iydim=int(imgets.value)
			}
			ix=x_shift+1-14; if(ix<1)ix=1
                	iy=y_shift+1-14; if(iy<1)iy=1
                	xe=x_shift + ixdim + 14; if(xe>oxdim)xe=oxdim
                	ye=y_shift + iydim + 14; if(ye>oydim)ye=oydim
        		msect = "["//ix//":"//xe//","//iy//":"//ye//"]"
			print ("Rough shift from "//image_name//" to "//sigprefix//image_root//"all"//n_iter//"_1stpass is "//x_shift//" "//y_shift//" pixels")
			immatch.xreg (image_name, sigprefix//image_root//"all"//n_iter//"_1stpass", msect, image_root//"all"//n_iter//"_xregout", datab+, records="", xlag=-1*x_shift, ylag=-1*y_shift, xwindow=25, ywindow=25, append+)
		}
	}
	tempx = mktemp("tempx"); tempy = mktemp("tempy")
	grep ('xshift', image_root//"all"//n_iter//"_xregout") | fields ("junk", "2", >> tempx)
	grep ('yshift', image_root//"all"//n_iter//"_xregout") | fields ("junk", "2", >> tempy)
	if(!access(image_root//".xregexact.shall"//n_iter)) join (tempx, tempy, output=image_root//".xregexact.shall"//n_iter)
	del (tempx, ver-); del (tempy, ver-)
# Create optimum rounded-off integer offsets
	if (!access(rroundname)) {
      print("\n===> Finding optimum second-pass rounded-off integer offsets...")
	    old_x = 999.; old_y = 999.
# for each 0.01 pixel step...
	    for (i=-50; i<50; i+=1) {
		step = i / 100.
#		print (i)
# ...recalculate integer offsets...
		list=image_root//".xregexact.shall"//n_iter
		del (rroundname, ver-, >& "dev$null")
        	while(fscan(list,xoff,yoff) !=EOF) {
			xoff=xoff+step
			yoff=yoff+step
                	if(((abs(xoff))>=0.5) && ((abs(yoff))>=0.5)) {
                        	print(xoff,yoff, >> "temp."//rroundname)
                	} else {
                        	if((abs(xoff))<=0.5) {
                                	xoff=0.
                        	}
                        	if((abs(yoff))<=0.5) {
                                	yoff=0.
                        	}
                        	print(xoff,yoff, >> "temp."//rroundname)
                	}
     filecalc("temp."//rroundname, "$1;$2", format="%4.0f %4.0f", >> rroundname)
     			flpr; flpr; flpr
                	del ("temp."//rroundname, ver-, >& "dev$null")
        	}
        	del ("temp."//rroundname, ver-, >& "dev$null")
# calculate sum of squares of residuals of exact offsets - new integer offsets
		filecalc (image_root//".xregexact.shall"//n_iter//","//rroundname, "$1@1-$1@2+"//step) | average (option = "new_sample", >& "dev$null")
		flpr; flpr; flpr
		new_x=average.sumsqrs
		filecalc (image_root//".xregexact.shall"//n_iter//","//rroundname, "$2@1-$2@2+"//step) | average (option = "new_sample", >& "dev$null")
		flpr; flpr; flpr
		new_y=average.sumsqrs
		if (new_x < old_x) { 
			old_x = new_x
			step_x = step
		}
		if (new_y < old_y) {
			old_y = new_y
			step_y = step
		}
	    }
# recalculate final integer offsets
	    list=image_root//".xregexact.shall"//n_iter
	    del(rroundname,ver-)
	    del ("temp."//rroundname, ver-, >& "dev$null")
	    while(fscan(list,xoff,yoff) !=EOF) {
		xoff=xoff+step_x
		yoff=yoff+step_y
        	if(((abs(xoff))>=0.5) && ((abs(yoff))>=0.5)) {
                	print(xoff,yoff, >> "temp."//rroundname)
        	} else {
                	if((abs(xoff))<=0.5) {
                       		xoff=0.
                	}
                	if((abs(yoff))<=0.5) {
                       		yoff=0.
                	}
                	print(xoff,yoff, >> "temp."//rroundname)
        	}
         filecalc("temp."//rroundname,"$1;$2",format="%4.0f %4.0f",>>rroundname)
		flpr; flpr; flpr
        	del ("temp."//rroundname, ver-, >& "dev$null")
	    }
	del ("temp."//rroundname, ver-, >& "dev$null")
	}
	del ("temp."//rroundname, ver-, >& "dev$null")
	filecalc (rroundname//","//image_root//".1stpass.shall"//n_iter, "$1@1-$1@2", lines="1-") | average
	filecalc (rroundname//","//image_root//".1stpass.shall"//n_iter, "$1@1-$1@2-"//average.mean, lines="1-", >> "temp."//rroundname) 
	filecalc (rroundname//","//image_root//".1stpass.shall"//n_iter, "$2@1-$2@2", lines="1-") | average
	filecalc (rroundname//","//image_root//".1stpass.shall"//n_iter, "$2@1-$2@2-"//average.mean, lines="1-", >> "temp."//rroundname) 
	graph ("temp."//rroundname, point-, title="mean-subtracted 2ndpass - 1stpass x and y offsets")
	del ("temp."//rroundname, ver-, >& "dev$null")
}


# Coadd images after second-pass cross-correlation
if (p_air=="all" && coadd_secondpass) {
	print ("\n===> Coadding after second-pass cross-correlation...")
	ircoadd(in_list//"_all", image_root//"all"//n_iter, "", rroundname, "none", "mode", "none", 1, no, "log_"//in_list//"_all", stdpars-, makeavg-, flip-, imcoaddpars="phiirs$ircoaddonis.par")
#	ircoadd(in_list//"_all", image_root//"all"//n_iter//"avg", "", rroundname, "none", "mode", "none", 1, no, "log_"//in_list//"_all", stdpars-, makeavg+, flip-, imcoaddpars="phiirs$ircoaddonis.par")
}


# Make coadded-image object mask
if (p_air=="all" && do_mkobjmask) {
	print ("\n===> Making coadded-image object mask...")
	mkobjmask (image_root//"all"//n_iter//"."//im_extn, "", suffix="objmask", nsmooth=3, subsample=3, filtsize=15, ngrow=1, interact+)
}


# Make individual-image object masks
if (p_air=="all" && do_masklet) {
	print ("\n===> Making individual-image object masks...")
	masklet (image_root//"all"//n_iter, "", "om", shiftfile="", editfile+, flip-, update+)
}


} else {
	print ("\nWARNING! Package stsdas.imgtools not loaded... task aborted.")
}

if (echo_g) beep

end

