# onis1348cn -- shift & coadd ONIS CNOC MS1358+62 Center-North dither pattern

# 990520 -- created from onis6pt.cl / onis3ew.cl
# 990521 -- skip shiftlongaxes step if redo- & *.roundoff.* exists

# ** could do xcor in a chain: after coaddlongaxes, 
# 	displ AB #1 CD #2 & measure 1-star offset, then 
# 	EHI #3 FGI #4 MLK #1 ON #2 TSP #3 URQ #4 VWX #1
# ** would allow all shifts to be done at once

# ** in pre-review: display xm=0.5 ym=0.5 with estimated offsets?

# -- ideally would include multimasklet.cl instead of masklet.cl,
#	to identify & exclude spurious objects from bright star residuals.

# layout of MS1358+62 Center-North pattern & image pairings (long axis vertical):
#
# L12 C3/I9  D4/H8 E5	KL BJCI AGDH FE
# K11 B2/J10 A1/G7 F6
#
# G7 = N A1	
# H8 = N B2
# I9 = N C3
# J10 = N D4
# K11 = N E5
# L12 = N F6

# layout of MS1358+62 South pattern & image pairings (long axis vertical):
#
# B2 C3 F6	AB DC EF
# A1 D4 E5

procedure onis1348cn(inlist,imageroot,niter)

string	inlist		{prompt="List of input images"}
string	imageroot	{prompt="Root name for output files"}
int    	niter=1     	{prompt="Number to append to output files"}

string	pairing="all"	{enum="AGDH|BJCI|FE|KL|all",prompt="Images pairings to process (or all)"}
bool inspect=yes	{prompt="Inspect image lists before proceeding?"}
bool shiftlongaxes=yes	{prompt="Find shifts for image pairings w/overlapping long axes?"}
bool coaddlongaxes=yes	{prompt="Coadd image pairings w/overlapping long axes?"}
bool nshiftall=yes	{prompt="Find shifts between all image pairings?"}
bool prereview=yes	{prompt="Display all image for pre-nshift check?"}
bool p_tv=yes	     {prompt="Use ptv instead of display for pre-nshift check?"}
bool use_images=yes  {prompt="Measure shifts from images (not file) in nshift?"}
bool display_im=yes	{prompt="Display images for imexam in nshift?"}
bool edit_files=yes     {prompt="Edit coords files after creation in nshift?"}
bool exclude	   {prompt="Exclude any image pairings from first-pass nshift?"}
string exstring	{prompt="Enter code(s) for excluded pairing(s) separated by commas (AGDH|BJCI|FE|KL|all)"}
bool redo=no		{prompt="Redo shift-finding if already done?"}
bool coaddallimages=yes	{prompt="Coadd all images?"}
bool secondpassxcor=yes	{prompt="Cross-correlate all images with coadded image?"}
bool coaddsecondpass=yes {prompt="Coadd images after second-pass cross-correlation?"}
bool domkobjmask=yes	{prompt="Make coadded-image object mask (INTERACTIVE)?"}
bool domasklet=yes	{prompt="Make individual-image object masks?"}
string imextn="fits"	{prompt="Image extension"}
int xsize=512		{prompt="X-axis individual image size (0 for autosense)"}
int ysize=1024		{prompt="Y-axis individual image size (0 for autosense)"}
bool echog=yes		{prompt="Beep when task is done?"}

struct *list
 
begin
 
string in_list, image_root, p_air, ref_image, tmpheaders, sigprefix, rroundname
string theory, method_shortaxes, tempfile2, msect, image_name, tempx, tempy
string rvw_image, ex_string, current_string, future_string, in_string, im_extn
string zero_out
int xmin, ymin, xint, yint, old_xint, old_yint, xmax_bcad, xmin_fead, ix, iy
int xmin_tspurq, xmax_tspurq, xtspurq, ytspurq, tspn1
int xmin_vwxurq, xmax_vwxurq, xvwxurq, yvwxurq, vwxn1
int xmin_mlkfgj, xmax_mlkfgj, xmlkfgj, ymlkfgj, mlkn1
int xmin_ehifgj, xmax_ehifgj, xehifgj, yehifgj, ehin1
int n_iter, xbcad, ybcad, xfead, yfead, bcn1, fen1, x_shift, y_shift, xe, ye
int xref, yref, int_x, int_y, junkint, ilen, ipos
int oxdim, oydim     # dimensions of mask image
int ixdim, iydim     # dimensions of individual input frames
int xbamost, ybamost, xbaba, ybaba, xcdba, ycdba, xehiba, yehiba, xfgjba, yfgjba
int xmlkba, ymlkba, xonba, yonba, xtspba, ytspba, xurqba, yurqba, xvwxba, yvwxba
real baba_xshiftexact, baba_yshiftexact, bamost_xshiftexact, bamost_yshiftexact
real cdba_xshiftexact, cdba_yshiftexact, ehiba_xshiftexact, ehiba_yshiftexact
real fgjba_xshiftexact, fgjba_yshiftexact, mlkba_xshiftexact, mlkba_yshiftexact 
real onba_xshiftexact, onba_yshiftexact, tspba_xshiftexact, tspba_yshiftexact
real urqba_xshiftexact, urqba_yshiftexact, vwxba_xshiftexact, vwxba_yshiftexact 
real xbcad2, ybcad2, xfead2, yfead2, bcad_x, bcad_y, fead_x, fead_y
real xtspurq2, ytspurq2, tspurq_xlag, tspurq_ylag
real xvwxurq2, yvwxurq2, vwxurq_xlag, vwxurq_ylag
real xurq, yurq, xtsp, ytsp, xvwx, yvwx
real xmlkfgj2, ymlkfgj2, mlkfgj_xlag, mlkfgj_ylag
real xehifgj2, yehifgj2, ehifgj_xlag, ehifgj_ylag
real xfgj, yfgj, xmlk, ymlk, xehi, yehi
real xad, yad, xbc, ybc, xfe, yfe, bcad_xlag, bcad_ylag, fead_xlag, fead_ylag
real bcad_xfirstpass, bcad_yfirstpass, fead_xfirstpass, fead_yfirstpass
real tspurq_xfirstpass, tspurq_yfirstpass, vwxurq_xfirstpass, vwxurq_yfirstpass 
real mlkfgj_xfirstpass, mlkfgj_yfirstpass, ehifgj_xfirstpass, ehifgj_yfirstpass 
real xoff, yoff, step, new_x, new_y, old_x, old_y, step_x, step_y, xsig, ysig
bool in_spect, shift_longaxes, echo_g, coadd_longaxes, abort, go, do_masklet
bool coadd_allimages, xcor_secondpass, coadd_secondpass, do_mkobjmask, re_do
bool im_display, images_use, shift_shortaxes, coadd_shortaxes, xreg_secondpass
bool nshift_all, files_edit, ex_clude, pt_v, pre_review

in_list=inlist
image_root=imageroot
n_iter=niter
p_air=pairing

in_spect=inspect
shift_longaxes=shiftlongaxes
images_use=use_images
im_display=display_im
files_edit=edit_files
ex_string=""
re_do=redo
coadd_longaxes=coaddlongaxes
nshift_all=nshiftall
pre_review=prereview
pt_v=p_tv
coadd_allimages=coaddallimages
xcor_secondpass=secondpassxcor
coadd_secondpass=coaddsecondpass
do_mkobjmask=domkobjmask
do_masklet=domasklet
im_extn=imextn
echo_g=echog

rroundname=image_root//".xregshifts.shall"//n_iter
sigprefix=sigmanorm.prefix
baba_xshiftexact = 0; baba_yshiftexact = 0; cdba_xshiftexact = 0
cdba_xshiftexact = 0; ehiba_xshiftexact = 0; ehiba_yshiftexact = 0
fgjba_xshiftexact = 0; fgjba_yshiftexact = 0; mlkba_xshiftexact = 0
mlkba_yshiftexact = 0; onba_xshiftexact = 0; onba_yshiftexact = 0
tspba_xshiftexact = 0; tspba_yshiftexact = 0; urqba_xshiftexact = 0
urqba_yshiftexact = 0; vwxba_xshiftexact = 0; vwxba_yshiftexact = 0

go=yes

if(!defpac("imgtools")) {go=no}
if(go) {

# Separate input list into lists of images overlapping along long axes
print ("\n===> Checking image pairing lists...")
tmpheaders=mktemp("tmpheaders")
hsel ("@"//in_list, "$I,i_title", yes, >> tmpheaders)
if (!access(in_list//"_AGDH")) {
 grep ('Position\ \ 1\ of\ \ 6', tmpheaders) | grep ("-v", 'N') | grep ("-v", 'S') | fields ("junk", "1", lines="1-9999", >> in_list//"_AGDH")
 grep ('Position\ \ 1\ of\ \ 6', tmpheaders) | grep ('N') | fields ("junk", "1", lines="1-9999", >> in_list//"_AGDH")
 grep ('Position\ \ 4\ of\ \ 6', tmpheaders) | grep ("-v", 'N') | grep ("-v", 'S') | fields ("junk", "1", lines="1-9999", >> in_list//"_AGDH")
 grep ('Position\ \ 2\ of\ \ 6', tmpheaders) | grep ('N') | fields ("junk", "1", lines="1-9999", >> in_list//"_AGDH")
} else {
 print (in_list//"_AGDH already exists; not created.")
}
if (!access(in_list//"_BJCI")) {
 grep ('Position\ \ 2\ of\ \ 6', tmpheaders) | grep ("-v", 'N') | grep ("-v", 'S') | fields ("junk", "1", lines="1-9999", >> in_list//"_BJCI")
 grep ('Position\ \ 4\ of\ \ 6', tmpheaders) | grep ('N') | fields ("junk", "1", lines="1-9999", >> in_list//"_BJCI")
 grep ('Position\ \ 3\ of\ \ 6', tmpheaders) | grep ("-v", 'N') | grep ("-v", 'S') | fields ("junk", "1", lines="1-9999", >> in_list//"_BJCI")
 grep ('Position\ \ 3\ of\ \ 6', tmpheaders) | grep ('N') | fields ("junk", "1", lines="1-9999", >> in_list//"_BJCI")
} else {
 print (in_list//"_BJCI already exists; not created.")
}
if (!access(in_list//"_FE")) {
 grep ('Position\ \ 6\ of\ \ 6', tmpheaders) | grep ("-v", 'N') | grep ("-v", 'S') | fields ("junk", "1", lines="1-9999", >> in_list//"_FE")
 grep ('Position\ \ 5\ of\ \ 6', tmpheaders) | grep ("-v", 'N') | grep ("-v", 'S') | fields ("junk", "1", lines="1-9999", >> in_list//"_FE")
} else {
 print (in_list//"_FE already exists; not created.")
}
if (!access(in_list//"_KL")) {
 grep ('Position\ \ 5\ of\ \ 6', tmpheaders) | grep ('N') | fields ("junk", "1", lines="1-9999", >> in_list//"_KL")
 grep ('Position\ \ 6\ of\ \ 6', tmpheaders) | grep ('N') | fields ("junk", "1", lines="1-9999", >> in_list//"_KL")
} else {
 print (in_list//"_KL already exists; not created.")
}
del(tmpheaders,ver-)
if (in_spect) vi (in_list//"_?? "//in_list//"_????")


# Do irshift on lists of images overlapping along long axes
if (shift_longaxes) {
print("\n===> Finding shifts for image pairings overlapping along long axes...")
	if (p_air=="AGDH" || p_air=="all") {
		fields (in_list//"_AGDH", 1, lines="2-2") | scan (ref_image)
		if (!access(image_root//".roundoff.shAGDH"//n_iter)) {
			print("\n----> Overlap region is UPPER part of the reference image; using irshift <----")
			irshift (in_list//"_AGDH", ref_image, image_root, "shAGDH"//n_iter, useimages=images_use, displayim=im_display)
		}
	}
	if (p_air=="BJCI" || p_air=="all") {
		fields (in_list//"_BJCI", 1, lines="2-2") | scan (ref_image)
		if (re_do || !access(image_root//".roundoff.shBJCI"//n_iter)) {
			print("\n----> Overlap region is UPPER part of the reference image; using irshift <----")
			irshift (in_list//"_BJCI", ref_image, image_root, "shBJCI"//n_iter, useimages=images_use, displayim=im_display)
		}
	}
	if (p_air=="FE" || p_air=="all") {
		fields (in_list//"_FE", 1, lines="2-2") | scan (ref_image)
		if (re_do || !access(image_root//".roundoff.shFE"//n_iter)) {
			print("\n----> Overlap region is UPPER part of the reference image; using irshift <----")
			irshift (in_list//"_FE", ref_image, image_root, "shFE"//n_iter, useimages=images_use, displayim=im_display)
		}
	}
	if (p_air=="KL" || p_air=="all") {
		fields (in_list//"_KL", 1, lines="2-2") | scan (ref_image)
		if (re_do || !access(image_root//".roundoff.shKL"//n_iter)) {
			print("\n----> Overlap region is UPPER part of the reference image; using irshift <----")
			irshift (in_list//"_KL", ref_image, image_root, "shKL"//n_iter, useimages=images_use, displayim=im_display)
		}
	}
}


# Coadd images overlapping along long axes
if (coadd_longaxes) {
	print ("\n===> Coadding image pairings overlapping along long axes...")
	if (p_air=="AGDH" || p_air=="all") {
		imdel (image_root//"AGDH"//n_iter, ver-, >& "dev$null")
		imdel (image_root//"AGDH"//n_iter//"em", ver-, >& "dev$null")
		del (image_root//"AGDH"//n_iter//".imshifts", ver-, >& "dev$null")
		imdel (sigprefix//image_root//"AGDH"//n_iter, ver-, >& "dev$null")
		ircoadd (in_list//"_AGDH", image_root//"AGDH"//n_iter, "", image_root//".roundoff.shAGDH"//n_iter, "none", "mode", "none", 1, no, "log_"//image_root//"_allatonce", makeavg+, flip-)
#		sigmanorm (image_root//"AGDH"//n_iter)
	}
	if (p_air=="BJCI" || p_air=="all") {
		imdel (image_root//"BJCI"//n_iter, ver-, >& "dev$null")
		imdel (image_root//"BJCI"//n_iter//"em", ver-, >& "dev$null")
		del (image_root//"BJCI"//n_iter//".imshifts", ver-, >& "dev$null")
		imdel (sigprefix//image_root//"BJCI"//n_iter, ver-, >& "dev$null")
		ircoadd (in_list//"_BJCI", image_root//"BJCI"//n_iter, "", image_root//".roundoff.shBJCI"//n_iter, "none", "mode", "none", 1, no, "log_"//image_root//"_allatonce", makeavg+, flip-)
#		sigmanorm (image_root//"BJCI"//n_iter)
	}
	if (p_air=="FE" || p_air=="all") {
		imdel (image_root//"FE"//n_iter, ver-, >& "dev$null")
		imdel (image_root//"FE"//n_iter//"em", ver-, >& "dev$null")
		del (image_root//"FE"//n_iter//".imshifts", ver-, >& "dev$null")
		imdel (sigprefix//image_root//"FE"//n_iter, ver-, >& "dev$null")
		ircoadd (in_list//"_FE", image_root//"FE"//n_iter, "", image_root//".roundoff.shFE"//n_iter, "none", "mode", "none", 1, no, "log_"//image_root//"_allatonce", makeavg+, flip-)
#		sigmanorm (image_root//"FE"//n_iter)
	}
	if (p_air=="KL" || p_air=="all") {
		imdel (image_root//"KL"//n_iter, ver-, >& "dev$null")
		imdel (image_root//"KL"//n_iter//"em", ver-, >& "dev$null")
		del (image_root//"KL"//n_iter//".imshifts", ver-, >& "dev$null")
		imdel (sigprefix//image_root//"KL"//n_iter, ver-, >& "dev$null")
		ircoadd (in_list//"_KL", image_root//"KL"//n_iter, "", image_root//".roundoff.shKL"//n_iter, "none", "mode", "none", 1, no, "log_"//image_root//"_allatonce", makeavg+, flip-)
#		sigmanorm (image_root//"KL"//n_iter)
	}
}


# Find shifts between all image pairings
if (p_air=="all" && nshift_all) {
    if (!access(image_root//".shiftexact.shall"//n_iter)) {
# ** should test this out
	if (re_do) {
		del (image_root//".imshifts.shall*"//n_iter, ver-, >& "dev$null")
		del (image_root//".offsets.shall*"//n_iter, ver-, >& "dev$null")
		del (image_root//".roundoff.shall*"//n_iter, ver-, >& "dev$null")
		del (image_root//".shiftexact.shall*"//n_iter, ver-, >& "dev$null")
		del (image_root//".shiftfrac.shall*"//n_iter, ver-, >& "dev$null")
		del (image_root//".shiftint.shall*"//n_iter, ver-, >& "dev$null")
		del (image_root//".imshifts.shmost*"//n_iter, ver-, >& "dev$null")
		del (image_root//".offsets.shmost*"//n_iter, ver-, >& "dev$null")
		del (image_root//".roundoff.shmost*"//n_iter, ver-, >& "dev$null")
		del (image_root//".shiftexact.shmost*"//n_iter, ver-, >& "dev$null")
		del (image_root//".shiftfrac.shmost*"//n_iter, ver-, >& "dev$null")
		del (image_root//".shiftint.shmost*"//n_iter, ver-, >& "dev$null")
		print ("\n*allcoords*, *refcoords*, & *coarse* preserved; all else will be recreated.")
	}
	if (!access(in_list//"_coadd_"//n_iter)) {
		print (image_root//"AGDH"//n_iter, >> in_list//"_coadd_"//n_iter)
		print (image_root//"BJCI"//n_iter, >> in_list//"_coadd_"//n_iter)
		print (image_root//"FE"//n_iter, >> in_list//"_coadd_"//n_iter)
		print (image_root//"KL"//n_iter, >> in_list//"_coadd_"//n_iter)
	}
# initial pre-review 
	if (pre_review) {
	  print ("Central image will be displayed in frame 1, and others in 2.")
	  print ("Note any images that do not overlap the central image.")
	  list=in_list//"_coadd_"//n_iter
	  junkint = 1
	  while (fscan (list, rvw_image) !=EOF) {
		if (junkint==1) {
			if (pt_v) {
				ptv (rvw_image, frame=1)
			} else {
				display (rvw_image, frame=1)
			}
		} else {
			if (pt_v) {
				imexam (rvw_image, frame=2, keep-, display="ptv(image='$1',frame=2)")
			} else {
				display (rvw_image, junkint)
				imexam (rvw_image, frame=junkint, keep-, display="display(image='$1',frame=$2)")
				#imexam (rvw_image, frame=2, keep-, display="display(image='$1',frame=2)")
			}
		}
		junkint = junkint + 1
		if (junkint==5) junkint = 2
	  }
	}
# exclude any images from nshift at first?
	print ("")
	ex_clude=exclude
# if excluding any images: 
	if (ex_clude) {
# **redo here too? not a global redo, since need to get paramaters below for use in coadding
		del ("_coaddTmp?_", ver-, >& "dev$null")
# get code(s) of excluded image(s)
		ex_string=exstring
# create file of codes of images excluded from most-images coadd
#		files (image_root//ex_string//n_iter, >> "_coaddTmp1_")
		files (ex_string, >> "_coaddTmp1_")
# create file of images included in most-images coadd
		copy (in_list//"_coadd_"//n_iter, "_coaddTmp2_", ver-)
		!fgrep -v -f _coaddTmp1_ _coaddTmp2_ > _coaddTmp3_
# **redo/!redo instead of delete & recreate?
		del (in_list//"_coaddmost_"//n_iter, ver-, >& "dev$null")
		rename ("_coaddTmp3_", in_list//"_coaddmost_"//n_iter, field="all", >& "dev$null")
# create file of images included in full coadd
# **redo/!redo instead of delete & recreate?
		del (in_list//"_coaddrest_"//n_iter, ver-, >& "dev$null")
		print (image_root//"AGDH"//n_iter, > "_coaddTmp4_")
		!fgrep -f _coaddTmp1_ _coaddTmp2_ >> _coaddTmp4_
		print (image_root//"most"//n_iter//"_1stpass", >> "_coaddTmp4_")
		type ("_coaddTmp4_", >> in_list//"_coaddrest_"//n_iter)
# find shifts for most-images coadd
# **redo here too?
		if(!access(image_root//".roundoff.shmost"//n_iter)) nshift("", in_list//"_coaddmost_"//n_iter, image_root//"AGDH"//n_iter, image_root, "shmost"//n_iter, redo=re_do, useimages=images_use, displayim=im_display, editfiles=files_edit)
    		if (!access(image_root//".imshifts.shmost"//n_iter)) join (in_list//"_coaddmost_"//n_iter, image_root//".shiftexact.shmost"//n_iter, >> image_root//".imshifts.shmost"//n_iter)
    		grep ("AGDH", image_root//".imshifts.shmost"//n_iter) | scan (theory, baba_xshiftexact, baba_yshiftexact)
    		grep ("BJCI", image_root//".imshifts.shmost"//n_iter) | scan (theory, cdba_xshiftexact, cdba_yshiftexact)
  		grep ("FE", image_root//".imshifts.shmost"//n_iter) | scan (theory, fgjba_xshiftexact, fgjba_yshiftexact)
  		grep ("KL", image_root//".imshifts.shmost"//n_iter) | scan (theory, mlkba_xshiftexact, mlkba_yshiftexact)
	} else {
# if not excluding any images: 
		if (!access(image_root//".shiftexact.shall"//n_iter)) nshift ("", in_list//"_coadd_"//n_iter, image_root//"AGDH"//n_iter, image_root, "shall"//n_iter, redo=re_do, useimages=images_use, displayim=im_display, editfiles=files_edit)
    		fields(image_root//".shiftexact.shall"//n_iter,"1-2",lines="1-1")|scan(baba_xshiftexact,baba_yshiftexact)
    		fields(image_root//".shiftexact.shall"//n_iter,"1-2",lines="2-2")|scan(cdba_xshiftexact,cdba_yshiftexact)
  		fields(image_root//".shiftexact.shall"//n_iter,"1-2",lines="3-3")|scan(ehiba_xshiftexact,ehiba_yshiftexact)
  		fields(image_root//".shiftexact.shall"//n_iter,"1-2",lines="4-4")|scan(fgjba_xshiftexact,fgjba_yshiftexact)
  		fields(image_root//".shiftexact.shall"//n_iter,"1-2",lines="5-5")|scan(mlkba_xshiftexact,mlkba_yshiftexact)
  		fields(image_root//".shiftexact.shall"//n_iter,"1-2",lines="6-6")|scan(onba_xshiftexact,onba_yshiftexact)
  		fields(image_root//".shiftexact.shall"//n_iter,"1-2",lines="7-7")|scan(tspba_xshiftexact,tspba_yshiftexact)
  		fields(image_root//".shiftexact.shall"//n_iter,"1-2",lines="8-8")|scan(urqba_xshiftexact,urqba_yshiftexact)
  		fields(image_root//".shiftexact.shall"//n_iter,"1-2",lines="9-9")|scan(vwxba_xshiftexact,vwxba_yshiftexact)
	}
    }
    if(baba_xshiftexact>=0)xbaba=int(0.5+baba_xshiftexact)
    if(baba_yshiftexact>=0)ybaba=int(0.5+baba_yshiftexact)
    if(baba_xshiftexact<0)xbaba=int(-0.5+baba_xshiftexact)
    if(baba_yshiftexact<0)ybaba=int(-0.5+baba_yshiftexact)
    if(cdba_xshiftexact>=0)xcdba=int(0.5+cdba_xshiftexact)
    if(cdba_yshiftexact>=0)ycdba=int(0.5+cdba_yshiftexact)
    if(cdba_xshiftexact<0)xcdba=int(-0.5+cdba_xshiftexact)
    if(cdba_yshiftexact<0)ycdba=int(-0.5+cdba_yshiftexact)
    if(ehiba_xshiftexact>=0)xehiba=int(0.5+ehiba_xshiftexact)
    if(ehiba_yshiftexact>=0)yehiba=int(0.5+ehiba_yshiftexact)
    if(ehiba_xshiftexact<0)xehiba=int(-0.5+ehiba_xshiftexact)
    if(ehiba_yshiftexact<0)yehiba=int(-0.5+ehiba_yshiftexact)
    if(fgjba_xshiftexact>=0)xfgjba=int(0.5+fgjba_xshiftexact)
    if(fgjba_yshiftexact>=0)yfgjba=int(0.5+fgjba_yshiftexact)
    if(fgjba_xshiftexact<0)xfgjba=int(-0.5+fgjba_xshiftexact)
    if(fgjba_yshiftexact<0)yfgjba=int(-0.5+fgjba_yshiftexact)
    if(mlkba_xshiftexact>=0)xmlkba=int(0.5+mlkba_xshiftexact)
    if(mlkba_yshiftexact>=0)ymlkba=int(0.5+mlkba_yshiftexact)
    if(mlkba_xshiftexact<0)xmlkba=int(-0.5+mlkba_xshiftexact)
    if(mlkba_yshiftexact<0)ymlkba=int(-0.5+mlkba_yshiftexact)
    if(onba_xshiftexact>=0)xonba=int(0.5+onba_xshiftexact)
    if(onba_yshiftexact>=0)yonba=int(0.5+onba_yshiftexact)
    if(onba_xshiftexact<0)xonba=int(-0.5+onba_xshiftexact)
    if(onba_yshiftexact<0)yonba=int(-0.5+onba_yshiftexact)
    if(tspba_xshiftexact>=0)xtspba=int(0.5+tspba_xshiftexact)
    if(tspba_yshiftexact>=0)ytspba=int(0.5+tspba_yshiftexact)
    if(tspba_xshiftexact<0)xtspba=int(-0.5+tspba_xshiftexact)
    if(tspba_yshiftexact<0)ytspba=int(-0.5+tspba_yshiftexact)
    if(urqba_xshiftexact>=0)xurqba=int(0.5+urqba_xshiftexact)
    if(urqba_yshiftexact>=0)yurqba=int(0.5+urqba_yshiftexact)
    if(urqba_xshiftexact<0)xurqba=int(-0.5+urqba_xshiftexact)
    if(urqba_yshiftexact<0)yurqba=int(-0.5+urqba_yshiftexact)
    if(vwxba_xshiftexact>=0)xvwxba=int(0.5+vwxba_xshiftexact)
    if(vwxba_yshiftexact>=0)yvwxba=int(0.5+vwxba_yshiftexact)
    if(vwxba_xshiftexact<0)xvwxba=int(-0.5+vwxba_xshiftexact)
    if(vwxba_yshiftexact<0)yvwxba=int(-0.5+vwxba_yshiftexact)
}


# coadd all images at once (w/intermediate most-image coadd if needed)
if (p_air=="all" && coadd_allimages) {

# first set minimum x,y offsets to be 0 & 0 (independently)
	if (!access(image_root//".roundoff.shAGDH"//n_iter//"min0")) {
		list=image_root//".roundoff.shAGDH"//n_iter
		xmin=9999; ymin=9999
		while (fscan(list,xint,yint) !=EOF) {
			if (xint < xmin) xmin = xint
			if (yint < ymin) ymin = yint
		}
		filecalc(image_root//".roundoff.shAGDH"//n_iter, "$1-"//xmin//";$2-"//ymin, format="%4.0f %4.0f", >> image_root//".roundoff.shAGDH"//n_iter//"min0")
	}
	if (!access(image_root//".roundoff.shBJCI"//n_iter//"min0")) {
		list=image_root//".roundoff.shBJCI"//n_iter
		xmin=9999; ymin=9999
		while (fscan(list,xint,yint) !=EOF) {
			if (xint < xmin) xmin = xint
			if (yint < ymin) ymin = yint
		}
		filecalc(image_root//".roundoff.shBJCI"//n_iter, "$1-"//xmin//";$2-"//ymin, format="%4.0f %4.0f", >> image_root//".roundoff.shBJCI"//n_iter//"min0")
	}
	if (!access(image_root//".roundoff.shFE"//n_iter//"min0")) {
		list=image_root//".roundoff.shFE"//n_iter
		xmin=9999; ymin=9999
		while (fscan(list,xint,yint) !=EOF) {
			if (xint < xmin) xmin = xint
			if (yint < ymin) ymin = yint
		}
		filecalc(image_root//".roundoff.shFE"//n_iter, "$1-"//xmin//";$2-"//ymin, format="%4.0f %4.0f", >> image_root//".roundoff.shFE"//n_iter//"min0")
	}
	if (!access(image_root//".roundoff.shKL"//n_iter//"min0")) {
		list=image_root//".roundoff.shKL"//n_iter
		xmin=9999; ymin=9999
		while (fscan(list,xint,yint) !=EOF) {
			if (xint < xmin) xmin = xint
			if (yint < ymin) ymin = yint
		}
		filecalc(image_root//".roundoff.shKL"//n_iter, "$1-"//xmin//";$2-"//ymin, format="%4.0f %4.0f", >> image_root//".roundoff.shKL"//n_iter//"min0")
	}

# if not excluding any images:
	if (!exclude) {
		if(ex_string=="")ex_string=exstring

# now offset by offsets to the central image (AGDH)
		if(!access(in_list//"_all")) concat (in_list//"_AGDH,"//in_list//"_BJCI,"//in_list//"_FE,"//in_list//"_KL", in_list//"_all")
		if(!access(image_root//".roundoff.shall"//n_iter)) {
			filecalc (image_root//".roundoff.shAGDH"//n_iter//"min0", "$1+"//xbaba//";$2+"//ybaba, format="%4.0f %4.0f", >> image_root//".roundoff.shall"//n_iter)
			filecalc (image_root//".roundoff.shBJCI"//n_iter//"min0", "$1+"//xcdba//";$2+"//ycdba, format="%4.0f %4.0f", >> image_root//".roundoff.shall"//n_iter)
			filecalc (image_root//".roundoff.shFE"//n_iter//"min0", "$1+"//xfgjba//";$2+"//yfgjba, format="%4.0f %4.0f", >> image_root//".roundoff.shall"//n_iter)
			filecalc (image_root//".roundoff.shKL"//n_iter//"min0", "$1+"//xmlkba//";$2+"//ymlkba, format="%4.0f %4.0f", >> image_root//".roundoff.shall"//n_iter)
		}
# do the coadd 
		print ("\n===> Coadding all images at once...")
		ircoadd (in_list//"_all", image_root//"all"//n_iter//"_1stpass", "", image_root//".roundoff.shall"//n_iter, "none", "mode", "none", 1, no, "log_"//in_list//"_all", stdpars-, makeavg+, flip-, imcoaddpars="phiirs$ircoaddonis.par")

# if excluding any images: 
	} else {

		if(!access(in_list//"_most")) {
# create list of all image lists 
			print (in_list//"_AGDH\n"//in_list//"_BJCI\n"//in_list//"_FE\n"//in_list//"_KL", > "_coaddTmp5_")
# create list of all image lists to be included in coadd
			!fgrep -v -f _coaddTmp1_ _coaddTmp5_ > _coaddTmp6_
			concat ("@_coaddTmp6_", in_list//"_most")
		}
# offset the shift lists of all the images to be included in the coadd
# by their offsets to the central image (AGDH)
		if(!access(image_root//".1stpass.shmost"//n_iter)) {
# create list of codes of all image lists 
			print ("AGDH\nBJCI\nFE\nKL", > "_coaddTmp7_")
# create list of codes of all image lists to be included in coadd
			!fgrep -v -f _coaddTmp1_ _coaddTmp7_ > _coaddTmp8_
			list = "_coaddTmp8_"
			while (fscan (list, in_string) !=EOF) {
				if (in_string=="AGDH") {
					filecalc (image_root//".roundoff.shAGDH"//n_iter//"min0", "$1+"//xbaba//";$2+"//ybaba, format="%4.0f %4.0f", >> image_root//".1stpass.shmost"//n_iter)
				} else if (in_string=="BJCI") {
					filecalc (image_root//".roundoff.shBJCI"//n_iter//"min0", "$1+"//xcdba//";$2+"//ycdba, format="%4.0f %4.0f", >> image_root//".1stpass.shmost"//n_iter)
				} else if (in_string=="FE") {
					filecalc (image_root//".roundoff.shFE"//n_iter//"min0", "$1+"//xfgjba//";$2+"//yfgjba, format="%4.0f %4.0f", >> image_root//".1stpass.shmost"//n_iter)
				} else if (in_string=="KL") {
					filecalc (image_root//".roundoff.shKL"//n_iter//"min0", "$1+"//xmlkba//";$2+"//ymlkba, format="%4.0f %4.0f", >> image_root//".1stpass.shmost"//n_iter)
				}
			}
			del ("_coaddTmp?_", ver-, >& "dev$null")
		}

# do the most-images coadd 
		if (!access(image_root//"most"//n_iter//"_1stpass."//im_extn)) {
			print ("\n===> Coadding most images at once...")
			ircoadd (in_list//"_most", image_root//"most"//n_iter//"_1stpass", "", image_root//".1stpass.shmost"//n_iter, "none", "mode", "none", 1, no, "log_"//in_list//"_all", stdpars-, makeavg+, flip-, imcoaddpars="phiirs$ircoaddonis.par")
		}

# do nshift on most-images coadd
		if (!access(image_root//".roundoff.shrest"//n_iter)) nshift ("", in_list//"_coaddrest_"//n_iter, image_root//"most"//n_iter//"_1stpass", image_root, "shrest"//n_iter, redo=re_do, useimages=images_use, displayim=im_display, editfiles=files_edit)
# get shifts for images excluded from most-images coadd
		if (!access(image_root//".imshifts.shrest"//n_iter)) join(in_list//"_coaddrest_"//n_iter,image_root//".shiftexact.shrest"//n_iter, >>image_root//".imshifts.shrest"//n_iter)
		grep("AGDH",image_root//".imshifts.shrest"//n_iter)|scan(theory, bamost_xshiftexact, bamost_yshiftexact)
    		if(bamost_xshiftexact>=0)xbamost=int(0.5+bamost_xshiftexact)
    		if(bamost_yshiftexact>=0)ybamost=int(0.5+bamost_yshiftexact)
    		if(bamost_xshiftexact<0)xbamost=int(-0.5+bamost_xshiftexact)
    		if(bamost_yshiftexact<0)ybamost=int(-0.5+bamost_yshiftexact)
                if (stridx(ex_string,"AGDH")!=0) {
			grep("AGDH",image_root//".imshifts.shrest"//n_iter)|scan(theory, baba_xshiftexact, baba_yshiftexact)
    			if(baba_xshiftexact>=0)xbaba=int(0.5+baba_xshiftexact)-xbamost
    			if(baba_yshiftexact>=0)ybaba=int(0.5+baba_yshiftexact)-ybamost
    			if(baba_xshiftexact<0)xbaba=int(-0.5+baba_xshiftexact)-xbamost
    			if(baba_yshiftexact<0)ybaba=int(-0.5+baba_yshiftexact)-ybamost
                }
		if (stridx(ex_string,"BJCI")!=0) {
			grep("BJCI",image_root//".imshifts.shrest"//n_iter)|scan(theory,cdba_xshiftexact,cdba_yshiftexact)
    			if(cdba_xshiftexact>=0)xcdba=int(0.5+cdba_xshiftexact)-xbamost
    			if(cdba_yshiftexact>=0)ycdba=int(0.5+cdba_yshiftexact)-ybamost
    			if(cdba_xshiftexact<0)xcdba=int(-0.5+cdba_xshiftexact)-xbamost
    			if(cdba_yshiftexact<0)ycdba=int(-0.5+cdba_yshiftexact)-ybamost
                }
		if (stridx(ex_string,"FE")!=0) {
			grep("FE",image_root//".imshifts.shrest"//n_iter)|scan(theory,fgjba_xshiftexact,fgjba_yshiftexact)
    			if(fgjba_xshiftexact>=0)xfgjba=int(0.5+fgjba_xshiftexact)-xbamost
    			if(fgjba_yshiftexact>=0)yfgjba=int(0.5+fgjba_yshiftexact)-ybamost
    			if(fgjba_xshiftexact<0)xfgjba=int(-0.5+fgjba_xshiftexact)-xbamost
    			if(fgjba_yshiftexact<0)yfgjba=int(-0.5+fgjba_yshiftexact)-ybamost
                }
		if (stridx(ex_string,"KL")!=0) {
			grep("KL",image_root//".imshifts.shrest"//n_iter)|scan(theory,mlkba_xshiftexact,mlkba_yshiftexact)
    			if(mlkba_xshiftexact>=0)xmlkba=int(0.5+mlkba_xshiftexact)-xbamost
    			if(mlkba_yshiftexact>=0)ymlkba=int(0.5+mlkba_yshiftexact)-ybamost
    			if(mlkba_xshiftexact<0)xmlkba=int(-0.5+mlkba_xshiftexact)-xbamost
    			if(mlkba_yshiftexact<0)ymlkba=int(-0.5+mlkba_yshiftexact)-ybamost
                }

# now offset by offsets to the central image (AGDH)
		if(!access(in_list//"_all")) concat (in_list//"_AGDH,"//in_list//"_BJCI,"//in_list//"_FE,"//in_list//"_KL", in_list//"_all")
		if(!access(image_root//".1stpass.shall"//n_iter)) {
			filecalc (image_root//".roundoff.shAGDH"//n_iter//"min0", "$1+"//xbaba//";$2+"//ybaba, format="%4.0f %4.0f", >> image_root//".1stpass.shall"//n_iter)
			filecalc (image_root//".roundoff.shBJCI"//n_iter//"min0", "$1+"//xcdba//";$2+"//ycdba, format="%4.0f %4.0f", >> image_root//".1stpass.shall"//n_iter)
			filecalc (image_root//".roundoff.shFE"//n_iter//"min0", "$1+"//xfgjba//";$2+"//yfgjba, format="%4.0f %4.0f", >> image_root//".1stpass.shall"//n_iter)
			filecalc (image_root//".roundoff.shKL"//n_iter//"min0", "$1+"//xmlkba//";$2+"//ymlkba, format="%4.0f %4.0f", >> image_root//".1stpass.shall"//n_iter)
		}

# do the full coadd 
		print ("\n===> Coadding all images at once...")
		ircoadd (in_list//"_all", image_root//"all"//n_iter//"_1stpass", "", image_root//".1stpass.shall"//n_iter, "none", "mode", "none", 1, no, "log_"//in_list//"_all", stdpars-, makeavg+, flip-, imcoaddpars="phiirs$ircoaddonis.par")
	}
}


# cross-correlate individual images with coadded image
if (p_air=="all" && xcor_secondpass) {
	ixdim=xsize; iydim=ysize
	if(!access(sigprefix//image_root//"all"//n_iter//"_1stpass."//im_extn)){
		print ("\n===> Creating constant-RMS coadded image...")
		sigmanorm(image_root//"all"//n_iter//"_1stpass."//im_extn)
	}
	imgets(sigprefix//image_root//"all"//n_iter//"_1stpass","i_naxis1")
	oxdim=int(imgets.value)
	imgets(sigprefix//image_root//"all"//n_iter//"_1stpass","i_naxis2")
	oydim=int(imgets.value)
	list=image_root//"all"//n_iter//"_1stpass.imshifts"
	if (!access(image_root//"all"//n_iter//"_xregout")) {
	  print("\n===> Cross-correlating individual images w/coadded image...")
		while (fscan(list,image_name,x_shift,y_shift) !=EOF) {
# zero out bad pixels if not already done
                        imgets(image_name,"ZEROOUT")
                        zero_out=imgets.value
                        if(zero_out=="0"){
                                imgets(image_name,"BPMASK")
                                if(imgets.value!="0"){
imar(image_name,"*",imgets.value,image_name,pixt="real",calct="real",ver-,noac-)
hedit(image_name,"ZEROOUT",imgets.value,add+,ver-,show-,update+)
                                } else {
print("Bad pixels not zeroed for "//image_name//", and no BP mask in header!")
				}
                        }
			if(xsize==0){
				imgets(image_name,"i_naxis1")
				ixdim=int(imgets.value)
			}
			if(ysize==0){
				imgets(image_name,"i_naxis2")
				iydim=int(imgets.value)
			}
			ix=x_shift+1-14; if(ix<1)ix=1
                	iy=y_shift+1-14; if(iy<1)iy=1
                	xe=x_shift + ixdim + 14; if(xe>oxdim)xe=oxdim
                	ye=y_shift + iydim + 14; if(ye>oydim)ye=oydim
        		msect = "["//ix//":"//xe//","//iy//":"//ye//"]"
			print ("Rough shift from "//image_name//" to "//sigprefix//image_root//"all"//n_iter//"_1stpass is "//x_shift//" "//y_shift//" pixels")
			immatch.xreg (image_name, sigprefix//image_root//"all"//n_iter//"_1stpass", msect, image_root//"all"//n_iter//"_xregout", datab+, records="", xlag=-1*x_shift, ylag=-1*y_shift, xwindow=25, ywindow=25, append+)
		}
	}
	tempx = mktemp("tempx"); tempy = mktemp("tempy")
	grep ('xshift', image_root//"all"//n_iter//"_xregout") | fields ("junk", "2", >> tempx)
	grep ('yshift', image_root//"all"//n_iter//"_xregout") | fields ("junk", "2", >> tempy)
	if(!access(image_root//".xregexact.shall"//n_iter)) join (tempx, tempy, output=image_root//".xregexact.shall"//n_iter)
	del (tempx, ver-); del (tempy, ver-)
# Create optimum rounded-off integer offsets
	if (!access(rroundname)) {
      print("\n===> Finding optimum second-pass rounded-off integer offsets...")
	    old_x = 999.; old_y = 999.
# for each 0.01 pixel step...
	    for (i=-50; i<50; i+=1) {
		step = i / 100.
#		print (i)
# ...recalculate integer offsets...
		list=image_root//".xregexact.shall"//n_iter
		del (rroundname, ver-, >& "dev$null")
        	while(fscan(list,xoff,yoff) !=EOF) {
			xoff=xoff+step
			yoff=yoff+step
                	if(((abs(xoff))>=0.5) && ((abs(yoff))>=0.5)) {
                        	print(xoff,yoff, >> "temp."//rroundname)
                	} else {
                        	if((abs(xoff))<=0.5) {
                                	xoff=0.
                        	}
                        	if((abs(yoff))<=0.5) {
                                	yoff=0.
                        	}
                        	print(xoff,yoff, >> "temp."//rroundname)
                	}
     filecalc("temp."//rroundname, "$1;$2", format="%4.0f %4.0f", >> rroundname)
     			flpr; flpr; flpr
                	del ("temp."//rroundname, ver-, >& "dev$null")
        	}
        	del ("temp."//rroundname, ver-, >& "dev$null")
# calculate sum of squares of residuals of exact offsets - new integer offsets
		filecalc (image_root//".xregexact.shall"//n_iter//","//rroundname, "$1@1-$1@2+"//step) | average (option = "new_sample", >& "dev$null")
		flpr; flpr; flpr
		new_x=average.sumsqrs
		filecalc (image_root//".xregexact.shall"//n_iter//","//rroundname, "$2@1-$2@2+"//step) | average (option = "new_sample", >& "dev$null")
		flpr; flpr; flpr
		new_y=average.sumsqrs
		if (new_x < old_x) { 
			old_x = new_x
			step_x = step
		}
		if (new_y < old_y) {
			old_y = new_y
			step_y = step
		}
	    }
# recalculate final integer offsets
	    list=image_root//".xregexact.shall"//n_iter
	    del(rroundname,ver-)
	    del ("temp."//rroundname, ver-, >& "dev$null")
	    while(fscan(list,xoff,yoff) !=EOF) {
		xoff=xoff+step_x
		yoff=yoff+step_y
        	if(((abs(xoff))>=0.5) && ((abs(yoff))>=0.5)) {
                	print(xoff,yoff, >> "temp."//rroundname)
        	} else {
                	if((abs(xoff))<=0.5) {
                       		xoff=0.
                	}
                	if((abs(yoff))<=0.5) {
                       		yoff=0.
                	}
                	print(xoff,yoff, >> "temp."//rroundname)
        	}
         filecalc("temp."//rroundname,"$1;$2",format="%4.0f %4.0f",>>rroundname)
		flpr; flpr; flpr
        	del ("temp."//rroundname, ver-, >& "dev$null")
	    }
	del ("temp."//rroundname, ver-, >& "dev$null")
	}
	del ("temp."//rroundname, ver-, >& "dev$null")
	filecalc (rroundname//","//image_root//".1stpass.shall"//n_iter, "$1@1-$1@2", lines="1-") | average
	filecalc (rroundname//","//image_root//".1stpass.shall"//n_iter, "$1@1-$1@2-"//average.mean, lines="1-", >> "temp."//rroundname) 
	filecalc (rroundname//","//image_root//".1stpass.shall"//n_iter, "$2@1-$2@2", lines="1-") | average
	filecalc (rroundname//","//image_root//".1stpass.shall"//n_iter, "$2@1-$2@2-"//average.mean, lines="1-", >> "temp."//rroundname) 
	graph ("temp."//rroundname, point-, title="mean-subtracted 2ndpass - 1stpass x and y offsets")
	del ("temp."//rroundname, ver-, >& "dev$null")
}


# Coadd images after second-pass cross-correlation
if (p_air=="all" && coadd_secondpass) {
	print ("\n===> Coadding after second-pass cross-correlation...")
	ircoadd(in_list//"_all", image_root//"all"//n_iter, "", rroundname, "none", "mode", "none", 1, no, "log_"//in_list//"_all", stdpars-, makeavg-, flip-, imcoaddpars="phiirs$ircoaddonis.par")
#	ircoadd(in_list//"_all", image_root//"all"//n_iter//"avg", "", rroundname, "none", "mode", "none", 1, no, "log_"//in_list//"_all", stdpars-, makeavg+, flip-, imcoaddpars="phiirs$ircoaddonis.par")
}


# Make coadded-image object mask
if (p_air=="all" && do_mkobjmask) {
	print ("\n===> Making coadded-image object mask...")
	mkobjmask (image_root//"all"//n_iter//"."//im_extn, "", suffix="objmask", nsmooth=3, subsample=3, filtsize=15, ngrow=1, interact+)
}


# Make individual-image object masks
if (p_air=="all" && do_masklet) {
	print ("\n===> Making individual-image object masks...")
	masklet (image_root//"all"//n_iter, "", "om", shiftfile="", editfile+, flip-, update+)
}


} else {
	print ("\nWARNING! Package stsdas.imgtools not loaded... task aborted.")
}

if (echo_g) beep

end

