# awk script used by the PHIIRS routine photcheck for formatting output data 
# files.   The input data is taken from txdump, and consists of a series of 
# N lines for each apphot.phot photometry file, where column 1 of each line has
# the name of the image from which the photometry was measured, column 2 
# records the airmass of observation, column 3 contains the magnitude measured
# for the star, and column 4 contains the error estimate in that magnitude.  
# There will a number of comment lines beginning with "#" between each set
# of N input data lines, indicating a break before the next photometry file.
#
# originally created by Mark Dickinson, 3 Aug 1993, revised 2 March 1994
# modified March 1996 by Pat Hall
#
#   Set new file switch.
#
BEGIN {newfile = 1}
#
{
#
#   If input line is a lone #, set newfile=1 and print carriage return
#   except at the start of the file, thus avoiding a leading blank line.
#
	if (substr($0,1,1)=="#") {
		if (substr($0,2,2)=="") {
			if(newfile) {
				newfile = 1
			} else {
				newfile = 1
				printf ("\n")
			}
		}
#
#   Otherwise, if new file, print image name in column 1, airmass in column 2,
#   and set newfile=0.
#
	   } else {
		if (newfile) {
			printf ("%26s %6.4f ",$1,$2)
			newfile = 0
		}
#
#   If magnitude record = INDEF, print "INDEF" on end of current output record.
#
		if ($3=="INDEF") {
			printf (" %6s"," INDEF")
#
#   Otherwise, print magnitude on end of current output record.
#
		   } else {
			printf (" %6.3f",$3)
		}
#
#   If error record = INDEF, print "INDEF" on end of current output record.
#
		if ($4=="INDEF") {
			printf (" %6s"," INDEF")
#
#   Otherwise, print error on end of current output record.
#
		   } else {
			printf (" %6.3f",$4)
		}
	}
}
