# piscesfirst.cl -- subtract dark & add header params to PISCES images
#
# 1. subtract dark
# 2. hedit image headers, adding: 
# 	1. BPM,BPMASK = bad pixel mask name
#	2. GAIN,RDNOISE if not present (currently assumes 8, 25.)
#	3. time-obs = copy of 'time' (UT time of observation)
# 	4. obs-date = date-obs but in dd/mm/yy format
# 	5. ut       = time-obs (or time-obs + 7)
# 	6. date-obs = yyyy-mm-dd format (old date-obs saved as dateorig)
# (following are optional)
# 	7. epoch, ra, dec = entered as parameters, if desired
# 	8. st = sidereal time, calculated from ut & date using astutil.asttimes
# 		& the observatory parameter (kpno, mtbigelow, mmto, ctio)
# 	9. airmass  = calculated from epoch, ra, dec, st, and obs-date
#
# 980414: added compatibility to incoming date-obs in new-millenium format and
#   output of date-obs in same format (valid for data from 1990 to 2089);
#   original date-obs is output as dateorig
# 981228: added check for dateorig -- if it exists, date-obs etc. not redone
# 990112: created from first.cl and modified for Don McCarthy's array
# 990219: edited setairmass.intype and inserted "datetmp" header card
# 990223: added addition of "coadds" header parameter (always assumed = 1)
# 990309: checks for blank OBJECT keyword and replaces it with "blank"
# 990320: fixed bug where exptime & itime were feedbacking on each other
# 990419: existing gain & rdnoise not overwritten, + minor bug fixed
# 990426: epoch,ra,dec made optional again; copy SECZ keyword to AIRMASS
# 990427: DARKSUB keyword added to darksubbed images, not original ones
#
# **bug: airmass+ airauto+ option assumes you're observing from MST time zone!
#	...but that's only for the "asttimes" task, which may be irrelevant
# **should be more options for computing ut than just time-obs or time-obs+7h

procedure piscesfirst(imagelist,darksub,badmask)

string imagelist  {prompt="List of images to process (do not use an `@')"}
bool   darksub=no {prompt="Subtract off dark frame?"}
string badmask    {prompt="Name of bad pixel mask to add to image header"}

string dark       {prompt="Name of the dark frame to subtract", mode="q"}
string darkstring {prompt="String to append to dark-subtracted images",mode="q"}
bool   coords=yes {prompt="Add RA, DEC, and EPOCH to image header?"}
bool  airmass=yes {prompt="Enter airmass into image header?"}
bool  airauto=yes {prompt="Find airmass of image automatically?"}
real   airval     {prompt="Airmass of image", mode="q"}
string epoch	  {prompt="Epoch of coordinates"}
string ra	  {prompt="Right Ascension (surround with quotes)"}
string dec	  {prompt="Declination (surround w/quotes; use - but not +"}
string observatory {"kpno",prompt="Observatory where observations were made"}
bool   ut=yes      {prompt="Is 'time' in header ut or not?"}
bool   ddmmyy=yes  {prompt="Is 'date' in day/month/year format?"}
bool   echog=yes   {prompt="Beep when task is done?"}
struct *list

begin

string ddark, ddarkstring, bbadmask, check, base, iimagelist, ext
string darklist, flat, flatbase, meanfile, currentflat, tempflat, name
string editlist, current, obsdate, rra, ddec, eepoch, oobservatory, minsec
string obstime, timeobs, dateobs, century
real intime, aairval, ttime, mean, meanbad, x
int ncoadd, ddy, hhr, hhour, mmonth, yyear, dday, uttime, last2digits
bool ddarksub, ccoords, aairauto, uut, aairmass

iimagelist = imagelist
list = iimagelist
editlist = iimagelist
ddarksub = darksub
if (ddarksub) {
	ddark = dark
	ddarkstring = darkstring
	darklist = iimagelist//ddarkstring
}
bbadmask = badmask
ccoords = coords
aairmass = airmass
if (aairmass) {
	aairauto = airauto
#	if(!aairauto)aairval = airval
	if(!defpac("astutil")) {
		beep
  		print ("WARNING!!  Package ''astutil'' is not loaded; cannot set airmass in images.")
  		print ("Load the package and rerun the task.")
		return
	}
}
if (coords) {
	eepoch = epoch
	rra = ra
	ddec = dec
}
oobservatory = observatory
uut = ut

# Add BPM and other goodies to the image headers, just like in the hadd.cl task
print("===> Adding various parameters to the image headers...")
list = iimagelist
while (fscan (list, current) !=EOF) {
  hedit(current,"datetmp",("(@'date')"),add+,ver-,show-,updat+)
  hedit(current,"BPM",bbadmask,add+,ver-,show+,updat+)
  hedit(current,"BPMASK",bbadmask,add+,ver-,show+,updat+)
  imgets(current,"SECZ", >& "dev$null")
  if (imgets.value!="0" || imgets.value!="") {
    hedit(current,"AIRMASS",real(imgets.value),add+,ver-,show+,updat+)
#    hedit(current,"AIRMASS",("(@'SECZ')"),add+,ver-,show+,updat+)
  }
  imgets(current,"OBJECT", >& "dev$null")
  if (imgets.value=="0" || imgets.value=="") {
    hedit(current,"OBJECT","blank",add+,ver-,show+,updat+)
  }
# Currently assumes gain=8, rdnoise=25, and only 1 coadd per image
  imgets(current,"GAIN", >& "dev$null")
  if (imgets.value=="0" || imgets.value=="") {
    hedit(current,"GAIN","8.",add+,ver-,show+,updat+)
  }
  imgets(current,"RDNOISE", >& "dev$null")
  if (imgets.value=="0" || imgets.value=="") {
    hedit(current,"RDNOISE","25.",add+,ver-,show+,updat+)
  }
  hedit(current,"coadds",1.,add+,ver-,show+,updat+)
# No 'date-obs' keyword, only 'date', in format "dd/mm/yy".
# Create new 'date-obs' keyword in the new format "yyyy-mm-dd".
  imgets(current,"dateorig", >& "dev$null")
  if (imgets.value!="0") {
    print ("DATEORIG keyword already exists -- *first already run on image?")
  } else {
    hedit(current,"time-obs",("(@'time')"),add+,ver-,show+,updat+)
    imgets(current,"datetmp")
# Copy original 'date' keyword to 'dateorig' in original format.
    hedit(current,"dateorig",imgets.value,add+,ver-,show+,updat+)
    hedit(current,"datetmp","",add-,del+,ver-,show-,updat+)
# Create header keyword 'obs-date', always in "dd/mm/yy" format.
    if(!ddmmyy){
# mm/dd/yy format:
      if(substr(imgets.value,3,3)=="/") {
        obsdate=substr(imgets.value,4,6)//substr(imgets.value,1,3)//substr(imgets.value,7,8)
      } else {
# yyyy-mm-dd format:
# 1234567890
        obsdate=substr(imgets.value,9,10)//"/"//substr(imgets.value,6,7)//"/"//substr(imgets.value,3,4)
      }
# create obs-date if date-obs in mm/dd/yy or yyyy-mm-dd format:
      hedit(current,"obs-date",obsdate,add+,ver-,show+,updat+)
    } else {
# create obs-date if date-obs in dd/mm/yy format:
      hedit(current,"obs-date",imgets.value,add+,ver-,show+,updat+)
    }
# Create new 'date-obs' in new format "yyyy-mm-dd" from 'obs-date' "dd/mm/yy"
#								    12345678
    imgets(current,"obs-date")
    if ((substr(imgets.value,7,7))=="9") {
  	  century=19
    } else {
  	  century=20
    }
    dateobs=century//substr(imgets.value,7,8)//"-"//substr(imgets.value,4,5)//"-"//substr(imgets.value,1,2)
    hedit(current,"date-obs",dateobs,add+,ver-,show+,updat+)
#
# Sometimes the time in the image header is ut, sometimes not.
# so this option lets you choose either ut or mst
  if(uut){hedit(current,"ut",("(@'time-obs')"),add+,ver-,show+,updat+)}
  else{
    imgets(current,"time-obs")
    obstime=substr(imgets.value,1,2)
    uttime=int(obstime)+7
    if(uttime>=24){uttime=uttime-24}
    if(uttime<=9){timeobs="0"//str(uttime)//substr(imgets.value,3,8)}
    if(uttime>=10){timeobs=str(uttime)//substr(imgets.value,3,8)}
#need to hedit ut twice cause is not in hh:mm:ss format 1st time around
    hedit(current,"ut",timeobs,add+,ver-,show+,updat+)
    hedit(current,"ut",timeobs,add+,ver-,show+,updat+)
    }
  }
}

if(ccoords){
	list = iimagelist
	while (fscan (list, current) !=EOF) {
  		hedit(current,"epoch",eepoch,add+,ver-,show+,updat+)
  		hedit(current,"ra",rra,add+,ver-,show-,updat+)
  		hedit(current,"ra",rra,add+,ver-,show+,updat+)
  		hedit(current,"dec",ddec,add+,ver-,show-,updat+)
  		hedit(current,"dec",ddec,add+,ver-,show+,updat+)
#hedit test ra "(str('21:51:59.0'))" add+ ver- show-
	}
}

if(aairmass){
  if(aairauto){
    list = iimagelist
    while (fscan (list, current) !=EOF) {
  	imgets(current,"obs-date")
  	dday=int(substr(imgets.value,1,2))
  	mmonth=int(substr(imgets.value,4,5))
#  	yyear=int("19"//substr(imgets.value,7,8))
	last2digits=int(substr(imgets.value,7,8))
	if(last2digits <= 80) {
  	  yyear=int("20"//substr(imgets.value,7,8))
	} else {
  	  yyear=int("19"//substr(imgets.value,7,8))
	}
  	imgets(current,"ut")
  	hhr=int(substr(imgets.value,1,2))
  	minsec=substr(imgets.value,3,8)
	hhour=hhr-7
	if(hhour<0){
		dday=dday-1
		hhour=hhour+24.
		}
	if(dday<=0){
		mmonth=mmonth-1
# this will not always be totally accurate, and may fail for feb., but hey.
		dday=30
		}
# observing on new year's eve will require a new correction!
	ttime=real(str(int(hhour))//minsec)
	asttimes(observatory=oobservatory, year=yyear, month=mmonth, day=dday, time=ttime)
   	hedit(current,"st",asttimes.lmst,add+,ver-,show-,updat+)
   	hedit(current,"st",asttimes.lmst,add+,ver-,show+,updat+)
	setairmass(current, observatory=oobservatory, intype="end", outtype="effective", date="obs-date",exposure="exptime", utmiddle="utmiddle", show+, updat+, override+)
    }
  } else {
	list = iimagelist 
	while (fscan (list, current) !=EOF) { 
		print("Setting airmass for image "//current)
		hedit(current,"airmass",airval,add+,ver-,show+,updat+)
	}
  }
}

# Subtract off Dark Current 
if (ddarksub) {
	list = iimagelist 
# if darklist doesn't exist, assume dsub imgs don't either; create it & dsubs
	if(!access(darklist)) {
	    print("===> Subtracting off Dark Current...")
	    while (fscan (list, name) !=EOF) {
		fileroot(name,validim+)
		base=fileroot.root
		ext=fileroot.extension
		if(ext!="")ext="."//ext
		print(base//ddarkstring//ext, >> darklist)
  		imgets(name,"DARKSUB",>&"dev$null")
		if (imgets.value=="0") {
		    imarith(name, "-", ddark, base//ddarkstring//ext, verbose-)
	   hedit(base//ddarkstring//ext,"DARKSUB",ddark,add+,ver-,show+,update+)
		} else {
print("DARKSUB keyword exists; piscesfirst already run? "//name//" just copied")
		    imcopy(name, base//ddarkstring//ext, verbose-)
		}
	    }
	} else {
	    print("Darksubbed image list "//darklist//" already exists; images assumed to exist too.")
	}
#imarith("@"//iimagelist, "-", ddark, "@"//darklist, verbose-)
#hedit("@"//darklist,"DARKSUB",ddark,add+,ver-,show+,update+)
print(" ") 
print("===> Dark-subtracted images are listed in the file "//darklist//".") 
}

if(echog)beep

end

