# precoadd -- prepare images w/same pixel scales but dif't sizes for coadding
#
# Calls fileroot (old version also called minv, imcalc)
#
# 970402: created
# 970430: added TPL division option
# 970507: added iterstat lower=-10000.
#
# **ASSUMES that bad pixel masks originally have bad pixels = zero

procedure precoadd(listsmall,imgbig)

string listsmall  {prompt="List of smaller-size images"}
string imgbig	  {prompt="Sample larger-size image"}

bool	tpl=no		{prompt="Divide by throughput correction image first?"}
string	tplimg		{prompt="Name of throughput correction image"}
#bool	badzero=yes	{prompt="Do bad pixels in masks have value of 0?"}
bool	delorig=no	{prompt="Delete original images?"}
bool   echog        {yes,prompt="Beep when task is done?"}

begin

string img, llistsmall, bigimg, tmpzero, bpimage, bpimg, origlist, plistsmall
string bplist, origimg, image, intcosca, ttplimg
real s_maskval
int smallx, smally, ncoadds, bigx, bigy
bool ddelorig,go,ttpl
#bool bbadzero
struct theadline

# Set up parameters
llistsmall = listsmall
	plistsmall = "p_"//llistsmall
	del(plistsmall,ver-,>&"dev$null")
	bplist = "bp_"//llistsmall
	del(bplist,ver-,>&"dev$null")
bigimg = imgbig

ttpl=tpl
	if(ttpl)ttplimg=tplimg
#bbadzero=badzero
ddelorig=delorig


# Certain tasks/packages must be loaded for task to run.
go=yes
#if(!defpac("imgtools")) {go=no}
if(!deftask("fileroot")) {go=no}
#if(!deftask("minv")) {go=no}
if(go) {


# Scale images by NUMBER OF INTERNAL COADDS
    list=llistsmall
    print("Scaling by the number of internal coadds if necessary...")
    while (fscan (list, img) !=EOF) {
# Check: has this scaling already been done?  Look for INTCOSCA header card
	imgets(img, "intcosca", >& "dev$null")
	intcosca=str(imgets.value)
# If so, nothing is done.
# If not, look for the "coadds" header card (from 4m & later first.cl versions)
	if(intcosca=="0") {
		imgets(img, "coadds", >& "dev$null")
		ncoadds=int(imgets.value)
# If no "coadds" header card, search for the "ncoadds" header card.  It is
# usually only found in 'fits256.e' images (ie laird-type), but does not run
# the risk of being confused with any other comment cards in the image header,
# so it is looked for first.
		if(ncoadds==0) {
			imgets(img,"ncoadds", >& "dev$null")
			ncoadds=int(imgets.value)
# If no "ncoadds" header card, get ncoadds from the comment card
			if(ncoadds==0) {
				imgets(img,"comment", >& "dev$null")
     ncoadds=int(substr(imgets.value,strlen(imgets.value),strlen(imgets.value)))
			}	
		}
# Unless ncoadds=1 or 0, multiply through
	if(ncoadds!=1 && ncoadds!=0) imar(img,"*",ncoadds,img)
# Add INTCOSCA header card
        hedit(img,"INTCOSCA","Internal coadd scaling by factor "//ncoadds//" DONE",add+,ver-,show-,update+)
	}
    }
list=""


tmpzero=mktemp("zero")
imgets(bigimg,"i_naxis1"); bigx=int(imgets.value)
imgets(bigimg,"i_naxis2"); bigy=int(imgets.value)
imar (bigimg, "-", bigimg, tmpzero, ver-, noact-)

print ("Copying into larger-size blank images and creating bad pixel images...")
list=llistsmall
while (fscan (list,img) != EOF) {
# Divide by tpl image if desired
	if(ttpl){
		imgets(img,"TPLIMG")
		if(imgets.value=="0"){
			imar (img, "/", ttplimg, img, divzero=0., ver-, noact-)
			hedit (img,"TPLIMG",ttplimg,add+,ver-,update+,show-)
		} else {
      print ("TPL division not done -- image already has TPLIMG header keyword")
		}
	}
# Copy smaller-size image into subregions of larger-size images
	imgets(img,"i_naxis1"); smallx=int(imgets.value)
	imgets(img,"i_naxis2"); smally=int(imgets.value)
	if ((smallx==bigx) && (smally==bigy)) {
		imcopy (img, "p_"//img, ver-)
		print ("p_"//img, >> plistsmall)
	} else {
		imcopy (tmpzero, "p_"//img, ver-)
		print ("p_"//img, >> plistsmall)
		imcopy (img, "p_"//img//"[1:"//smallx//",1:"//smally//"]", ver-)
		imgets(img,"airmass")
		hedit("p_"//img,"airmass",imgets.value,add+,ver-,show-,update+)
		imgets(img,"objmask")
		hedit("p_"//img,"objmask",imgets.value,add+,ver-,show-,update+)
		imgets(img,"bpmask")
		hedit("p_"//img,"bpmask",imgets.value,add+,ver-,show-,update+)
		imgets(img,"bpm")
		hedit("p_"//img,"bpm",imgets.value,add+,ver-,show-,update+)
		imgets(img,"intcosca")
		hedit("p_"//img,"intcosca",imgets.value,add+,ver-,show-,update+)
		imgets(img,"exptime")
		hedit("p_"//img,"exptime",imgets.value,add+,ver-,show-,update+)
		imgets(img,"itime")
		hedit("p_"//img,"itime",imgets.value,add+,ver-,show-,update+)
	}
	imgets (img,"BPMASK"); bpimg=str(imgets.value)
	fileroot(bpimg,validim+); bpimage=fileroot.root
# If larger-size bad pixel mask doesn't already exist...
	if(!access("bp_"//bpimage//".pl")) {	
# ...copy smaller-size bad pixel mask into subregions of larger-size mask
		imdel ("t_"//bpimage//".pl", ver-, >& "dev$null")
		imcopy (tmpzero, "t_"//bpimage//".pl", ver-)
     imcopy (bpimage, "t_"//bpimage//".pl[1:"//smallx//",1:"//smally//"]", ver-)
# ...invert so that bad pixels are equal to one (temporarily) if not already so
#		if(bbadzero){
#			minv("t_"//bpimage//".pl", "bp_"//bpimage//".pl")
#			imdel ("t_"//bpimage//".pl", ver-)
#		} else {

	       	       imren ("t_"//bpimage//".pl", "bp_"//bpimage//".pl", ver-)

#		}
# Invert existing larger-size bad pixel mask if necessary
#	} else {
#		if(bbadzero){
#			imdel ("t_"//bpimage//".pl", ver-, >& "dev$null")
#			minv ("bp_"//bpimage//".pl", "t_"//bpimage//".pl")
#			imdel ("bp_"//bpimage//".pl", ver-)
#			imren ("t_"//bpimage//".pl", "bp_"//bpimage//".pl", ve-)
#		}
	}
	if(!access(bplist)) { print ("bp_"//bpimage//".pl", >> bplist) }

	hedit ("p_"//img,"BPMASK","bp_"//bpimage//".pl",add+,ver-,show-,update+)
	hedit ("p_"//img,"BPM","bp_"//bpimage//".pl",add+,ver-,show-,update+)
	hedit ("p_"//img,"ORIGIMG",img,add+,ver-,show-,update+)
}

if(ddelorig){
	print ("Deleting original images...")
	imdel("@"//llistsmall, ver-)
} else {
	print ("Renaming original images...")
	list=llistsmall
	origlist=llistsmall//"orig"
	del(origlist,ver-,>&"dev$null")
	while (fscan (list,img) != EOF) {
		imren (img, "orig"//img, ver-)
		print ("orig"//img, >> origlist)
	}
}

print ("Creating final larger-size images with bad pixels set to -99999.")
list=plistsmall
# Create final larger-size images with bad pixels set to -99999.
while (fscan (list,img) != EOF) {
	imgets (img,"BPMASK")
	bpimg=str(imgets.value)
	imgets (img,"ORIGIMG")
	origimg=str(imgets.value)
	imar (img, "/", bpimg, origimg, divz=-99999., ver-)
#	imcalc (img//","//bpimg, origimg, "im1-im2*99999", ver-)
}

#if(bbadzero){
#	print ("Fixing bad pixel masks so bad pixels = 0...")
#	list=bplist
#	while (fscan (list,img) != EOF) {
#		fileroot(img,validim+)
#		image=fileroot.root
#		minv (image, "t_"//image//".pl")
#		imdel (image//".pl", ver-)
#		imren ("t_"//image//".pl", image//".pl", ver-)
#	}
#}

print ("Deleting temporary images...")
imdel (tmpzero, ver-)
imdel ("@"//plistsmall, ver-)
del (plistsmall, ver-)

print ("Getting robust statistics...")
iterstat ("@"//llistsmall, lower=-10000., upper=INDEF, nsig=5., maxiter=10, print+, ver-, >> "stats_"//llistsmall)

print("Images in "//llistsmall//" now same size as "//bigimg)
print("and have statistics listed in headers and stats_"//llistsmall)
print("and have new badpix masks listed in headers and "//bplist)
if(!ddelorig)print("Original images renamed to files listed in "//origlist)

} else {
#        print("WARNING: tasks stsdas.toolbox.imgtools.imcalc,")
#	print("fileroot, and/or minv are not loaded!")
        print("WARNING: fileroot task is not loaded!")
	print("Please load the appropriate package(s) and restart this task.")
}

list=""
if(echog)beep

end

