# sqiidfirst.cl -- task to subtract dark & add header params to SQIID images
#
# 1. hedit image headers, adding: (basically this task is hadd.cl plus #0)
# 	1. BPM,BPMASK = bad pixel mask name
#	2. GAIN,RDNOISE if not present (currently assumes 10.,40.)
#	3. coadds       = number of coadds (copy of ncoadds)
# 	4. itime,exptime  = exposure time per coadd (old exptime)
#				and total exposure time (itime * coadds)
# 	5. obs-date = date-obs but in dd/mm/yy format
# 	6. date-obs = yyyy-mm-dd format (old date-obs saved as dateorig)
# 2. optionally subtract dark
# 3. optionally reorient so N up and E left
#
# 000521: created SQIID version
# 000528: added REORIENT & changed so DARKSUB can be done for all filters at once
#
# **add undo-REORIENT step
# **refuse to mix images w/ & w/o REORIENT header keyword

procedure sqiidfirst(imagelist,darksub,bpmj,bpmh,bpmk,bpml)

string imagelist  {prompt="List of images to process (do not use an `@')"}
bool   darksub=no {prompt="Subtract off dark frame?"}
string bpmj       {prompt="Name of bad pixel mask to add to J image headers"}
string bpmh       {prompt="Name of bad pixel mask to add to H image headers"}
string bpmk       {prompt="Name of bad pixel mask to add to K image headers"}
string bpml       {prompt="Name of bad pixel mask to add to L image headers"}

string darkj      {prompt="Name of the J dark frame to subtract", mode="q"}
string darkh      {prompt="Name of the H dark frame to subtract", mode="q"}
string darkk      {prompt="Name of the K dark frame to subtract", mode="q"}
string darkl      {prompt="Name of the L dark frame to subtract", mode="q"}
string darkstring {prompt="String to append to dark-subtracted images",mode="q"}
string observatory="kpno" {prompt="Observatory where observations were made"}
bool   ddmmyy=yes  {prompt="Is 'date-obs' in day/month/year format?"}
bool   orient=yes  {prompt="Reorient so N is up and E is at left"}
bool   echog       {yes,prompt="Beep when task is done?"}
struct *list

begin

string ddark, ddarkstring, bbadmask, check, base, iimagelist, ext
string jdark, hdark, kdark, ldark, jbadmask, hbadmask, kbadmask, lbadmask
string darklist, flat, flatbase, meanfile, currentflat, tempflat, name
string editlist, current, obsdate, rra, ddec, eepoch, oobservatory, minsec
string obstime, timeobs, dateobs, century, channel
real intime, aairval, ttime, mean, meanbad, x
int ncoadd, ddy, hhr, hhour, mmonth, yyear, dday, uttime, last2digits
bool ddarksub, ccoords, aairauto, uut, aairmass, oorient

iimagelist = imagelist
	list = iimagelist
	editlist = iimagelist
ddarksub = darksub
jbadmask = bpmj
hbadmask = bpmh
kbadmask = bpmk
lbadmask = bpml
if (ddarksub) {
	jdark = darkj
	hdark = darkh
	kdark = darkk
	ldark = darkl
	ddarkstring = darkstring
	darklist = iimagelist//ddarkstring
}
oobservatory = observatory
oorient=orient

# Add BPM and other goodies to the image headers, just like in the hadd.cl task
print("===> Adding various parameters to the image headers...")
list = iimagelist
while (fscan (list, current) !=EOF) {
# Currently assumes gain=10, rdnoise=40
  imgets(current,"GAIN", >& "dev$null")
  if (imgets.value=="0" || imgets.value=="") {
    hedit(current,"GAIN","10.",add+,ver-,show+,updat+)
  }
  imgets(current,"RDNOISE", >& "dev$null")
  if (imgets.value=="0" || imgets.value=="") {
    hedit(current,"RDNOISE","40.",add+,ver-,show+,updat+)
  }
# Get channel (j or h or k or l)
  imgets(current,"channel", >& "dev$null")
  channel=imgets.value
# J ----------------------------------------------------------------------------
  if (channel=="j") {
    hedit(current,"BPM",jbadmask,add+,ver-,show+,update+)
    hedit(current,"BPMASK",jbadmask,add+,ver-,show+,update+)
# Set coadds=(# of coadds)
    imgets(current,"coaddsj", >& "dev$null")
    ncoadd=real(imgets.value)
    hedit(current,"coadds",(ncoadd),add+,ver-,show+,update+)
# Set itime=exposure time per coadd
    imgets(current,"itime", >& "dev$null")
    if (imgets.value!="0") {
      print ("ITIME keyword already exists -- sqiidfirst already run on image?")
    } else {
      imgets(current,"int_sj", >& "dev$null")
      intime=real(imgets.value)
      hedit(current,"itime",(intime),add+,ver-,show+,update+)
# Set exptime=itime * (# of coadds)
      hedit(current,"exptime",(ncoadd*intime),add+,ver-,show+,update+)
    }
# H ----------------------------------------------------------------------------
  } else if (channel=="h") {
    hedit(current,"BPM",hbadmask,add+,ver-,show+,update+)
    hedit(current,"BPMASK",hbadmask,add+,ver-,show+,update+)
# Set coadds=(# of coadds)
    imgets(current,"coaddsh", >& "dev$null")
    ncoadd=real(imgets.value)
    hedit(current,"coadds",(ncoadd),add+,ver-,show+,update+)
# Set itime=exposure time per coadd
    imgets(current,"itime", >& "dev$null")
    if (imgets.value!="0") {
      print ("ITIME keyword already exists -- sqiidfirst already run on image?")
    } else {
      imgets(current,"int_sh", >& "dev$null")
      intime=real(imgets.value)
      hedit(current,"itime",(intime),add+,ver-,show+,update+)
# Set exptime=itime * (# of coadds)
      hedit(current,"exptime",(ncoadd*intime),add+,ver-,show+,update+)
    }
# K ----------------------------------------------------------------------------
  } else if (channel=="k") {
    hedit(current,"BPM",kbadmask,add+,ver-,show+,update+)
    hedit(current,"BPMASK",kbadmask,add+,ver-,show+,update+)
# Set coadds=(# of coadds)
    imgets(current,"coaddsk", >& "dev$null")
    ncoadd=real(imgets.value)
    hedit(current,"coadds",(ncoadd),add+,ver-,show+,update+)
# Set itime=exposure time per coadd
    imgets(current,"itime", >& "dev$null")
    if (imgets.value!="0") {
      print ("ITIME keyword already exists -- sqiidfirst already run on image?")
    } else {
      imgets(current,"int_sk", >& "dev$null")
      intime=real(imgets.value)
      hedit(current,"itime",(intime),add+,ver-,show+,update+)
# Set exptime=itime * (# of coadds)
      hedit(current,"exptime",(ncoadd*intime),add+,ver-,show+,update+)
    }
# L ----------------------------------------------------------------------------
  } else if (channel=="l") {
    hedit(current,"BPM",lbadmask,add+,ver-,show+,update+)
    hedit(current,"BPMASK",lbadmask,add+,ver-,show+,update+)
# Set coadds=(# of coadds)
    imgets(current,"coaddsl", >& "dev$null")
    ncoadd=real(imgets.value)
    hedit(current,"coadds",(ncoadd),add+,ver-,show+,update+)
# Set itime=exposure time per coadd
    imgets(current,"itime", >& "dev$null")
    if (imgets.value!="0") {
      print ("ITIME keyword already exists -- sqiidfirst already run on image?")
    } else {
      imgets(current,"int_sl", >& "dev$null")
      intime=real(imgets.value)
      hedit(current,"itime",(intime),add+,ver-,show+,update+)
# Set exptime=itime * (# of coadds)
      hedit(current,"exptime",(ncoadd*intime),add+,ver-,show+,update+)
    }
  }
#
# The 'date-obs' keyword comes in either the old format "dd/mm/yy",
# the erroneous format "mm/dd/yy", or the new format "yyyy-mm-dd".
# Create new 'date-obs' keyword in the new format "yyyy-mm-dd".
  imgets(current,"dateorig")
  if (imgets.value!="0") {
   print ("DATEORIG keyword already exists -- sqiidfirst already run on image?")
  } else {
    imgets(current,"date-obs")
# Copy 'date-obs' keyword to 'dateorig' in original format.
    hedit(current,"dateorig",imgets.value,add+,ver-,show+,update+)
# Create header keyword 'obs-date', always in "dd/mm/yy" format.
    if(!ddmmyy){
# mm/dd/yy format:
      if(substr(imgets.value,3,3)=="/") {
        obsdate=substr(imgets.value,4,6)//substr(imgets.value,1,3)//substr(imgets.value,7,8)
      } else {
# yyyy-mm-dd format:
#  1234567890
        obsdate=substr(imgets.value,9,10)//"/"//substr(imgets.value,6,7)//"/"//substr(imgets.value,3,4)
      }
# create obs-date if date-obs in mm/dd/yy or yyyy-mm-dd format:
      hedit(current,"obs-date",obsdate,add+,ver-,show+,update+)
    } else {
# create obs-date if date-obs in dd/mm/yy format:
      hedit(current,"obs-date",imgets.value,add+,ver-,show+,update+)
    }
# Create new 'date-obs' in new format "yyyy-mm-dd" from 'obs-date' "dd/mm/yy"
#								    12345678
    imgets(current,"obs-date")
    if ((substr(imgets.value,7,7))=="9") {
  	  century=19
    } else {
  	  century=20
    }
    dateobs=century//substr(imgets.value,7,8)//"-"//substr(imgets.value,4,5)//"-"//substr(imgets.value,1,2)
    hedit(current,"date-obs",dateobs,add+,ver-,show+,update+)
#
  }
}

# Subtract off Dark Current (and reorient dark-subtracted images, if desired)
if (ddarksub) {
	list = iimagelist 
# if darklist doesn't exist, assume dsub imgs don't either; create it & dsubs
	if(!access(darklist)) {
	    print("===> Subtracting off Dark Current...")
	    while (fscan (list, name) !=EOF) {
		fileroot(name,validim+)
		base=fileroot.root
		ext=fileroot.extension
		if(ext!="")ext="."//ext
		print(base//ddarkstring//ext, >> darklist)
# Get channel (j or h or k or l)
  		imgets(name,"channel", >& "dev$null")
  		channel=imgets.value
  		imgets(name,"DARKSUB", >& "dev$null")
		if (imgets.value=="0") {
# J ----------------------------------------------------------------------------
  		    if (channel=="j") {
		        imarith(name,"-",jdark,base//ddarkstring//ext,verbose-)
	   hedit(base//ddarkstring//ext,"DARKSUB",jdark,add+,ver-,show+,update+)
			if (oorient) {
                            imgets(name,"REORIENT", >& "dev$null")
                            if (imgets.value=="0") {
		      imtranspose(base//ddarkstring//ext,base//ddarkstring//ext)
	  hedit(base//ddarkstring//ext,"REORIENT","yes",add+,ver-,show+,update+)
		            } else {
		       print("REORIENT keyword exists; sqiidfirst already run?")
		            }
		        }
# H ----------------------------------------------------------------------------
  		    } else if (channel=="h") {
		        imarith(name,"-",hdark,base//ddarkstring//ext,verbose-)
	   hedit(base//ddarkstring//ext,"DARKSUB",hdark,add+,ver-,show+,update+)
			if (oorient) {
                            imgets(name,"REORIENT", >& "dev$null")
                            if (imgets.value=="0") {
	    imcopy(base//ddarkstring//ext//"[-*,*]",base//ddarkstring//ext,ver-)
	  hedit(base//ddarkstring//ext,"REORIENT","yes",add+,ver-,show+,update+)
		            } else {
		       print("REORIENT keyword exists; sqiidfirst already run?")
		            }
		        }
# K ----------------------------------------------------------------------------
  		    } else if (channel=="k") {
		        imarith(name,"-",kdark,base//ddarkstring//ext,verbose-)
	   hedit(base//ddarkstring//ext,"DARKSUB",kdark,add+,ver-,show+,update+)
			if (oorient) {
                            imgets(name,"REORIENT", >& "dev$null")
                            if (imgets.value=="0") {
	    imcopy(base//ddarkstring//ext//"[*,-*]",base//ddarkstring//ext,ver-)
	  hedit(base//ddarkstring//ext,"REORIENT","yes",add+,ver-,show+,update+)
		            } else {
		       print("REORIENT keyword exists; sqiidfirst already run?")
		            }
		        }
# L ----------------------------------------------------------------------------
  		} else if (channel=="l") {
		        imarith(name,"-",ldark,base//ddarkstring//ext,verbose-)
	   hedit(base//ddarkstring//ext,"DARKSUB",ldark,add+,ver-,show+,update+)
			if (oorient) {
                            imgets(name,"REORIENT", >& "dev$null")
                            if (imgets.value=="0") {
			        print ("WARNING! raw orientation of L unknown.")
#			    hedit(name,"REORIENT","yes",add+,ver-,show+,update+)
		            } else {
		       print("REORIENT keyword exists; sqiidfirst already run?")
		            }
		        }
		    }
		} else {
 print("DARKSUB keyword exists; sqiidfirst already run? "//name//" just copied")
		    imcopy(name, base//ddarkstring//ext, verbose-)
		}
	    }
	} else {
	    print("Darksubbed image list "//darklist//" already exists; images assumed to exist too.")
	}
print(" ") 
print("===> Dark-subtracted images are listed in the file "//darklist//".") 
}

# Reorient original images
if (oorient) {
	list = iimagelist 
	while (fscan (list, name) !=EOF) {
# Get channel (j or h or k or l)
  		imgets(name,"channel", >& "dev$null")
  		channel=imgets.value
# J ----------------------------------------------------------------------------
  		if (channel=="j") {
  		    imgets(name,"REORIENT", >& "dev$null")
		    if (imgets.value=="0") {
			imtranspose (name, name)
			hedit(name,"REORIENT","yes",add+,ver-,show+,update+)
		    } else {
		       print("REORIENT keyword exists; sqiidfirst already run?")
		    }
# H ----------------------------------------------------------------------------
  		} else if (channel=="h") {
  		    imgets(name,"REORIENT", >& "dev$null")
		    if (imgets.value=="0") {
			imcopy (name//"[-*,*]", name, ver-)
			hedit(name,"REORIENT","yes",add+,ver-,show+,update+)
		    } else {
		       print("REORIENT keyword exists; sqiidfirst already run?")
		    }
# K ----------------------------------------------------------------------------
  		} else if (channel=="k") {
  		    imgets(name,"REORIENT", >& "dev$null")
		    if (imgets.value=="0") {
			imcopy (name//"[*,-*]", name, ver-)
			hedit(name,"REORIENT","yes",add+,ver-,show+,update+)
		    } else {
		       print("REORIENT keyword exists; sqiidfirst already run?")
		    }
# L ----------------------------------------------------------------------------
  		} else if (channel=="l") {
  		    imgets(name,"REORIENT", >& "dev$null")
		    if (imgets.value=="0") {
			print ("WARNING! raw orientation of L unknown.")
#			hedit(name,"REORIENT","yes",add+,ver-,show+,update+)
		    } else {
		       print("REORIENT keyword exists; sqiidfirst already run?")
		    }
		}
	}
}

if(echog)beep

end

