# tpl.cl -- create throughput correction image

# 950123: created
# 950201: fixed so it would work and normalize correctly.
# 950214: fitting loop, tplpars
# 950224: added plot of object flux as f(time)
# 950523: fixed various minor bugs
# 990317: fixed to work with V2.11 imexam format

procedure tplnew(rootname,imagelist,outname)

string rootname   {prompt="Rootname for output log files, etc."}
string imagelist  {prompt="List of images to use to create throughput image"}
string outname    {prompt="Name of throughput image to create"}

real   ncols      {256.,prompt="Number of columns in output image"}
real   nlines     {256.,prompt="Number of columns in output image"}
real   radius=10. {prompt="Aperture photometry radius for imexamine"}
real   buffer=5.  {prompt="Aperture photometry sky buffer for imexamine"}
real   width=10.  {prompt="Aperture photometry sky annulus width for imexamine"}
bool   fit        {prompt="Adjust parameters and refit image?", mode="q"}
int    z          {prompt="Number of image to save?", mode="q"}
bool   echog        {yes,prompt="Beep when task is done?"}
pset   tplpars    {prompt="Utilities.Surfit parameters"}

begin

string logimex, logsurf, countfile, rrootname, iimagelist, ooutname, ffunc
string wweighting
real max, nncols, nnlines
int x, y, zz, xxord, yyord
bool go, ffit, xxterms

rrootname=rootname
iimagelist=imagelist
ooutname=outname

nncols=ncols
nnlines=nlines

ffunc=tplpars.func
xxord=tplpars.xord
yyord=tplpars.yord
xxterms=tplpars.xterms
wweighting=tplpars.weighting

logimex="log."//rrootname//".tplimex"
logsurf="log."//rrootname//".tplsurf"
countfile="log."//rrootname//".tplcount"

# check for addcomment and utilities.surfit; if not defined abort task.
go=yes
if(!deftask("addcomment")) {go=no}
if(!deftask("surfit")) {go=no}
if(go) {

# examine tpl images and find std. star flux in each
print (" ")
print ("===> Find std star flux with 'a' in each image.")
print ("===> When done, or if image is bad, hit 'n' to go to next image.")
print ("===> After last image, or if you've already done this stage, hit 'q'.")
rimexam.radius=radius
rimexam.buffer=buffer
rimexam.width=width
rimexam.center=yes
rimexam.background=yes
rimexam.xorder=1
rimexam.yorder=1
print("#TPL photometry: radius="//radius//", buffer="//buffer//", width="//width, >> logimex)
imexam ("@"//iimagelist, keeplog+, logfile=logimex)

del("_tmp.countfile",ver-, >& "dev$null")
print("#1:x   2:y    3:flux    4:err", >> countfile)
#--OLD IMEXAM FORMAT (x,y,flux,sky,npix)
#fields (logimex, "1,2,4,5,6", >> "_tmp.countfile")
#filecalc ("_tmp.countfile","$1;$2;$3;((sqrt($3+$4*$5))/$3)",format="%7.2f%7.2f%9.1f%9.5f", >> countfile)
#--NEW IMEXAM FORMAT (x,y,flux,sky,radius)
fields (logimex, "1,2,7,8,6", >> "_tmp.countfile")
filecalc ("_tmp.countfile","$1;$2;$3;((sqrt($3+$4*3.14159*$5**2.))/$3)",format="%7.2f%7.2f%9.1f%9.5f", >> countfile)
del("_tmp.countfile",ver-)

x=1
y=0
ffit=yes
# Fitting loop
while(ffit) {
	if(x!=1){
		copy(countfile//"_"//str(x-1),countfile//"_"//str(x))
		vi(countfile//"_"//str(x))
		tplpars
		ffunc=tplpars.func
		xxord=tplpars.xord
		yyord=tplpars.yord
		xxterms=tplpars.xterms
		wweighting=tplpars.weighting
	}
	else {
		copy(countfile,countfile//"_"//str(x))
	}

# fit this output (x,y,counts) file using utilities.surfit:
	print (" ")
	print ("===> Fitting surface to data points...")
	surfit (countfile//"_"//str(x),image=ooutname//"_"//str(x),ncols=nncols,nlines=nnlines,xord=xxord,yord=yyord,xterms=xxterms,weight=wweighting,func=ffunc, >> logsurf)
	imstat (ooutname//"_"//str(x), fields="max", format-) | scan (max)
	imar (ooutname//"_"//str(x), "/", max, ooutname//"_"//str(x), ver-)

# inspect the fitted images & decide on best order/funct.
	print (" ")
print ("===> Displaying surface-fit image and plotting flux time series...")
	y=y+1
	if(y>4)y=1
	displ (ooutname//"_"//str(x), y)
	fields (countfile//"_"//str(x), "3") | graph (point-)

# clean up
	del(logimex//"sort",ver-, >& "dev$null")

# abort loop?
	ffit=fit
	x+=1
}

# clean up
zz=z
if(zz==0){
	del(countfile,ver-, >& "dev$null")
	print("No throughput image saved.")
}
else {
	imcopy(ooutname//"_"//str(zz),ooutname,ver-) 
	del(countfile,ver-, >& "dev$null")
	rename(countfile//"_"//str(zz),countfile, >& "dev$null") 
	hedit(ooutname,"title","Throughput correction image from "//iimagelist//" and "//countfile,add+,ver-,update+,show-) 
addcomment(ooutname,"Image created from "//iimagelist//" and "//countfile,ver-) 
addcomment(ooutname,"using func="//ffunc//", xord="//xxord//", yord="//yyord//",",ver-) 
addcomment(ooutname,"xterms="//xxterms//", and weighting="//wweighting,ver-) 
}

y=0
while(y<x){
	imdel(ooutname//"_"//str(y),ver-, >& "dev$null")
	del(countfile//"_"//str(y),ver-, >& "dev$null")
	y+=1
}

}
else{
print("The tasks `utilities.surfit' and/or `addcomment' are not loaded,")
print("and this task needs them to run.  Please load it and start again.")
}

if(echog)beep

end
